@@ -7,12 +7,51 @@
 #include "main.h"
 #include "audio.h"
 #include "events.h"
+#include "callbacks.h"
 #include "effects-weed.h"
 #include "support.h"
 
 #include "../libweed/weed.h"
-#include "../libweed/weed-effects.h"
 #include "../libweed/weed-host.h"
+#include "../libweed/weed-palettes.h"
+
+
+
+// keep first 16 audio_in filesysten handles open
+#define NSTOREDFDS 16
+static gchar *storedfnames[NSTOREDFDS];
+static int storedfds[NSTOREDFDS];
+static gboolean storedfdsset=FALSE;
+
+static void audio_reset_stored_fnames(void) {
+  int i;
+  for (i=0;i<NSTOREDFDS;i++) {
+    storedfnames[i]=NULL;
+    storedfds[i]=-1;
+  }
+  storedfdsset=TRUE;
+}
+
+
+
+void audio_free_fnames(void) {
+  // cleanup stored filehandles after playback/fade/render
+  
+  int i;
+
+  if (!storedfdsset) return;
+
+  for (i=0;i<NSTOREDFDS;i++) {
+    if (storedfnames!=NULL) {
+      g_free(storedfnames[i]);
+      storedfnames[i]=NULL;
+      if (storedfds[i]>-1) close(storedfds[i]);
+      storedfds[i]=-1;
+    }
+  }
+}
+
+
 
 
 inline void sample_silence_dS (float *dst, unsigned long nsamples) {
@@ -375,11 +414,12 @@ long sample_move_abuf_float (float **obuf, int nchans, int nsamps, int out_arate
       // process each sample
 
       if ((curval=ioffs+src_offset_i)>=abuf->samples_filled) {
-	// current buffer is full
+	// current buffer is consumed
 	break;
       }
       xchan=0;
       for (j=0;j<nchans;j++) {
+	// copy channel by channel (de-interleave)
 	pthread_mutex_lock(&mainw->abuf_mutex);
 	if (mainw->jackd->read_abuf<0) {
 	  pthread_mutex_unlock(&mainw->abuf_mutex);
@@ -390,6 +430,7 @@ long sample_move_abuf_float (float **obuf, int nchans, int nsamps, int out_arate
 	pthread_mutex_unlock(&mainw->abuf_mutex);
 	xchan++;
       }
+      // resample on the fly
       src_offset_i=(int)(src_offset_f+=scale);
       samps++;
       samples_out++;
@@ -400,7 +441,7 @@ long sample_move_abuf_float (float **obuf, int nchans, int nsamps, int out_arate
     offs+=samps;
 
     if (nsamps>0) {
-      // buffer was filled, move on to next buffer
+      // buffer was consumed, move on to next buffer
 
       pthread_mutex_lock(&mainw->abuf_mutex);
       // request main thread to fill another buffer
@@ -709,6 +750,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 
   if (out_achans==0) return 0l;
 
+  if (!storedfdsset) audio_reset_stored_fnames();
 
   if (to_file>-1) {
     // prepare outfile stuff
@@ -769,21 +811,38 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
     in_unsigned[track]=infile->signed_endian&AFORM_UNSIGNED;
     in_bendian=infile->signed_endian&AFORM_BIG_ENDIAN;
 
-    if ((!in_bendian&&(G_BYTE_ORDER==G_BIG_ENDIAN))||(in_bendian&&(G_BYTE_ORDER==G_LITTLE_ENDIAN))) in_reverse_endian[track]=TRUE;
-    else in_reverse_endian[track]=FALSE;
+    if (G_UNLIKELY(in_achans[track]==0)) is_silent[track]=TRUE;
+    else {
+      if ((!in_bendian&&(G_BYTE_ORDER==G_BIG_ENDIAN))||(in_bendian&&(G_BYTE_ORDER==G_LITTLE_ENDIAN))) in_reverse_endian[track]=TRUE;
+      else in_reverse_endian[track]=FALSE;
+      
+      seekstart[track]=(off64_t)(fromtime[track]*in_arate[track])*in_achans[track]*in_asamps[track];
+      seekstart[track]=((off64_t)(seekstart[track]/in_achans[track]/(in_asamps[track])))*in_achans[track]*in_asamps[track];
+      
+      zavel=avels[track]*(gdouble)in_arate[track]/(gdouble)out_arate*in_asamps[track]*in_achans[track]/sizeof(float);
+      if (ABS(zavel)>zavel_max) zavel_max=ABS(zavel);
+      
+      infilename=g_strdup_printf("%s/%s/audio",prefs->tmpdir,infile->handle);
 
-    seekstart[track]=(off64_t)(fromtime[track]*in_arate[track])*in_achans[track]*in_asamps[track];
-    seekstart[track]=((off64_t)(seekstart[track]/in_achans[track]/(in_asamps[track])))*in_achans[track]*in_asamps[track];
-    
-    zavel=avels[track]*(gdouble)in_arate[track]/(gdouble)out_arate*in_asamps[track]*in_achans[track]/sizeof(float);
-    if (ABS(zavel)>zavel_max) zavel_max=ABS(zavel);
 
-    infilename=g_strdup_printf("%s/%s/audio",prefs->tmpdir,infile->handle);
-    in_fd[track]=open(infilename,O_RDONLY);
-    
-    lseek64(in_fd[track],seekstart[track],SEEK_SET);
+      // try to speed up access by keeping some files open
+      if (track<NSTOREDFDS&&storedfnames[track]!=NULL&&!strcmp(infilename,storedfnames[track])) {
+	in_fd[track]=storedfds[track];
+      }
+      else {
+	if (track<NSTOREDFDS&&storedfds[track]>-1) close(storedfds[track]);
+	in_fd[track]=open(infilename,O_RDONLY);
+	
+	if (track<NSTOREDFDS) {
+	  storedfds[track]=in_fd[track];
+	  storedfnames[track]=g_strdup(infilename);
+	}
+      }
 
-    g_free(infilename);
+      lseek64(in_fd[track],seekstart[track],SEEK_SET);
+      
+      g_free(infilename);
+    }
   }
 
   for (track=0;track<nfiles;track++) {
@@ -899,7 +958,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 	}
       }
 
-      nframes=tbytes/(in_asamps[track])/in_achans[track]/ABS(zavel);
+      nframes=(tbytes/(in_asamps[track])/in_achans[track]/ABS(zavel)+.001);
 
       // convert to float
       if (in_asamps[track]==1) {
@@ -917,7 +976,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
     }
 
     // now we send small chunks at a time to the audio vol/pan effect
-    
+    shortcut=NULL;
     blocksize=RENDER_BLOCK_SIZE;
 
     for (i=0;i<xsamples;i+=RENDER_BLOCK_SIZE) {
@@ -982,7 +1041,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
   // close files
   for (track=0;track<nfiles;track++) {
     if (!is_silent[track]) {
-      close (in_fd[track]);
+      if (track>=NSTOREDFDS) close (in_fd[track]);
     }
     for (c=0;c<out_achans;c++) {
       g_free(chunk_float_buffer[track*out_achans+c]);
@@ -1008,7 +1067,7 @@ inline void aud_fade(gint fileno, gdouble startt, gdouble endt, gdouble startv,
 
 
 #ifdef ENABLE_JACK
-void jack_rec_audio_to_clip(gint fileno, gboolean is_window_grab) {
+void jack_rec_audio_to_clip(gint fileno, gint old_file, gshort rec_type) {
   // open audio file for writing
   file *outfile=mainw->files[fileno];
   gchar *outfilename=g_strdup_printf("%s/%s/audio",prefs->tmpdir,outfile->handle);
@@ -1029,13 +1088,23 @@ void jack_rec_audio_to_clip(gint fileno, gboolean is_window_grab) {
   jack_read_driver_activate(mainw->jackd_read);
 
   // in grab window mode, just return, we will call rec_audio_end on playback end
-  if (is_window_grab) return;
+  if (rec_type==RECA_WINDOW_GRAB) return;
 
   mainw->cancelled=CANCEL_NONE;
   mainw->cancel_type=CANCEL_SOFT;
   // show countdown/stop dialog
+  mainw->suppress_dprint=FALSE;
   d_print(_("Recording audio..."));
-  do_auto_dialog(_("Recording audio"),1);
+  mainw->suppress_dprint=TRUE;
+  if (rec_type==RECA_NEW_CLIP) {
+    do_auto_dialog(_("Recording audio"),1);
+  }
+  else {
+    gint current_file=mainw->current_file;
+    mainw->current_file=old_file;
+    on_playsel_activate(NULL,NULL);
+    mainw->current_file=current_file;
+  }
   jack_rec_audio_end();
 }
 
@@ -1050,6 +1119,7 @@ void jack_rec_audio_end(void) {
 
   // close file
   close(mainw->aud_rec_fd);
+  mainw->aud_rec_fd=-1;
   mainw->cancel_type=CANCEL_KILL;
 }
 
@@ -1059,7 +1129,7 @@ void jack_rec_audio_end(void) {
 
 
 #ifdef HAVE_PULSE_AUDIO
-void pulse_rec_audio_to_clip(gint fileno, gboolean is_window_grab) {
+void pulse_rec_audio_to_clip(gint fileno, gint old_file, gshort rec_type) {
   // open audio file for writing
   file *outfile=mainw->files[fileno];
   gchar *outfilename=g_strdup_printf("%s/%s/audio",prefs->tmpdir,outfile->handle);
@@ -1079,13 +1149,21 @@ void pulse_rec_audio_to_clip(gint fileno, gboolean is_window_grab) {
   pulse_driver_activate(mainw->pulsed_read);
 
   // in grab window mode, just return, we will call rec_audio_end on playback end
-  if (is_window_grab) return;
+  if (rec_type==RECA_WINDOW_GRAB) return;
 
   mainw->cancelled=CANCEL_NONE;
   mainw->cancel_type=CANCEL_SOFT;
   // show countdown/stop dialog
+  mainw->suppress_dprint=FALSE;
   d_print(_("Recording audio..."));
-  do_auto_dialog(_("Recording audio"),1);
+  mainw->suppress_dprint=TRUE;
+  if (rec_type==RECA_NEW_CLIP) do_auto_dialog(_("Recording audio"),1);
+  else {
+    gint current_file=mainw->current_file;
+    mainw->current_file=old_file;
+    on_playsel_activate(NULL,NULL);
+    mainw->current_file=current_file;
+  }
   pulse_rec_audio_end();
 }
 
@@ -1095,14 +1173,17 @@ void pulse_rec_audio_end(void) {
   // recording ended
 
   // stop recording
-  pulse_close_client(mainw->pulsed_read);
 
+  pa_threaded_mainloop_lock(mainw->pulsed->mloop);
   pulse_flush_read_data(mainw->pulsed_read,0,NULL);
+  pulse_close_client(mainw->pulsed_read);
+  pa_threaded_mainloop_unlock(mainw->pulsed->mloop);
 
   mainw->pulsed_read=NULL;
 
   // close file
   close(mainw->aud_rec_fd);
+  mainw->aud_rec_fd=-1;
   mainw->cancel_type=CANCEL_KILL;
 }
 
@@ -1421,7 +1502,7 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
 
   last_tc=fill_tc;
 
-  if (mainw->abufs_to_fill) {
+  if (mainw->abufs_to_fill>0) {
     pthread_mutex_lock(&mainw->abuf_mutex);
     mainw->abufs_to_fill--;
     pthread_mutex_unlock(&mainw->abuf_mutex);
@@ -1444,10 +1525,10 @@ void init_jack_audio_buffers (gint achans, gint arate, gboolean exact) {
     
     mainw->jackd->abufs[i]->achans=achans;
     mainw->jackd->abufs[i]->arate=arate;
-    mainw->jackd->abufs[i]->sample_space=XSAMPLES;
+    mainw->jackd->abufs[i]->sample_space=XSAMPLES/prefs->num_rtaudiobufs;
     mainw->jackd->abufs[i]->data.floatbuf=g_malloc(achans*sizeof(float *));
     for (chan=0;chan<achans;chan++) {
-      mainw->jackd->abufs[i]->data.floatbuf[chan]=g_malloc(XSAMPLES*sizeof(float));
+      mainw->jackd->abufs[i]->data.floatbuf[chan]=g_malloc(XSAMPLES/prefs->num_rtaudiobufs*sizeof(float));
     }
   }
 #endif
@@ -1466,8 +1547,8 @@ void init_pulse_audio_buffers (gint achans, gint arate, gboolean exact) {
     
     mainw->pulsed->abufs[i]->achans=achans;
     mainw->pulsed->abufs[i]->arate=arate;
-    mainw->pulsed->abufs[i]->sample_space=XSAMPLES;  // sample_space here is in stereo samples
-    mainw->pulsed->abufs[i]->data.int16buf=g_malloc(XSAMPLES*achans*sizeof(short));
+    mainw->pulsed->abufs[i]->sample_space=XSAMPLES/prefs->num_rtaudiobufs;  // sample_space here is in stereo samples
+    mainw->pulsed->abufs[i]->data.int16buf=g_malloc(XSAMPLES/prefs->num_rtaudiobufs*achans*sizeof(short));
   }
 #endif
 }
@@ -1534,21 +1615,25 @@ gboolean resync_audio(gint frameno) {
   if (!(prefs->audio_opts&AUDIO_OPTS_FOLLOW_FPS)) return FALSE;
 
 #ifdef ENABLE_JACK
-  if (prefs->audio_player==AUD_PLAYER_JACK&&mainw->jackd!=NULL&&!mainw->is_rendering) {
-    jack_audio_seek_frame(mainw->jackd,frameno);
-    mainw->rec_aclip=mainw->current_file;
-    mainw->rec_avel=cfile->pb_fps/cfile->fps;
-    mainw->rec_aseek=(gdouble)mainw->jackd->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+  if (prefs->audio_player==AUD_PLAYER_JACK&&mainw->jackd!=NULL) {
+    if (!mainw->is_rendering) {
+      jack_audio_seek_frame(mainw->jackd,frameno);
+      mainw->rec_aclip=mainw->current_file;
+      mainw->rec_avel=cfile->pb_fps/cfile->fps;
+      mainw->rec_aseek=(gdouble)mainw->jackd->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+    }
     return TRUE;
   }
 #endif
 
 #ifdef HAVE_PULSE_AUDIO
-  if (prefs->audio_player==AUD_PLAYER_PULSE&&mainw->pulsed!=NULL&&!mainw->is_rendering) {
-    pulse_audio_seek_frame(mainw->pulsed,frameno);
-    mainw->rec_aclip=mainw->current_file;
-    mainw->rec_avel=cfile->pb_fps/cfile->fps;
-    mainw->rec_aseek=(gdouble)mainw->pulsed->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+  if (prefs->audio_player==AUD_PLAYER_PULSE&&mainw->pulsed!=NULL) {
+    if (!mainw->is_rendering) {
+      pulse_audio_seek_frame(mainw->pulsed,frameno);
+      mainw->rec_aclip=mainw->current_file;
+      mainw->rec_avel=cfile->pb_fps/cfile->fps;
+      mainw->rec_aseek=(gdouble)mainw->pulsed->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+    }
     return TRUE;
   }
 #endif
