@@ -57,7 +57,7 @@
 typedef struct _aserver_message_t {
   gint command;
   gchar *data;
-  struct _aserver_message_t *next;
+  volatile struct _aserver_message_t *next;
 } aserver_message_t;
 
 
@@ -100,13 +100,19 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 
 void aud_fade(gint fileno, gdouble startt, gdouble endt, gdouble startv, gdouble endv); // fade in/fade out
 
+
+#define RECA_WINDOW_GRAB 1
+#define RECA_NEW_CLIP 2
+#define RECA_EXISTING 3
+
+
 #ifdef ENABLE_JACK
-void jack_rec_audio_to_clip(gint fileno, gboolean is_window_grab);  // record from external source to clip
+void jack_rec_audio_to_clip(gint fileno, gint oldfileno, gshort rec_type);  // record from external source to clip
 void jack_rec_audio_end(void);
 #endif
 
 #ifdef HAVE_PULSE_AUDIO
-void pulse_rec_audio_to_clip(gint fileno, gboolean is_window_grab);  // record from external source to clip
+void pulse_rec_audio_to_clip(gint fileno, gint oldfileno, gshort rec_type);  // record from external source to clip
 void pulse_rec_audio_end(void);
 #endif
 
@@ -125,4 +131,7 @@ void free_jack_audio_buffers(void);
 void init_pulse_audio_buffers (gint achans, gint arate, gboolean exact);
 void free_pulse_audio_buffers(void);
 
+void audio_free_fnames(void);
+
+
 #endif
