@@ -1,6 +1,6 @@
 // callbacks.h
 // LiVES
-// (c) G. Finch <salsaman@xs4all.nl> 2003 - 2009
+// (c) G. Finch <salsaman@xs4all.nl> 2003 - 2010
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -32,10 +32,6 @@ void
 on_stop_clicked                         (GtkMenuItem     *menuitem,
                                          gpointer         user_data);
 
-void
-on_save_activate                      (GtkMenuItem     *menuitem,
-				       gpointer         user_data);
-
 void
 on_save_selection_activate            (GtkMenuItem     *menuitem,
 				       gpointer         user_data);
@@ -144,10 +140,6 @@ void
 on_rev_clipboard_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void 
-on_audp_entry_changed (GtkWidget *vidp_entry, gpointer);
-
-
 void 
 on_encoder_entry_changed                 (GtkEntry *encoder_entry, 
 					  gpointer ptr);
@@ -230,6 +222,9 @@ void
 on_open_sel_ok_button_clicked                  (GtkButton       *button,
                                         gpointer         user_data);
 
+
+void on_save_textview_clicked (GtkButton *button, gpointer user_data);
+
 void
 on_cancel_button1_clicked              (GtkButton       *button,
                                         gpointer         user_data);
@@ -304,6 +299,10 @@ void
 on_info_ok_button_clicked2            (GtkButton       *button,
 				      gpointer         user_data);
 
+void
+on_details_button_clicked            (GtkButton       *button,
+				      gpointer         user_data);
+
 
 void
 on_resize_activate                    (GtkMenuItem     *menuitem,
@@ -521,7 +520,6 @@ on_record_perf_activate                      (GtkMenuItem     *menuitem,
 
 gboolean record_toggle_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data);
 
-gboolean dirchange_callback (GtkAccelGroup *, GObject *, guint, GdkModifierType, gpointer user_data); // defined as extern in dialogs.c
 
 gboolean fps_reset_callback (GtkAccelGroup *, GObject *, guint, GdkModifierType, gpointer user_data);
 
@@ -785,4 +783,3 @@ void on_lerrors_close_clicked (GtkButton *, gpointer);
 void on_lerrors_clear_clicked (GtkButton *, gpointer);
 void on_lerrors_delete_clicked (GtkButton *, gpointer);
 
-void on_init_aplayer_toggled (GtkToggleButton *, gpointer new_player);
