@@ -139,8 +139,9 @@ gboolean check_clip_integrity(file *sfile, const lives_clip_data_t *cdata) {
 
 
 
-gboolean check_if_non_virtual(file *sfile) {
+gboolean check_if_non_virtual(gint fileno) {
   register int i;
+  file *sfile=mainw->files[fileno];
 
   if (sfile->frame_index!=NULL) {
     for (i=1;i<=sfile->frames;i++) {
@@ -151,16 +152,23 @@ gboolean check_if_non_virtual(file *sfile) {
   sfile->clip_type=CLIP_TYPE_DISK;
   del_frame_index(sfile);
 
+  if (sfile->interlace!=LIVES_INTERLACE_NONE) {
+    sfile->interlace=LIVES_INTERLACE_NONE; // all frames should have been deinterlaced
+    save_clip_value(fileno,CLIP_DETAILS_INTERLACE,&sfile->interlace);
+  }
+
   return TRUE;
 }
 
 
 
-void virtual_to_images(gint sfileno, gint sframe, gint eframe) {
+void virtual_to_images(gint sfileno, gint sframe, gint eframe, gboolean update_progress) {
   // pull frames from a clip to images
   // from sframe to eframe inclusive (first frame is 1)
 
-  // should be threadsafe
+  // if update_progress, set mainw->msg with number of frames pulled
+
+  // should be threadsafe apart from progress update
 
   register int i;
   file *sfile=mainw->files[sfileno];
@@ -168,6 +176,8 @@ void virtual_to_images(gint sfileno, gint sframe, gint eframe) {
   GError *error=NULL;
   gchar *oname;
 
+  gint progress=1;
+
   if (sframe<1) sframe=1;
 
   for (i=sframe;i<=eframe;i++) {
@@ -207,7 +217,15 @@ void virtual_to_images(gint sfileno, gint sframe, gint eframe) {
 
       if (pixbuf!=NULL) gdk_pixbuf_unref(pixbuf);
       pixbuf=NULL;
+
+      // another thread may have called check_if_non_virtual
+      if (sfile->frame_index==NULL) break;
       sfile->frame_index[i-1]=-1;
+
+      if (update_progress) {
+	// sig_progress...
+	g_snprintf (mainw->msg,256,"%d",progress++);
+      }
     }
     pthread_mutex_unlock(&mainw->gtk_mutex);
 
@@ -216,12 +234,12 @@ void virtual_to_images(gint sfileno, gint sframe, gint eframe) {
     sync();
 
     if (mainw->cancelled!=CANCEL_NONE) {
-      if (!check_if_non_virtual(sfile)) save_frame_index(sfileno);
+      if (!check_if_non_virtual(sfileno)) save_frame_index(sfileno);
       return;
     }
   }
 
-  if (!check_if_non_virtual(sfile)) save_frame_index(sfileno);
+  if (!check_if_non_virtual(sfileno)) save_frame_index(sfileno);
 }
 
 
