@@ -10,6 +10,7 @@
 #include <stdlib.h>
 
 #include "../libweed/weed.h"
+#include "../libweed/weed-palettes.h"
 #include "../libweed/weed-effects.h"
 #include "../libweed/weed-host.h"
 
@@ -280,11 +281,13 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
       cfile->vsize=weed_get_int_value(first_out,"height",&error);
     }
 
-    if (cfile->hsize==cfile->ohsize&&cfile->vsize==cfile->ovsize) cfile->undo_action=UNDO_EFFECT;
-    else {
-      save_clip_value(mainw->current_file,CLIP_DETAILS_WIDTH,&cfile->hsize);
-      save_clip_value(mainw->current_file,CLIP_DETAILS_HEIGHT,&cfile->vsize);
-      cfile->undo_action=UNDO_RESIZABLE;
+    if (rfx->num_in_channels>0) {
+      if (cfile->hsize==cfile->ohsize&&cfile->vsize==cfile->ovsize) cfile->undo_action=UNDO_EFFECT;
+      else {
+	save_clip_value(mainw->current_file,CLIP_DETAILS_WIDTH,&cfile->hsize);
+	save_clip_value(mainw->current_file,CLIP_DETAILS_HEIGHT,&cfile->vsize);
+	cfile->undo_action=UNDO_RESIZABLE;
+      }
     }
   }
 
@@ -390,6 +393,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 	cfile->is_loaded=TRUE;
 	add_to_winmenu();
 	save_clip_values(new_file);
+	if (prefs->crash_recovery) add_to_recovery_file(cfile->handle);
 
 	if (mainw->multitrack!=NULL) {
 	  mt_init_clips(mainw->multitrack,mainw->current_file,TRUE);
@@ -502,7 +506,7 @@ gint realfx_progress (gboolean reset) {
     mainw->internal_messaging=FALSE;
 
     if (cfile->clip_type==CLIP_TYPE_FILE) {
-      if (!check_if_non_virtual(cfile)) save_frame_index(mainw->current_file);
+      if (!check_if_non_virtual(mainw->current_file)) save_frame_index(mainw->current_file);
     }
 
   }
@@ -782,7 +786,7 @@ gboolean rtemode_callback_hook (GtkToggleButton *button, gpointer user_data) {
 gboolean swap_fg_bg_callback (GtkAccelGroup *group, GObject *obj, guint keyval, GdkModifierType mod, gpointer user_data) {
   gint old_file=mainw->current_file;
 
-  if (mainw->playing_file<0||mainw->num_tr_applied==0||mainw->noswitch||mainw->blend_file==-1||mainw->blend_file==mainw->current_file||mainw->files[mainw->blend_file]==NULL||mainw->preview||mainw->noswitch) {
+  if (mainw->playing_file<1||mainw->num_tr_applied==0||mainw->noswitch||mainw->blend_file==-1||mainw->blend_file==mainw->current_file||mainw->files[mainw->blend_file]==NULL||mainw->preview||mainw->noswitch||(mainw->is_processing&&cfile->is_loaded)) {
     return TRUE;
   }
 
