@@ -16,18 +16,6 @@
 
 static jack_client_t *jack_transport_client;
 
-int lives_start_ready_callback (jack_transport_state_t state, jack_position_t *pos, void *arg) {
-  // we will return FALSE until we call gettimeofday for the origin
-
-  gboolean seek_ready=mainw->video_seek_ready;
-
-  // reset for next time
-  mainw->video_seek_ready=FALSE;
-
-  return seek_ready;
-}
-
-
 
 gboolean lives_jack_init (void) {
   gchar *jt_client=g_strdup_printf("LiVES-%d",getpid());
@@ -47,7 +35,7 @@ gboolean lives_jack_init (void) {
       get_location("jackd",jackd_loc,512);
       if (strlen(jackd_loc)) {
 #ifndef IS_DARWIN
-	com=g_strdup_printf("echo \"%s -Z -d alsa\">%s",jackd_loc,prefs->jack_aserver);
+	com=g_strdup_printf("echo \"%s -Z -r -d alsa\">%s",jackd_loc,prefs->jack_aserver);
 #else
 #ifdef IS_SOLARIS
 	// use OSS on Solaris
@@ -79,12 +67,12 @@ gboolean lives_jack_init (void) {
   if (jack_transport_client==NULL) return FALSE;
 
 #ifdef ENABLE_JACK_TRANSPORT
-  mainw->video_seek_ready=TRUE;
   jack_activate(jack_transport_client);
-  if (jack_transport_client!=NULL) jack_set_sync_callback (jack_transport_client, lives_start_ready_callback, NULL);
+  jack_set_sync_timeout(jack_transport_client,5000000); // seems to not work
+  jack_set_sync_callback (jack_transport_client, lives_start_ready_callback, NULL);
   gtk_timeout_add(KEY_RPT_INTERVAL,&lives_jack_poll,NULL);
 #else
-  if (jack_transport_client!=NULL) jack_client_close (jack_transport_client);
+  jack_client_close (jack_transport_client);
   jack_transport_client=NULL;
 #endif
 
@@ -119,11 +107,12 @@ gdouble jack_transport_get_time(void) {
 
 
 
-static void jack_transport_check_state (void) {
 #ifdef ENABLE_JACK_TRANSPORT
+static void jack_transport_check_state (void) {
   jack_position_t pos;
   jack_transport_state_t jacktstate;
 
+  // go away until the app has started up properly
   if (mainw->go_away) return;
 
   if (!(prefs->jack_opts&JACK_OPTS_TRANSPORT_CLIENT)) return;
@@ -137,15 +126,15 @@ static void jack_transport_check_state (void) {
     mainw->jack_can_stop=TRUE;
     on_playall_activate(NULL,NULL);
   }
-  else if (jacktstate==JackTransportRolling||jacktstate==JackTransportStarting) {
-    mainw->video_seek_ready=TRUE;
-  }
+
   if (jacktstate==JackTransportStopped) {
-     if (mainw->playing_file>-1&&mainw->jack_can_stop) on_stop_activate (NULL,NULL);
-     mainw->jack_can_start=TRUE;
+    if (mainw->playing_file>-1&&mainw->jack_can_stop) {
+      on_stop_activate (NULL,NULL);
+    }
+    mainw->jack_can_start=TRUE;
   }
-#endif
 }
+#endif
 
 gboolean lives_jack_poll(gpointer data) {
   // data is always NULL
@@ -157,7 +146,9 @@ gboolean lives_jack_poll(gpointer data) {
 }
 
 void lives_jack_end (void) {
+#ifdef ENABLE_JACK_TRANSPORT
   jack_client_t *client=jack_transport_client;
+#endif
   jack_transport_client=NULL; // stop polling transport
 #ifdef ENABLE_JACK_TRANSPORT
   if (client!=NULL) {
@@ -189,6 +180,8 @@ static jack_driver_t outdev[JACK_MAX_OUTDEVICES];
 static jack_driver_t indev[JACK_MAX_OUTDEVICES];
 
 
+/* not used yet */
+/*
 static float set_pulse(float *buf, size_t bufsz, int step) {
   float *ptr=buf;
   float *end=buf+bufsz;
@@ -203,7 +196,7 @@ static float set_pulse(float *buf, size_t bufsz, int step) {
   }
   if (count>0) return tot/(float)count;
   return 0.;
-}
+  }*/
 
 
 void jack_get_rec_avals(jack_driver_t *jackd) {
@@ -234,7 +227,7 @@ static int audio_process (nframes_t nframes, void *arg) {
   if (!mainw->is_ready||jackd==NULL||(mainw->playing_file==-1&&jackd->is_silent&&jackd->msgq==NULL)) return 0;
 
   /* process one message */
-  if((msg=(aserver_message_t *)jackd->msgq)!=NULL) {
+  while ((msg=(aserver_message_t *)jackd->msgq)!=NULL) {
 
     switch (msg->command) {
     case ASERVER_CMD_FILE_OPEN:
@@ -247,7 +240,7 @@ static int audio_process (nframes_t nframes, void *arg) {
 	  g_printerr("jack: error opening %s\n",filename);
 	  jackd->playing_file=-1;
 	}
-	if (jackd->fd>0) {
+	else {
 	  if (jackd->aPlayPtr->data!=NULL) g_free(jackd->aPlayPtr->data);
 	  if (jackd->buffer_size>0) jackd->aPlayPtr->data=g_malloc(jackd->buffer_size*100);
 	  else (jackd->aPlayPtr->data)=NULL;
@@ -259,23 +252,22 @@ static int audio_process (nframes_t nframes, void *arg) {
       }
       break;
     case ASERVER_CMD_FILE_CLOSE:
-      if (jackd->fd>0) close(jackd->fd);
-      jackd->fd=0;
+      if (jackd->fd>=0) close(jackd->fd);
+      jackd->fd=-1;
       if (jackd->aPlayPtr->data!=NULL) g_free(jackd->aPlayPtr->data);
       jackd->aPlayPtr->data=NULL;
       jackd->aPlayPtr->size=0;
       jackd->playing_file=-1;
       break;
     case ASERVER_CMD_FILE_SEEK:
-      if (jackd->fd<1) break;
+      if (jackd->fd<0) break;
       xseek=seek=atol(msg->data);
       if (seek<0.) xseek=0.;
       if (!jackd->mute) {
 	lseek(jackd->fd,xseek,SEEK_SET);
       }
       jackd->seek_pos=seek;
-      gettimeofday(&tv, NULL);
-      jackd->audio_ticks=U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*U_SEC_RATIO;
+      jackd->audio_ticks=mainw->currticks;
       jackd->frames_written=0;
       break;
     default:
@@ -283,10 +275,11 @@ static int audio_process (nframes_t nframes, void *arg) {
     }
     if (msg->data!=NULL) g_free(msg->data);
     msg->command=ASERVER_CMD_PROCESSED;
-    jackd->msgq = NULL;    /* take this message off of the queue */
+    if (msg->next==NULL) jackd->msgq=NULL; 
+    else jackd->msgq = msg->next;
+  }
 
-    return 0;
-  }  
+  if (nframes==0) return 0;
 
   if (nframes != jackd->chunk_size) jackd->chunk_size = nframes;
 
@@ -302,7 +295,7 @@ static int audio_process (nframes_t nframes, void *arg) {
   /* handle playing state */
   if (jackd->state==JackTransportRolling||jackd->play_when_stopped) {
     gulong jackFramesAvailable = nframes; /* frames we have left to write to jack */
-   gulong inputFramesAvailable;          /* frames we have available this loop */
+    gulong inputFramesAvailable;          /* frames we have available this loop */
     gulong numFramesToWrite;              /* num frames we are writing this loop */
     glong in_frames=0;
     gulong in_bytes=0;
@@ -318,7 +311,7 @@ static int audio_process (nframes_t nframes, void *arg) {
 
     jackd->num_calls++;
 
-    if (!jackd->in_use||((jackd->fd<1||jackd->seek_pos<0.)&&jackd->read_abuf<0)||jackd->is_paused) {
+    if (!jackd->in_use||((jackd->fd<0||jackd->seek_pos<0.)&&jackd->read_abuf<0)||jackd->is_paused) {
       /* output silence if nothing is being outputted */
       if (!jackd->is_silent) for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
       jackd->is_silent=TRUE;
@@ -354,7 +347,7 @@ static int audio_process (nframes_t nframes, void *arg) {
 
       }
       else {
-	if (G_LIKELY(jackd->fd>0)) {
+	if (G_LIKELY(jackd->fd>=0)) {
 	  jackd->aPlayPtr->size=0;
 	  in_bytes=ABS((in_frames=((gdouble)jackd->sample_in_rate/(gdouble)jackd->sample_out_rate*(gdouble)jackFramesAvailable+((gdouble)fastrand()/(gdouble)G_MAXUINT32))))*jackd->num_input_channels*jackd->bytes_per_channel;
 	  if ((shrink_factor=(gfloat)in_frames/(gfloat)jackFramesAvailable)<0.f) {
@@ -475,11 +468,11 @@ static int audio_process (nframes_t nframes, void *arg) {
 #endif
 	buffer = jackd->aPlayPtr->data;
 	
-	numFramesToWrite = MIN(jackFramesAvailable, inputFramesAvailable/ABS(shrink_factor)); /* write as many bytes as we have space remaining, or as much as we have data to write */
+	numFramesToWrite = MIN(jackFramesAvailable, (inputFramesAvailable/ABS(shrink_factor)+.001)); /* write as many bytes as we have space remaining, or as much as we have data to write */
 	
 
 #ifdef DEBUG_AJACK
-	g_printerr("inputFramesAvailable after conversion %ld\n", (gulong)((gdouble)inputFramesAvailable/shrink_factor));
+	g_printerr("inputFramesAvailable after conversion %ld\n", (gulong)((gdouble)inputFramesAvailable/shrink_factor+.001));
 	g_printerr("nframes == %d, jackFramesAvailable == %ld,\n\tjackd->num_input_channels == %ld, jackd->num_output_channels == %ld, nf2w %ld, in_bytes %d, sf %.8f\n",  nframes, jackFramesAvailable, jackd->num_input_channels, jackd->num_output_channels, numFramesToWrite, in_bytes, shrink_factor);
 #endif
 	
@@ -564,9 +557,54 @@ static int audio_process (nframes_t nframes, void *arg) {
 
 
 
+int lives_start_ready_callback (jack_transport_state_t state, jack_position_t *pos, void *arg) {
+  gboolean seek_ready;
+
+  // mainw->video_seek_ready is generally FALSE
+  // if we are not playing, the transport poll should start playing which will set set 
+  // mainw->video_seek_ready to true, as soon as the audio seeks to the right place
+
+  // if we are playing, we set mainw->scratch
+  // this will either force a resync of audio in free playback
+  // or reset the event_list position in multitrack playback
+
+  // go away until the app has started up properly
+  if (mainw->go_away) {
+    if (state==JackTransportStopped) mainw->jack_can_start=TRUE;
+    else mainw->jack_can_start=mainw->jack_can_stop=FALSE;
+    return TRUE;
+  }
+
+  if (!(prefs->jack_opts&JACK_OPTS_TRANSPORT_CLIENT)) return TRUE;
+
+  if (jack_transport_client==NULL) return TRUE;
+
+  if (state!=JackTransportStarting) return TRUE;
+
+  // work around a bug in jack
+  //seek_ready=mainw->video_seek_ready;
+  seek_ready=TRUE;
+
+  if (mainw->playing_file!=-1&&prefs->jack_opts&JACK_OPTS_TIMEBASE_CLIENT) {
+    // trigger audio resync
+    if (mainw->scratch==SCRATCH_NONE) mainw->scratch=SCRATCH_JUMP;
+  }
+
+  // reset for next seek
+  if (seek_ready) mainw->video_seek_ready=FALSE;
+
+  return seek_ready;
+
+}
+
+
+
 
 static int audio_read (nframes_t nframes, void *arg) {
   // read nframes from jack buffer, and then write to mainw->aud_rec_fd
+
+  // this is the jack callback for when we are recording audio
+
   jack_driver_t* jackd = (jack_driver_t*)arg;
   void *holding_buff;
   float *in_buffer[jackd->num_input_channels];
@@ -577,7 +615,9 @@ static int audio_read (nframes_t nframes, void *arg) {
 
   if (mainw->effects_paused) return 0; // pause during record
 
-  frames_out=(long)((gdouble)nframes/out_scale+.5);
+  if (mainw->rec_samples==0) return 0; // wrote enough already, return until main thread stop
+
+  frames_out=(long)((gdouble)nframes/out_scale+1.);
 
   holding_buff=malloc(frames_out*afile->achans*afile->asampsize/8);
 
@@ -628,7 +668,7 @@ static int jack_get_bufsize (nframes_t nframes, void *arg) {
   
   if (jackd->is_output) {
     if (jackd->aPlayPtr->data!=NULL) g_free(jackd->aPlayPtr->data);
-    if (jackd->fd>0) jackd->aPlayPtr->data=g_malloc(jackd->buffer_size*100);
+    if (jackd->fd>=0) jackd->aPlayPtr->data=g_malloc(jackd->buffer_size*100);
   }
 
   return 0;
@@ -1104,6 +1144,8 @@ int jack_read_driver_activate (jack_driver_t *jackd) {
 
   jackd->is_paused=FALSE;
 
+  jackd->audio_ticks=0;
+
   d_print(_("Started jack audio reader.\n"));
 
   return 0;
@@ -1159,7 +1201,7 @@ int jack_audio_init(void) {
     for (j=0;j<JACK_MAX_OUTPUT_PORTS;j++) jackd->volume[j]=1.0f;
     jackd->state=JackTClosed;
     jackd->sample_out_rate=jackd->sample_in_rate=0;
-    jackd->fd=0;
+    jackd->fd=-1;
     jackd->seek_pos=jackd->seek_end=0;
     jackd->msgq=NULL;
     jackd->num_calls=0;
@@ -1194,7 +1236,7 @@ int jack_audio_read_init(void) {
     for (j=0;j<JACK_MAX_INPUT_PORTS;j++) jackd->volume[j]=1.0f;
     jackd->state=JackTClosed;
     jackd->sample_out_rate=jackd->sample_in_rate=0;
-    jackd->fd=0;
+    jackd->fd=-1;
     jackd->seek_pos=jackd->seek_end=0;
     jackd->msgq=NULL;
     jackd->num_calls=0;
@@ -1226,6 +1268,7 @@ gint64 lives_jack_get_time(jack_driver_t *jackd, gboolean absolute) {
   gdouble frames_written=jackd->frames_written;
   if (frames_written<0.) frames_written=0.;
   if (msg!=NULL&&msg->command==ASERVER_CMD_FILE_SEEK) while (jack_get_msgq(jackd)!=NULL); // wait for seek
+
   if (jackd->is_output) return jackd->audio_ticks*absolute+(gint64)(frames_written/(gdouble)jackd->sample_out_rate*U_SEC);
   return jackd->audio_ticks*absolute+(gint64)(frames_written/(gdouble)jackd->sample_in_rate*U_SEC);
 }
@@ -1238,8 +1281,6 @@ gdouble lives_jack_get_pos(jack_driver_t *jackd) {
 
 
 
-static aserver_message_t jack_message;
-
 void jack_audio_seek_frame (jack_driver_t *jackd, gint frame) {
   // seek to frame "frame" in current audio file
   // position will be adjusted to (floor) nearest sample
@@ -1253,7 +1294,7 @@ void jack_audio_seek_frame (jack_driver_t *jackd, gint frame) {
   if (jackd->playing_file==-1) return;
   if (frame>afile->frames) frame=afile->frames;
   seekstart=(long)((gdouble)(frame-1.)/afile->fps*afile->arate)*afile->achans*(afile->asampsize/8);
-  afile->aseek_pos=jack_audio_seek_bytes(jackd,seekstart);
+  jack_audio_seek_bytes(jackd,seekstart);
 }
 
 
