@@ -6,11 +6,12 @@
 
 #include "../libweed/weed.h"
 #include "../libweed/weed-host.h"
-#include "../libweed/weed-effects.h"
+#include "../libweed/weed-palettes.h"
 
 #include "main.h"
 
 #include "support.h"
+#include "interface.h"
 #include "lives-yuv4mpeg.h"
 #include <sys/types.h>
 #include <sys/file.h>
@@ -248,6 +249,7 @@ void weed_layer_set_from_yuv4m (weed_plant_t *layer, file *sfile) {
   weed_set_int_value(layer,"width",sfile->hsize);
   weed_set_int_value(layer,"height",sfile->vsize);
   weed_set_int_value(layer,"current_palette",WEED_PALETTE_YUV420P);
+  weed_set_int_value(layer,"YUV_subspace",WEED_YUV_SUBSPACE_YCBCR);
 
   create_empty_pixel_data(layer);
 
@@ -413,7 +415,7 @@ void on_open_yuv4m_activate (GtkMenuItem *menuitem, gpointer user_data) {
 
 
 gboolean 
-lives_yuv_stream_start_write (lives_yuv4m_t * yuv4mpeg, gchar *filename, gint hsize, gint vsize, gdouble fps) {
+lives_yuv_stream_start_write (lives_yuv4m_t * yuv4mpeg, const gchar *filename, gint hsize, gint vsize, gdouble fps) {
   int i;
 
   if (mainw->fixed_fpsd>-1.&&mainw->fixed_fpsd!=fps) {
@@ -539,11 +541,20 @@ on_live_tvcard_activate                      (GtkMenuItem     *menuitem,
 
   GtkWidget *card_dialog;
 
+  tvcardw_t *tvcardw;
+
+  mainw->open_deint=FALSE;
+
   card_dialog=create_cdtrack_dialog(4,NULL);
+
+  tvcardw=g_object_get_data(G_OBJECT(card_dialog),"tvcard_data");
+
+
   response=gtk_dialog_run(GTK_DIALOG(card_dialog));
   if (response==GTK_RESPONSE_CANCEL) {
     gtk_widget_destroy(card_dialog);
     g_free(fifofile);
+    g_free(tvcardw);
     return;
   }
 
@@ -556,6 +567,7 @@ on_live_tvcard_activate                      (GtkMenuItem     *menuitem,
     do_card_in_use_error();
     g_free(chanstr);
     g_free(fifofile);
+    g_free(tvcardw);
     return;
   }
 
@@ -565,6 +577,7 @@ on_live_tvcard_activate                      (GtkMenuItem     *menuitem,
     g_free(chanstr);
     g_free(fifofile);
     g_free(fname);
+    g_free(tvcardw);
     return;
   }
 
@@ -576,6 +589,7 @@ on_live_tvcard_activate                      (GtkMenuItem     *menuitem,
     g_free(chanstr);
     g_free(fifofile);
     g_free(fname);
+    g_free(tvcardw);
     return;
   }
 
@@ -583,10 +597,35 @@ on_live_tvcard_activate                      (GtkMenuItem     *menuitem,
 
   mainw->current_file=new_file;
 
+  cfile->deinterlace=mainw->open_deint;
+
   unlink(fifofile);
   mkfifo(fifofile,S_IRUSR|S_IWUSR);
 
-  com=g_strdup_printf("smogrify open_tv_card %s \"%s\" %s %s",cfile->handle,chanstr,devstr,fifofile);
+  if (!tvcardw->use_advanced) {
+    com=g_strdup_printf("smogrify open_tv_card %s \"%s\" %s %s",cfile->handle,chanstr,devstr,fifofile);
+  }
+  else {
+    gdouble fps=0.;
+    gchar *driver=NULL,*outfmt=NULL;
+    gint width=0,height=0;
+    gint input=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tvcardw->spinbuttoni));
+    
+    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tvcardw->radiobuttond))) {
+      width=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tvcardw->spinbuttonw));
+      height=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tvcardw->spinbuttonh));
+      fps=gtk_spin_button_get_value(GTK_SPIN_BUTTON(tvcardw->spinbuttonf));
+    }
+
+    driver=g_strdup(gtk_entry_get_text(GTK_ENTRY((GTK_COMBO(tvcardw->combod))->entry)));
+    outfmt=g_strdup(gtk_entry_get_text(GTK_ENTRY((GTK_COMBO(tvcardw->comboo))->entry)));
+
+    com=g_strdup_printf("smogrify open_tv_card %s \"%s\" %s %s %d %d %d %.3f %s %s",cfile->handle,chanstr,devstr,fifofile,input,width,height,fps,driver,outfmt);
+    g_free(driver);
+    g_free(outfmt);
+
+  }
+
   dummyvar=system(com);
   g_free(com);
 
@@ -595,6 +634,7 @@ on_live_tvcard_activate                      (GtkMenuItem     *menuitem,
     g_free(chanstr);
     g_free(fifofile);
     g_free(devstr);
+    g_free(tvcardw);
     return;
   }
 			       
@@ -607,6 +647,7 @@ on_live_tvcard_activate                      (GtkMenuItem     *menuitem,
   g_free(chanstr);
   g_free(devstr);
   g_free(fifofile);
+  g_free(tvcardw);
   
 }
 
@@ -630,6 +671,8 @@ on_live_fw_activate                      (GtkMenuItem     *menuitem,
 
   GtkWidget *card_dialog;
 
+  mainw->open_deint=FALSE;
+
   card_dialog=create_cdtrack_dialog(5,NULL);
   response=gtk_dialog_run(GTK_DIALOG(card_dialog));
   if (response==GTK_RESPONSE_CANCEL) {
@@ -659,6 +702,7 @@ on_live_fw_activate                      (GtkMenuItem     *menuitem,
   fw_cards=g_list_append(fw_cards,GINT_TO_POINTER(cardno));
 
   mainw->current_file=new_file;
+  cfile->deinterlace=mainw->open_deint;
 
   unlink(fifofile);
   mkfifo(fifofile,S_IRUSR|S_IWUSR);
