@@ -61,6 +61,7 @@ POSSIBILITY OF SUCH DAMAGES.
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
+#include <assert.h>
 
 // don't change this unless the backend is changed as well
 // i.e. $GUI_BOOTSTRAP_FILE in smogrify
@@ -109,10 +110,6 @@ POSSIBILITY OF SUCH DAMAGES.
 #endif
 #endif
 
-// insert methods
-#define AFTER_END 1
-#define BEFORE_START 0
-
 // LiVES will show a warning if this (MBytes) is exceeded on load
 // (can be overridden in prefs)
 #define WARN_FILE_SIZE 500
@@ -214,6 +211,131 @@ typedef struct {
 #include "events.h"
 
 
+typedef enum {
+  UNDO_NONE=0,
+  UNDO_EFFECT,
+  UNDO_RESIZABLE,
+  UNDO_MERGE,
+  UNDO_RESAMPLE,
+  UNDO_TRIM_AUDIO,
+  UNDO_CHANGE_SPEED,
+  UNDO_AUDIO_RESAMPLE,
+  UNDO_APPEND_AUDIO,
+  UNDO_INSERT,
+  UNDO_CUT,
+  UNDO_DELETE,
+  UNDO_DELETE_AUDIO,
+  UNDO_INSERT_SILENCE,
+  UNDO_NEW_AUDIO,
+
+  // resample/resize and resample audio for encoding
+  UNDO_ATOMIC_RESAMPLE_RESIZE,
+
+  // resample/reorder/resize/apply effects
+  UNDO_RENDER,
+
+  UNDO_FADE_AUDIO,
+
+  // record audio to selection
+  UNDO_REC_AUDIO,
+
+  UNDO_INSERT_WITH_AUDIO
+} lives_undo_t;
+
+
+// which stream end should cause playback to finish ?
+typedef enum {
+  NEVER_STOP=0,
+  STOP_ON_VID_END,
+  STOP_ON_AUD_END
+} lives_whentostop_t;
+
+
+// cancel reason
+typedef enum {
+  // no cancel
+  CANCEL_NONE=0,
+
+  // user pressed stop
+  CANCEL_USER=1,
+
+  // cancel but keep opening
+  CANCEL_NO_PROPOGATE=2,
+
+  // effect processing finished during preview
+  CANCEL_PREVIEW_FINISHED=3,
+
+  // application quit
+  CANCEL_APP_QUIT=4,
+
+  // ran out of preview frames
+  CANCEL_NO_MORE_PREVIEW=5,
+
+  // image could not be captured
+  CANCEL_CAPTURE_ERROR=6,
+
+  // event_list completed
+  CANCEL_EVENT_LIST_END=7,
+
+  // video playback completed
+  CANCEL_VID_END=8,
+
+  // generator was stopped
+  CANCEL_GENERATOR_END=9,
+
+  // user pressed 'Keep'
+  CANCEL_KEEP=10,
+
+  // video playback completed
+  CANCEL_AUD_END=11,
+
+  // cancelled because of error
+  CANCEL_ERROR=12,
+
+  // cancelled and paused
+  CANCEL_USER_PAUSED=13,
+
+  // special cancel for TV toy
+  CANCEL_KEEP_LOOPING=100
+
+} lives_cancel_t;
+
+
+typedef enum {
+  CANCEL_KILL=0,  // normal - kill background processes working on current clip
+  CANCEL_SOFT     // just cancel in GUI (for keep, etc)
+} lives_cancel_type_t;
+
+
+
+
+typedef enum {
+  CLIP_TYPE_DISK, // imported video, broken into frames
+  CLIP_TYPE_YUV4MPEG,
+  CLIP_TYPE_GENERATOR,
+  CLIP_TYPE_FILE, // unimported video, not or partially broken in frames
+  CLIP_TYPE_LIVES2LIVES // type for LiVES to LiVES streaming
+} lives_clip_type_t;
+
+
+typedef enum {
+  IMG_TYPE_UNKNOWN=(1<<0),
+  IMG_TYPE_JPEG=(1<<1),
+  IMG_TYPE_PNG=(1<<2)
+} lives_image_type_t;
+
+
+
+#define AFORM_SIGNED 0
+#define AFORM_LITTLE_ENDIAN 0
+
+#define AFORM_UNSIGNED 1
+#define AFORM_BIG_ENDIAN (1<<1)
+#define AFORM_UNKNOWN 65536
+
+
+
+
 
 typedef struct {                // corresponds to one clip in the GUI
   // cfile - one clip
@@ -226,13 +348,6 @@ typedef struct {                // corresponds to one clip in the GUI
   gint arps; // audio sample rate
   guint signed_endian; // bitfield
 
-  #define AFORM_SIGNED 0
-  #define AFORM_LITTLE_ENDIAN 0
-
-  #define AFORM_UNSIGNED 1
-  #define AFORM_BIG_ENDIAN (1<<1)
-  #define AFORM_UNKNOWN 65536
-
   gint arate; // audio playback rate
   gint64 unique_id;    // this and the handle can be used to uniquely id a file
   gint achans;
@@ -246,6 +361,8 @@ typedef struct {                // corresponds to one clip in the GUI
   gchar keywords[256];
   ////////////////
 
+  gint interlace; // interlace type (if known - none, topfirst, bottomfirst or : see plugins.h)
+
   // extended info (not saved)
   gint header_version;
 
@@ -323,42 +440,8 @@ typedef struct {                // corresponds to one clip in the GUI
   GList *layout_map;
   ////////////////////////////////////////////////////////////////////////////////////////
 
-
-
-
-
   //undo
-  gshort undo_action;
-#define UNDO_NONE 0
-#define UNDO_EFFECT 1
-#define UNDO_RESIZABLE 2
-#define UNDO_MERGE 3
-
-
-#define UNDO_RESAMPLE 5
-#define UNDO_TRIM_AUDIO 6
-#define UNDO_CHANGE_SPEED 7
-#define UNDO_AUDIO_RESAMPLE 8
-#define UNDO_APPEND_AUDIO 9
-#define UNDO_INSERT 10
-#define UNDO_CUT 11
-#define UNDO_DELETE 12
-#define UNDO_DELETE_AUDIO 13
-#define UNDO_INSERT_SILENCE 14
-#define UNDO_NEW_AUDIO 15
-
-  // resample/resize and resample audio for encoding
-#define UNDO_ATOMIC_RESAMPLE_RESIZE 16
-
-  // resample/reorder/resize/apply effects
-#define UNDO_RENDER 17
-
-#define UNDO_FADE_AUDIO 18
-
-  // record audio to selection
-#define UNDO_REC_AUDIO 19
-
-#define UNDO_INSERT_WITH_AUDIO 20
+  lives_undo_t undo_action;
 
   gint undo_start;
   gint undo_end;
@@ -385,12 +468,8 @@ typedef struct {                // corresponds to one clip in the GUI
   gint undo_asampsize;
   gint undo_arps; // audio sample rate
 
-  gshort clip_type;
-#define CLIP_TYPE_DISK 0 // imported video, broken into frames
-#define CLIP_TYPE_YUV4MPEG 1
-#define CLIP_TYPE_GENERATOR 2
-#define CLIP_TYPE_FILE 3 // unimported video, not or partially broken in frames
-#define CLIP_TYPE_LIVES2LIVES 4 // type for LiVES to LiVES streaming
+  lives_clip_type_t clip_type;
+
   void *ext_src; // points to opaque source for non-disk types
 
   int *frame_index; // index of frames for CLIP_TYPE_FILE
@@ -417,11 +496,7 @@ typedef struct {                // corresponds to one clip in the GUI
 
   gboolean deinterlace;
 
-#define IMG_TYPE_UNKNOWN (1<<0)
-#define IMG_TYPE_JPEG (1<<1)
-#define IMG_TYPE_PNG (1<<2)
-
-  guint img_type;
+  lives_image_type_t img_type;
 
   // layout map for the current layout
   gint stored_layout_frame;
@@ -483,10 +558,8 @@ capability *capable;
 #ifdef HAVE_JACK_JACK_H
 
 #ifdef HAVE_JACK_TRANSPORT_H
-#define ENABLE_JACK_TRANSPORT
 #endif
 
-#define ENABLE_JACK
 #include "jack.h"
 #endif
 
@@ -513,6 +586,44 @@ struct timeval tv;
 size_t sizint, sizdbl, sizshrt;
 
 
+
+typedef enum {
+  CLIP_DETAILS_BPP,
+  CLIP_DETAILS_FPS,
+  CLIP_DETAILS_PB_FPS,
+  CLIP_DETAILS_WIDTH,
+  CLIP_DETAILS_HEIGHT,
+  CLIP_DETAILS_UNIQUE_ID,
+  CLIP_DETAILS_ARATE,
+  CLIP_DETAILS_PB_ARATE,
+  CLIP_DETAILS_ACHANS,
+  CLIP_DETAILS_ASIGNED,
+  CLIP_DETAILS_AENDIAN,
+  CLIP_DETAILS_ASAMPS,
+  CLIP_DETAILS_FRAMES,
+  CLIP_DETAILS_TITLE,
+  CLIP_DETAILS_AUTHOR,
+  CLIP_DETAILS_COMMENT,
+  CLIP_DETAILS_PB_FRAMENO,
+  CLIP_DETAILS_FILENAME,
+  CLIP_DETAILS_CLIPNAME,
+  CLIP_DETAILS_HEADER_VERSION,
+  CLIP_DETAILS_KEYWORDS,
+  CLIP_DETAILS_INTERLACE
+} lives_clip_details_t;
+
+
+typedef enum {
+  LIVES_CURSOR_NORMAL=0,  // must be zero
+  LIVES_CURSOR_BLOCK,
+  LIVES_CURSOR_AUDIO_BLOCK,
+  LIVES_CURSOR_BUSY,
+  LIVES_CURSOR_FX_BLOCK
+} lives_cursor_t;
+
+
+
+
 // some useful functions
 
 //gui.c
@@ -568,8 +679,8 @@ void do_aud_during_play_error(void);
 void do_rendered_fx_dialog(void);
 void do_layout_scrap_file_error(void);
 void do_set_load_lmap_error(void);
-gboolean do_set_duplicate_warning (gchar *new_set);
-gboolean do_set_rename_old_layouts_warning(gchar *new_set);
+gboolean do_set_duplicate_warning (const gchar *new_set);
+gboolean do_set_rename_old_layouts_warning(const gchar *new_set);
 gboolean do_layout_alter_frames_warning(void);
 gboolean do_layout_alter_audio_warning(void);
 gboolean do_yuv4m_open_warning(void);
@@ -587,20 +698,21 @@ void do_vpp_palette_error (void);
 void do_vpp_fps_error (void);
 void do_decoder_palette_error (void);
 void do_rmem_max_error (gint size);
-gboolean do_original_lost_warning(gchar *fname);
-void do_no_decoder_error(gchar *fname);
+gboolean do_original_lost_warning(const gchar *fname);
+void do_no_decoder_error(const gchar *fname);
 void do_jack_noopen_warn(void);
 void do_jack_noopen_warn2(void);
-void do_file_perm_error(gchar *file_name);
-void do_dir_perm_error(gchar *dir_name);
+void do_jack_noopen_warn3(void);
+void do_file_perm_error(const gchar *file_name);
+void do_dir_perm_error(const gchar *dir_name);
 void do_encoder_img_ftm_error(render_details *rdet);
 void do_after_crash_warning (void);
 void do_bad_layout_error(void);
 void do_card_in_use_error(void);
-void do_dev_busy_error(gchar *devstr);
+void do_dev_busy_error(const gchar *devstr);
 
 gboolean process_one (gboolean visible);
-void do_threaded_dialog(gchar *text, gboolean has_cancel);
+void do_threaded_dialog(const gchar *text, gboolean has_cancel);
 void end_threaded_dialog(void);
 void response_ok (GtkButton *button, gpointer user_data);
 void response_cancel (GtkButton *button, gpointer user_data);
@@ -622,7 +734,7 @@ void do_text_window (const gchar *title, const gchar *text);
 void add_file_info(const gchar *check_handle, gboolean aud_only);
 void save_file_comments (void);
 void reget_afilesize (int fileno);
-void deduce_file(gchar *filename, gdouble start_time, gint end);
+void deduce_file(const gchar *filename, gdouble start_time, gint end);
 void open_file (const gchar *filename);
 void open_file_sel(const gchar *file_name,gdouble start_time, gint frames);
 void open_fw_device (void);
@@ -635,11 +747,11 @@ void play_file (void);
 gboolean save_frame(gint frame, const gchar *filename);
 void wait_for_stop (const gchar *stop_command);
 void save_clip_values(gint which_file);
-void add_to_recovery_file (gchar *handle);
+void add_to_recovery_file (const gchar *handle);
 void rewrite_recovery_file(void);
 gboolean check_for_recovery_files (gboolean auto_recover);
 void recover_layout_map(gint numclips);
-gchar *get_deinterlace_string(void);
+const gchar *get_deinterlace_string(void);
 
 // saveplay.c backup
 void backup_file(const gchar *filename);
@@ -651,7 +763,7 @@ void restore_file(const gchar *filename);
 gboolean read_headers(const gchar *file_name);
 
 // saveplay.c sets
-void open_set_file (gchar *set_name, gint clipnum);
+void open_set_file (const gchar *set_name, gint clipnum);
 
 
 // saveplay.c scrap file
@@ -663,9 +775,9 @@ void close_scrap_file (void);
 
 // main.c
 void catch_sigint(int signum);
-gboolean startup_message_fatal(gchar *msg);
-gboolean startup_message_nonfatal(gchar *msg);
-gboolean startup_message_nonfatal_dismissable(gchar *msg, gint warning_mask);
+gboolean startup_message_fatal(const gchar *msg);
+gboolean startup_message_nonfatal(const gchar *msg);
+gboolean startup_message_nonfatal_dismissable(const gchar *msg, gint warning_mask);
 capability *get_capabilities(void);
 void get_monitors(void);
 void load_start_image(gint frame);
@@ -722,7 +834,7 @@ void get_filename(gchar *filename);
 gchar *get_extension(const gchar *filename);
 void d_print(const gchar *text);
 void init_clipboard(void);
-void cache_file_contents(gchar *filename);
+void cache_file_contents(const gchar *filename);
 gchar *get_val_from_cached_list(const gchar *key, size_t maxlen);
 void get_pref(const gchar *key, gchar *val, gint maxlen);
 void get_pref_utf8(const gchar *key, gchar *val, gint maxlen);
@@ -744,7 +856,7 @@ gint get_box_child_index (GtkBox *box, GtkWidget *tchild);
 void reset_clip_menu (void);
 void get_play_times(void);
 void get_total_time (file *file);
-guint get_signed_endian (int asigned, int endian);
+guint get_signed_endian (gboolean is_signed, gboolean little_endian);
 void fullscreen_internal(void);
 void unhide_cursor(GdkWindow *window);
 void hide_cursor(GdkWindow *window);
@@ -759,12 +871,12 @@ void switch_aud_to_mplayer(void);
 void prepare_to_play_foreign(void);
 gboolean after_foreign_play(void);
 gboolean check_file(const gchar *file_name, gboolean check_exists);  // check if file exists
-gboolean check_dir_access (gchar *dir);
+gboolean check_dir_access (const gchar *dir);
 gulong get_file_size(int fd);
-gulong sget_file_size(gchar *name);
+gulong sget_file_size(const gchar *name);
 gulong get_fs_free(const gchar *dir);
 gboolean ensure_isdir(gchar *fname);
-gchar *ensure_extension(gchar *fname, gchar *ext);
+gchar *ensure_extension(const gchar *fname, const gchar *ext) WARN_UNUSED;
 void activate_url_inner(const gchar *link);
 void activate_url (GtkAboutDialog *about, const gchar *link, gpointer data);
 void show_manual_section (const gchar *lang, const gchar *section);
@@ -775,24 +887,24 @@ LIVES_INLINE gint calc_frame_from_time2 (gint filenum, gdouble time); // nearest
 LIVES_INLINE gint calc_frame_from_time3 (gint filenum, gdouble time); // nearest frame mid
 
 gboolean check_for_ratio_fps (gdouble fps);
-gdouble get_ratio_fps(gchar *string);
+gdouble get_ratio_fps(const gchar *string);
 void calc_maxspect(gint rwidth, gint rheight, gint *cwidth, gint *cheight);
 
 gchar *remove_trailing_zeroes(gdouble val);
 void toggle_button_toggle (GtkToggleButton *tbutton);
 void remove_layout_files(GList *lmap);
-gboolean add_lmap_error(int lerror, gchar *name, gpointer user_data, gint clipno, gint frameno, gdouble atime, gboolean affects_current);
+gboolean add_lmap_error(lives_lmap_error_t lerror, const gchar *name, gpointer user_data, gint clipno, gint frameno, gdouble atime, gboolean affects_current);
 void clear_lmap_errors(void);
 gboolean prompt_remove_layout_files(void);
-gboolean is_legal_set_name(gchar *set_name, gboolean allow_dupes);
-gchar *repl_tmpdir(gchar *entry, gboolean fwd);
-gboolean get_clip_value(int which, int what, void *retval, size_t maxlen);
-void save_clip_value(int which, int what, void *val);
+gboolean is_legal_set_name(const gchar *set_name, gboolean allow_dupes);
+gchar *repl_tmpdir(const gchar *entry, gboolean fwd);
+gboolean get_clip_value(int which, lives_clip_details_t, void *retval, size_t maxlen);
+void save_clip_value(int which, lives_clip_details_t, void *val);
 gboolean check_frame_count(gint idx);
 void get_frame_count(gint idx);
 gint count_resampled_frames (gint in_frames, gdouble orig_fps, gdouble resampled_fps);
 gboolean int_array_contains_value(int *array, int num_elems, int value);
-gboolean check_for_lock_file(gchar *set_name, gint type);
+gboolean check_for_lock_file(const gchar *set_name, gint type);
 void g_list_free_strings(GList *list);
 void gtk_tooltips_copy(GtkWidget *dest, GtkWidget *source);
 void adjustment_configure(GtkAdjustment *adjustment, gdouble value, gdouble lower, gdouble upper, gdouble step_increment, gdouble page_increment, gdouble page_size);
@@ -810,18 +922,21 @@ GdkPixmap* gdk_pixmap_copy (GdkPixmap *pixmap);
 GdkPixbuf *gdk_pixbuf_new_blank(gint width, gint height, int palette);
 void get_border_size (GtkWidget *win, gint *bx, gint *by);
 gchar *g_strappend (gchar *string, gint len, const gchar *new);
-GList *g_list_append_unique(GList *xlist, gchar *add);
+GList *g_list_append_unique(GList *xlist, const gchar *add);
 void find_when_to_stop (void);
-gint calc_new_playback_position(gint fileno, weed_timecode_t otc, weed_timecode_t *ntc);
+gint calc_new_playback_position(gint fileno, gint64 otc, gint64 *ntc);
 void minimise_aspect_delta (gdouble allowed_aspect,gint hblock,gint vblock,gint hsize,gint vsize,gint *width,gint *height);
+LIVES_INLINE gint get_interp_value(gshort quality);
+LIVES_INLINE GList *g_list_move_to_first(GList *list, GList *item) WARN_UNUSED;
+
 
 GList *get_set_list(const gchar *dir);
 void combo_set_popdown_strings (GtkCombo *combo, GList *list);
 
-gchar *subst (gchar *string, gchar *from, gchar *to);
+gchar *subst (const gchar *string, const gchar *from, const gchar *to);
 
 gint hextodec (const gchar *string);
-gint get_hex_digit (gchar *c);
+gint get_hex_digit (const gchar *c);
 
 guint32 fastrand(void);
 void fastsrand(guint32 seed);
@@ -832,14 +947,7 @@ void set_fg_colour(gint red, gint green, gint blue);
 gboolean label_act_toggle (GtkWidget *, GdkEventButton *, GtkToggleButton *);
 gboolean widget_act_toggle (GtkWidget *, GtkToggleButton *);
 
-void lives_set_cursor_style(gint cstyle, GdkWindow *window);
-
-#define LIVES_CURSOR_NORMAL 0  // must be zero
-#define LIVES_CURSOR_BLOCK 1
-#define LIVES_CURSOR_AUDIO_BLOCK 2
-#define LIVES_CURSOR_BUSY 3
-#define LIVES_CURSOR_FX_BLOCK 4
-
+void lives_set_cursor_style(lives_cursor_t cstyle, GdkWindow *window);
 
 
 gchar *text_view_get_text(GtkTextView *textview);
@@ -877,7 +985,7 @@ void *w_memcpy  (void *dest, const void *src, size_t n);
 gboolean lives_osc_init(guint osc_udp_port);
 gint lives_osc_poll(gpointer data);
 void lives_osc_end(void);
-gboolean lives_osc_notify(int msgtype, char *msgstring);
+gboolean lives_osc_notify(int msgtype, const char *msgstring);
 #include "osc_notify.h"
 #endif
 
@@ -901,35 +1009,8 @@ char *dummychar;
 #define L2U8(String) ( g_locale_to_utf8 (String,-1,NULL,NULL,NULL) ) 
 
 
-/* clip details keys */
-#define CLIP_DETAILS_BPP 1
-#define CLIP_DETAILS_FPS 2
-#define CLIP_DETAILS_PB_FPS 3
-#define CLIP_DETAILS_WIDTH 4
-#define CLIP_DETAILS_HEIGHT 5
-#define CLIP_DETAILS_UNIQUE_ID 6
-#define CLIP_DETAILS_ARATE 7
-#define CLIP_DETAILS_PB_ARATE 8
-#define CLIP_DETAILS_ACHANS 9
-#define CLIP_DETAILS_ASIGNED 10
-#define CLIP_DETAILS_AENDIAN 11
-#define CLIP_DETAILS_ASAMPS 12
-#define CLIP_DETAILS_FRAMES 13
-#define CLIP_DETAILS_TITLE 14
-#define CLIP_DETAILS_AUTHOR 15
-#define CLIP_DETAILS_COMMENT 16
-#define CLIP_DETAILS_PB_FRAMENO 17
-#define CLIP_DETAILS_FILENAME 18
-#define CLIP_DETAILS_CLIPNAME 19
-#define CLIP_DETAILS_HEADER_VERSION 20
-#define CLIP_DETAILS_KEYWORDS 21
-
 #define PREFS_TIMEOUT 10000000 // 10 seconds // TODO !
 
-#define LIVES_TOY_NONE 0
-#define LIVES_TOY_MAD_FRAMES 1
-#define LIVES_TOY_TV 2
-#define LIVES_TOY_AUTOLIVES 2
 #define LIVES_TV_CHANNEL1 "http://www.serverwillprovide.com/sorteal/livestvclips/livestv.ogm"
 
 #endif // #ifndef HAS_MAIN_H
