@@ -34,6 +34,10 @@
 // vert displacement up from center for sepwin (actual value is half this)
 #define SEPWIN_VADJUST 200
 
+// default size for generators
+#define DEF_GEN_WIDTH 640
+#define DEF_GEN_HEIGHT 480
+
 // number of function keys
 #define FN_KEYS 12
 
@@ -47,19 +51,39 @@
 // the rest of the keys are accessible through the multitrack renderer (must, be > FX_KEYS_MAX_VIRTUAL)
 #define FX_KEYS_MAX 65536
 
+#define EFFECT_NONE 0
+#define GU641 ((guint64)1)
 
-// maximum number of external control peripherals
-#define MAX_EXT_CNTL 32
+
+// max ext_cntl + 1
+#define MAX_EXT_CNTL 2
 
 // external control types
-#define EXT_CNTL_NONE 0 // not used
-#define EXT_CNTL_JS 1
-#define EXT_CNTL_MIDI 2
+typedef enum {
+  EXT_CNTL_NONE=-1, // not used
+  EXT_CNTL_JS=0,
+  EXT_CNTL_MIDI=1
+} lives_ext_cntl_t;
+
+
+// timebase sources
+typedef enum {
+  LIVES_TIME_SOURCE_NONE=0,
+  LIVES_TIME_SOURCE_SYSTEM,
+  LIVES_TIME_SOURCE_SOUNDCARD,
+  LIVES_TIME_SOURCE_EXTERNAL
+} lives_time_source_t;
+
+
+
+typedef enum {
+  LIVES_TOY_NONE=0,
+  LIVES_TOY_MAD_FRAMES,
+  LIVES_TOY_TV,
+  LIVES_TOY_AUTOLIVES
+} lives_toy_t;
 
 
-// default size for generators
-#define DEF_GEN_WIDTH 640
-#define DEF_GEN_HEIGHT 480
 
 
 
@@ -80,6 +104,8 @@ typedef struct {
   GdkColor light_yellow;
   GdkColor pink;
   GdkColor light_red;
+  GdkColor dark_red;
+  GdkColor light_green;
   GdkColor grey20;
   GdkColor grey25;
   GdkColor grey45;
@@ -171,58 +197,11 @@ typedef struct {
   gboolean record;
 
   gboolean in_fs_preview;
-  volatile gint cancelled;
-
-  // no cancel
-#define CANCEL_NONE 0
-
-  // user pressed stop
-#define CANCEL_USER 1
-
-  // cancel but keep opening
-#define CANCEL_NO_PROPOGATE 2
-
-  // effect processing finished during preview
-#define CANCEL_PREVIEW_FINISHED 3
-
-  // application quit
-#define CANCEL_APP_QUIT 4
-
-  // ran out of preview frames
-#define CANCEL_NO_MORE_PREVIEW 5
-
-  // image could not be captured
-#define CANCEL_CAPTURE_ERROR 6
-
-  // event_list completed
-#define CANCEL_EVENT_LIST_END 7
-
-  // video playback completed
-#define CANCEL_VID_END 8
-
-  // generator was stopped
-#define CANCEL_GENERATOR_END 9
-
-  // user pressed 'Keep'
-#define CANCEL_KEEP 10
-
-  // video playback completed
-#define CANCEL_AUD_END 11
-
-  // cancelled because of error
-#define CANCEL_ERROR 12
-
-  // cancelled and paused
-#define CANCEL_USER_PAUSED 13
-
-  // special cancel for TV toy
-#define CANCEL_KEEP_LOOPING 100
+  volatile lives_cancel_t cancelled;
 
   gboolean error;
 
-  guint cancel_type;
-#define CANCEL_KILL 0  // normal - kill background processes working on current clip
-#define CANCEL_SOFT 1 // just cancel in GUI (for keep, etc)
+  lives_cancel_type_t cancel_type;
 
   weed_plant_t *event_list; // current event_list, for recording
   weed_plant_t *stored_event_list; // stored mt -> clip editor
@@ -240,11 +219,7 @@ typedef struct {
   gint pwidth; // playback width in RGB pixels
   gint pheight; // playback height
 
-  gshort whentostop;
-  // which stream end should cause playback to finish ?
-#define NEVER_STOP 0
-#define STOP_ON_VID_END 1
-#define STOP_ON_AUD_END 2
+  lives_whentostop_t whentostop;
 
   gboolean noframedrop;
 
@@ -285,10 +260,8 @@ typedef struct {
   gboolean last_transition_align_start;
   gboolean last_transition_ins_frames;
 
-#define GU641 ((guint64)1)
-  guint64 rte; // current max for VJ mode == 64 effects on fg clip
 
-#define EFFECT_NONE 0
+  guint64 rte; // current max for VJ mode == 64 effects on fg clip
 
   guint last_grabable_effect;
   guint rte_keys; // which effect is bound to keyboard
@@ -307,10 +280,8 @@ typedef struct {
 
   gint scr_width;
   gint scr_height;
-  gint toy_type;
-#define TOY_NONE 0
-#define TOY_RANDOM_FRAMES 1
-#define TOY_TV 1
+  lives_toy_t toy_type;
+
   gboolean toy_go_wild;
 
   // copy/paste
@@ -368,11 +339,12 @@ typedef struct {
   gdouble period; // == 1./cfile->pb_fps (unless cfile->pb_fps is 0.)
   gint64 startticks; // effective ticks when last frame was (should have been) displayed
   gint64 timeout_ticks; // incremented if effect/rendering is paused/previewed
-  gint64 startsecs; // playback start seconds - subtracted from all other ticks to keep numbers smaller
-  gint64 currticks; // current playback ticks : goes from origticks upwards
+  gint64 origsecs; // playback start seconds - subtracted from all other ticks to keep numbers smaller
+  gint64 origusecs; // usecs at start of playback - ditto
+  gint64 offsetticks; // offset for external transport
+  gint64 currticks; // current playback ticks (relative)
   gint64 deltaticks; // deltaticks for scratching
-  gint64 firstticks; // ticks when audio started playing
-  gint64 origticks; // ticks at start of playback
+  gint64 firstticks; // ticks when audio started playing (for non-realtime audio plugins)
   gint64 stream_ticks;  // ticks since first frame sent to playback plugin
 
   gboolean size_warn; // warn the user that incorrectly sized frames were found
@@ -402,10 +374,11 @@ typedef struct {
   // video playback plugin was updated; write settings to a file
   gboolean write_vpp_file;
 
-  gshort scratch;
+  volatile gshort scratch;
 #define SCRATCH_NONE 0
 #define SCRATCH_BACK -1
 #define SCRATCH_FWD 1
+#define SCRATCH_JUMP 2
 
   // internal fx
   gboolean internal_messaging;
@@ -468,7 +441,7 @@ typedef struct {
   gboolean jack_can_stop;
   gboolean jack_can_start;
 
-  gboolean video_seek_ready;
+  volatile gboolean video_seek_ready;
 
   // selection pointers
   gulong mouse_fn1;
@@ -654,7 +627,6 @@ typedef struct {
   void ***pchains;
 
   // for the internal player
-  gint fixed_height;
   GtkWidget *image274;
   GtkWidget *play_window;
   weed_plant_t *frame_layer;
@@ -926,7 +898,12 @@ typedef struct {
   pthread_t sig_pid;
   gchar *sig_file;
 
+  // decoders
+  gboolean decoders_loaded;
+  GList *decoder_list;
+
   gboolean go_away;
+  gboolean debug; // debug crashes and asserts
 
 } mainwindow;
 
@@ -957,4 +934,16 @@ _merge_opts* merge_opts;
 // note, we can only have two of these currently, one for rendered effects, one for real time effects
 GtkWidget *fx_dialog[2];
 
+
+
+#ifdef ENABLE_JACK
+  volatile aserver_message_t jack_message;
+  volatile aserver_message_t jack_message2;
+#endif
+
+#ifdef HAVE_PULSE_AUDIO
+  volatile aserver_message_t pulse_message;
+  volatile aserver_message_t pulse_message2;
+#endif
+
 #endif // HAS_MAINWINDOW_H
