@@ -1,6 +1,6 @@
 // multitrack.h
 // LiVES
-// (c) G. Finch 2005 - 2009 <salsaman@xs4all.nl>
+// (c) G. Finch 2005 - 2010 <salsaman@xs4all.nl>
 // released under the GNU GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -21,7 +21,7 @@
 #define BLOCK_DRAW_TYPE BLOCK_DRAW_THUMB
 
 #define MT_PLAY_WIDTH_SMALL 320
-#define MT_PLAY_HEIGHT_SMALL 220
+#define MT_PLAY_HEIGHT_SMALL 216
 
 #define MT_PLAY_WIDTH_EXP 480
 #define MT_PLAY_HEIGHT_EXP 360
@@ -47,14 +47,119 @@ typedef struct _track_rect track_rect;
 
 typedef struct _mt_opts mt_opts;
 
+typedef struct _lives_amixer_t lives_amixer_t;
+
+#define MAX_DISP_VTRACKS 5
+
+typedef enum {
+  MOUSE_MODE_MOVE,
+  MOUSE_MODE_SELECT,
+  MOUSE_MODE_COPY
+} lives_mt_mouse_mode_t;
+
+
+typedef enum {
+  INSERT_MODE_NORMAL,  // the default (only insert if it fits)
+
+  // not implemented yet
+  INSERT_MODE_OVERWRITE, // overwite existing blocks
+  INSERT_MODE_EXPAND, // repeat to fill gap
+  INSERT_MODE_FILL_START, // insert enough to fill gap (from selection start)
+  INSERT_MODE_FILL_END // insert enough to fill gap (to selection end)
+} lives_mt_insert_mode_t;
+
+
+typedef enum {
+  GRAV_MODE_NORMAL,
+  GRAV_MODE_LEFT,
+  GRAV_MODE_RIGHT
+} lives_mt_grav_mode_t;
+
+
+typedef enum {
+// undo actions
+// no event_list
+  MT_UNDO_NONE=0,
+  MT_UNDO_INSERT_BLOCK=1,
+  MT_UNDO_INSERT_AUDIO_BLOCK=2,
+
+// minimal event_list
+  MT_UNDO_APPLY_FILTER=512,
+  MT_UNDO_DELETE_FILTER=513,
+  MT_UNDO_SPLIT=514,
+  MT_UNDO_SPLIT_MULTI=515,
+  MT_UNDO_FILTER_MAP_CHANGE=516,
+
+// full backups of event_list
+  MT_UNDO_DELETE_BLOCK=1024,
+  MT_UNDO_MOVE_BLOCK=1025,
+  MT_UNDO_REMOVE_GAPS=1026,
+  MT_UNDO_DELETE_AUDIO_BLOCK=1027,
+  MT_UNDO_MOVE_AUDIO_BLOCK=1028,
+  MT_UNDO_INSERT_GAP=1029
+} lives_mt_undo_t;
+
+
+
+typedef enum {
+  NB_ERROR_SEL,
+  NB_ERROR_NOEFFECT,
+  NB_ERROR_NOTRANS,
+  NB_ERROR_NOCOMP,
+  NB_ERROR_NOCLIP
+} lives_mt_nb_error_t;
+  
+
+
+typedef enum {
+  POLY_NONE=0,
+  POLY_CLIPS,
+  POLY_IN_OUT,
+  POLY_FX_STACK,
+  POLY_EFFECTS,
+  POLY_TRANS,
+  POLY_COMP,
+  POLY_PARAMS
+} lives_mt_poly_state_t;
+
+
+
+typedef enum {
+  DIRECTION_NEGATIVE,
+  DIRECTION_POSITIVE
+} lives_direction_t;
+
+
+typedef enum {
+  MT_LAST_FX_NONE=0,
+  MT_LAST_FX_BLOCK,
+  MT_LAST_FX_REGION
+} lives_mt_last_fx_type_t;
+
+
+typedef enum {
+  FX_ORD_NONE=0,
+  FX_ORD_BEFORE,
+  FX_ORD_AFTER
+} lives_mt_fx_order_t;
+
+
+typedef enum {
+// block state
+  BLOCK_UNSELECTED=0,
+  BLOCK_SELECTED
+} lives_mt_block_state_t;
+
+
+
 struct _mt_opts {
   gboolean set; // have we set opts (in mainw) ?
   gboolean move_effects; // should we move effects attached to a block ?
   gboolean fx_auto_preview;
   gboolean snap_over; // snap to overlap
-  gshort grav_mode;
-  gshort mouse_mode;
-  gshort insert_mode;
+  lives_mt_grav_mode_t grav_mode;
+  lives_mt_mouse_mode_t mouse_mode;
+  lives_mt_insert_mode_t insert_mode;
   gboolean show_audio;
   gboolean show_ctx;
   gboolean ign_ins_sel;
@@ -66,23 +171,6 @@ struct _mt_opts {
   gint back_audio_tracks; // number of backing audio tracks (currently 0 or 1)
 };
 
-#define MOUSE_MODE_MOVE 1
-#define MOUSE_MODE_SELECT 2
-#define MOUSE_MODE_COPY 3
-
-#define INSERT_MODE_NORMAL 1  // the default (only insert if it fits)
-
-  // not implemented yet
-#define INSERT_MODE_OVERWRITE 2 // overwite existing blocks
-#define INSERT_MODE_EXPAND 3 // repeat to fill gap
-#define INSERT_MODE_FILL_START 4 // insert enough to fill gap (from selection start)
-#define INSERT_MODE_FILL_END 5 // insert enough to fill gap (to selection end)
-
-
-#define GRAV_MODE_NORMAL 1
-#define GRAV_MODE_LEFT 2
-#define GRAV_MODE_RIGHT 3
-
 
 struct _mt {
   // widgets
@@ -121,6 +209,7 @@ struct _mt {
   GtkWidget *hpaned;
   GtkWidget *scrollbar;
   GtkWidget *playall;
+  GtkWidget *playsel;
   GtkWidget *jumpnext;
   GtkWidget *jumpback;
   GtkWidget *render;
@@ -136,8 +225,6 @@ struct _mt {
   GtkWidget *fx_region_2a;
   GtkWidget *fx_region_3;
   GtkWidget *move_fx;
-  GtkWidget *fx_auto_prev;
-  GtkWidget *fx_auto_prev_box;
   GtkWidget *mm_menuitem;
   GtkWidget *mm_move;
   GtkWidget *mm_select;
@@ -175,7 +262,6 @@ struct _mt {
   GtkWidget *sel_label;
   GtkWidget *l_sel_arrow;
   GtkWidget *r_sel_arrow;
-  GtkWidget *preview_button; // button
   GtkWidget *save_event_list; // menuitem
   GtkWidget *load_event_list; // menuitem
   GtkWidget *clear_event_list; // menuitem
@@ -231,13 +317,6 @@ struct _mt {
   GtkWidget *nb;
   GtkWidget *nb_label;
 
-#define NB_ERROR_SEL 1
-#define NB_ERROR_NOEFFECT 2
-#define NB_ERROR_NOTRANS 3
-#define NB_ERROR_NOCOMP 4
-#define NB_ERROR_NOCLIP 5
-
-
   GtkWidget *open_menu;
   GtkWidget *recent_menu;
   GtkWidget *recent1;
@@ -263,8 +342,6 @@ struct _mt {
   GtkWidget *snapo_checkbutton;
 
 
-
-
   GtkObject *spinbutton_in_adj;
   GtkObject *spinbutton_out_adj;
 
@@ -275,6 +352,7 @@ struct _mt {
   GList *audio_draws; // list of audio boxes, 0 == backing audio, 1 == track 0 audio, etc.
 
   GList *audio_vols; // layer volume levels (coarse control) - set in mixer
+  GList *audio_vols_back; // layer volume levels (coarse control) - reset levels
 
   GtkAccelGroup *accel_group;
   GList *video_draws; // list of video timeline eventboxes, in layer order
@@ -328,21 +406,11 @@ struct _mt {
 
   GList *selected_tracks;
 
-  gshort poly_state;  // state of polymorph window
-#define POLY_NONE 0
-#define POLY_CLIPS 1
-#define POLY_IN_OUT 2
-#define POLY_FX_STACK 3
-#define POLY_EFFECTS 4
-#define POLY_TRANS 5
-#define POLY_COMP 6
-#define POLY_PARAMS 7
+  lives_mt_poly_state_t poly_state;  // state of polymorph window
 
   gint render_file;
 
-  gshort last_direction; // last direction timeline cursor was moved in
-#define DIRECTION_NEGATIVE 0
-#define DIRECTION_POSITIVE 1
+  lives_direction_t last_direction; // last direction timeline cursor was moved in
 
   track_rect *block_selected; // pointer to current selected block, or NULL
   track_rect *putative_block; // putative block to move or copy, or NULL
@@ -361,7 +429,6 @@ struct _mt {
   gboolean is_rendering; // TRUE if we are in the process of rendering/pre-rendering to a clip, cf. mainw->is_rendering
   gboolean pr_audio; // TRUE if we are in the process of prerendering audio to a clip
 
-#define MAX_DISP_VTRACKS 5
   gint max_disp_vtracks;
   gint current_fx;
 
@@ -419,17 +486,9 @@ struct _mt {
   lives_special_framedraw_rect_t *framedraw;
   gint track_index;
 
-  gint last_fx_type;
+  lives_mt_last_fx_type_t last_fx_type;
 
-#define MT_LAST_FX_NONE 0
-#define MT_LAST_FX_BLOCK 1
-#define MT_LAST_FX_REGION 2
-
-  gshort fx_order;
-
-#define FX_ORD_NONE 0
-#define FX_ORD_BEFORE 1
-#define FX_ORD_AFTER 2
+  lives_mt_fx_order_t fx_order;
 
   mt_opts opts;
 
@@ -502,50 +561,35 @@ struct _mt {
 
   gboolean tl_mouse;
 
+  gboolean playing_sel; // are we playing just the time selection ?
+
   guint idlefunc; // autobackup function 
 
   GList *clip_labels;
 
+  lives_amixer_t *amixer;
+
 };  // lives_mt
 
 
-// undo actions
-// no event_list
-#define MT_UNDO_NONE 0
-#define MT_UNDO_INSERT_BLOCK 1
-#define MT_UNDO_INSERT_AUDIO_BLOCK 2
-
-// minimal event_list
-#define MT_UNDO_APPLY_FILTER 512
-#define MT_UNDO_DELETE_FILTER 513
-#define MT_UNDO_SPLIT 514
-#define MT_UNDO_SPLIT_MULTI 515
-#define MT_UNDO_FILTER_MAP_CHANGE 516
-
-// full backups of event_list
-#define MT_UNDO_DELETE_BLOCK 1024
-#define MT_UNDO_MOVE_BLOCK 1025
-#define MT_UNDO_REMOVE_GAPS 1026
-#define MT_UNDO_DELETE_AUDIO_BLOCK 1027
-#define MT_UNDO_MOVE_AUDIO_BLOCK 1028
-#define MT_UNDO_INSERT_GAP 1029
 
 
 typedef struct {
-  gint action;
+  lives_mt_undo_t action;
   void *extra;
   size_t data_len; // including this mt_undo
 } mt_undo;
 
 
-typedef struct {
+struct _lives_amixer_t {
+  GtkWidget *main_hbox;
   GtkWidget **ch_sliders;
   GtkWidget *gang_checkbutton;
   GtkWidget *inv_checkbutton;
   gulong *ch_slider_fns;
   gint nchans;
   lives_mt *mt;
-} lives_amixer_t;
+};
 
 
 // reasons for track invisibility (bitmap)
@@ -563,7 +607,7 @@ struct _track_rect {
 
   weed_timecode_t offset_start; // offset in sourcefile of first frame
 
-  gshort state;
+  lives_mt_block_state_t state;
   gboolean start_anchored;
   gboolean end_anchored;
   gboolean ordered; // are frames in sequential order ?
@@ -572,10 +616,6 @@ struct _track_rect {
 
 };
 
-// block state
-#define BLOCK_UNSELECTED 0
-#define BLOCK_SELECTED 1
-
 
 /* translation table for matching event_id to init_event */
 typedef struct {
@@ -610,7 +650,7 @@ gboolean multitrack_delete (lives_mt *, gboolean save); // free the lives_mt str
 gboolean multitrack_end (GtkMenuItem *, gpointer mt);
 
 // morph the poly window
-void polymorph (lives_mt *, gshort poly);
+void polymorph (lives_mt *, lives_mt_poly_state_t poly);
 
 /// sens/desens
 void mt_desensitise (lives_mt *);
@@ -657,9 +697,7 @@ void mt_render_aud_toggled (GtkMenuItem *, gpointer mt);
 void mt_fplay_toggled (GtkMenuItem *, gpointer mt);
 void mt_change_vals_activate (GtkMenuItem *, gpointer mt);
 void on_set_pvals_clicked  (GtkWidget *button, gpointer mt);
-void on_snap_over_changed (GtkMenuItem *, gpointer mt);
 void on_move_fx_changed (GtkMenuItem *, gpointer mt);
-void on_fx_auto_prev_changed (GtkMenuItem *, gpointer mt);
 void select_all_time (GtkMenuItem *, gpointer mt);
 void select_from_zero_time (GtkMenuItem *, gpointer mt);
 void select_to_end_time (GtkMenuItem *, gpointer mt);
@@ -710,9 +748,9 @@ gboolean on_track_move (GtkWidget *widget, GdkEventMotion *event, gpointer mt);
 gboolean on_track_header_move (GtkWidget *widget, GdkEventMotion *event, gpointer mt);
 
 void unselect_all (lives_mt *); // unselect all blocks
-void insert_frames (gint filenum, weed_timecode_t offset_start, weed_timecode_t offset_end, weed_timecode_t tc, gshort direction, GtkWidget *eventbox, lives_mt *, track_rect *in_block);
-void insert_audio (gint filenum, weed_timecode_t offset_start, weed_timecode_t offset_end, weed_timecode_t tc, gdouble avel, gshort direction, GtkWidget *eventbox, lives_mt *, track_rect *in_block);
-void on_seltrack_toggled (GtkToggleButton *, gpointer mt);
+void insert_frames (gint filenum, weed_timecode_t offset_start, weed_timecode_t offset_end, weed_timecode_t tc, lives_direction_t direction, GtkWidget *eventbox, lives_mt *, track_rect *in_block);
+void insert_audio (gint filenum, weed_timecode_t offset_start, weed_timecode_t offset_end, weed_timecode_t tc, gdouble avel, lives_direction_t direction, GtkWidget *eventbox, lives_mt *, track_rect *in_block);
+void on_seltrack_toggled (GtkWidget *, gpointer mt);
 void scroll_track_by_scrollbar (GtkVScrollbar *sbar, gpointer mt);
 
 // block functions
@@ -745,7 +783,7 @@ void mt_clear_timeline(lives_mt *mt);
 
 // context box text
 void clear_context (lives_mt *);
-void add_context_label (lives_mt *, gchar *text);
+void add_context_label (lives_mt *, const gchar *text);
 void mouse_mode_context(lives_mt *);
 void do_sel_context (lives_mt *);
 void do_fx_list_context (lives_mt *, gint fxcount);
@@ -753,6 +791,7 @@ void do_fx_move_context(lives_mt *mt);
 
 // playback / animation
 void multitrack_playall (lives_mt *);
+void multitrack_play_sel (GtkMenuItem *, gpointer mt);
 void animate_multitrack (lives_mt *);
 void unpaint_line(lives_mt *, GtkWidget *eventbox);
 void unpaint_lines(lives_mt *);
@@ -802,9 +841,9 @@ weed_plant_t *load_event_list(lives_mt *, gchar *eload_file);
 GList *load_layout_map(void);
 void add_markers(lives_mt *, weed_plant_t *event_list);
 void remove_markers(weed_plant_t *event_list);
-void save_layout_map (int *lmap, double *lmap_audio, gchar *file, gchar *dir);
+void save_layout_map (int *lmap, double *lmap_audio, const gchar *file, const gchar *dir);
 
-void migrate_layouts (gchar *old_set_name, gchar *new_set_name);
+void migrate_layouts (const gchar *old_set_name, const gchar *new_set_name);
 
 GList *layout_frame_is_affected(gint clipno, gint frame);
 GList *layout_audio_is_affected(gint clipno, gdouble time);
@@ -827,10 +866,15 @@ void write_backup_layout_numbering(lives_mt *);
 void mouse_select_end(GtkWidget *, lives_mt *);
 
 
+// amixer funcs
+void amixer_show (GtkButton *, gpointer mt);
+void on_amixer_close_clicked (GtkButton *, lives_mt *mt);
+GtkWidget * amixer_add_channel_slider (lives_mt *, gint i);
+
+
 // misc
 void mt_change_disp_tracks_ok (GtkButton *, gpointer user_data);
 void mt_swap_play_pause (lives_mt *, gboolean put_pause);
-void amixer_show (GtkButton *, gpointer user_data);
 gchar *set_values_from_defs(lives_mt *, gboolean from_prefs);
 
 
@@ -839,20 +883,22 @@ void mt_clip_select (lives_mt *, gboolean scroll);
 void mt_delete_clips(lives_mt *, gint file);
 void mt_init_clips (lives_mt *, gint orig_file, gboolean add);
 
-/* default to warn about */
-#define LMAP_ERROR_MISSING_CLIP 1
-#define LMAP_ERROR_CLOSE_FILE 2
-#define LMAP_ERROR_DELETE_FRAMES 3
-#define LMAP_ERROR_DELETE_AUDIO 6
 
-/*non-default*/
-#define LMAP_ERROR_SHIFT_FRAMES 4
-#define LMAP_ERROR_ALTER_FRAMES 5
-#define LMAP_ERROR_SHIFT_AUDIO 7
-#define LMAP_ERROR_ALTER_AUDIO 8
+typedef enum {
+  /* default to warn about */
+  LMAP_ERROR_MISSING_CLIP=1,
+  LMAP_ERROR_CLOSE_FILE=2,
+  LMAP_ERROR_DELETE_FRAMES=3,
+  LMAP_ERROR_DELETE_AUDIO=4,
 
-/* info */
-#define LMAP_INFO_SETNAME_CHANGED 16
+  /*non-default*/
+  LMAP_ERROR_SHIFT_FRAMES=65,
+  LMAP_ERROR_ALTER_FRAMES=66,
+  LMAP_ERROR_SHIFT_AUDIO=67,
+  LMAP_ERROR_ALTER_AUDIO=68,
 
+  /* info */
+  LMAP_INFO_SETNAME_CHANGED=1024
+} lives_lmap_error_t;
 
 #endif
