@@ -8,6 +8,7 @@
 #include <errno.h>
 
 #include "../libweed/weed.h"
+#include "../libweed/weed-palettes.h"
 #include "../libweed/weed-effects.h"
 #include "../libweed/weed-utils.h"
 #include "../libweed/weed-host.h"
@@ -27,7 +28,7 @@ static gboolean list_plugins;
 // command-line plugins
 
 
-static GList *get_plugin_result (gchar *command, gchar *delim, gboolean allow_blanks) {
+static GList *get_plugin_result (const gchar *command, const gchar *delim, gboolean allow_blanks) {
   gchar **array;
   gint bytes=0,pieces;
   int outfile_fd,i;
@@ -153,7 +154,7 @@ plugin_request_by_space (const gchar *plugin_type, const gchar *plugin_name, con
 
 
 GList *
-plugin_request_common (const gchar *plugin_type, const gchar *plugin_name, const gchar *request, gchar *delim, gboolean allow_blanks) {
+plugin_request_common (const gchar *plugin_type, const gchar *plugin_name, const gchar *request, const gchar *delim, gboolean allow_blanks) {
   // returns a GList of responses to -request, or NULL on error
   // by_line says whether we split on '\n' or on '|'
   GList *reslist=NULL;
@@ -192,7 +193,7 @@ plugin_request_common (const gchar *plugin_type, const gchar *plugin_name, const
 //////////////////
 // get list of plugins of various types
 
-GList *get_plugin_list (gchar *plugin_type, gboolean allow_nonex, gchar *plugdir, gchar *filter_ext) {
+GList *get_plugin_list (const gchar *plugin_type, gboolean allow_nonex, const gchar *plugdir, const gchar *filter_ext) {
   // returns a GList * of plugins of type plugin_type
   // returns empty list if there are no plugins of that type
 
@@ -204,7 +205,7 @@ GList *get_plugin_list (gchar *plugin_type, gboolean allow_nonex, gchar *plugdir
   gchar *com,*tmp;
   GList *pluglist;
 
-  gchar *ext=(filter_ext==NULL)?"":filter_ext;
+  const gchar *ext=(filter_ext==NULL)?"":filter_ext;
 
   if (!strcmp(plugin_type,PLUGIN_THEMES)) {
     com=g_strdup_printf ("smogrify list_plugins 0 1 \"%s%s\" \"\"",prefs->prefix_dir,THEME_DIR);
@@ -1318,7 +1319,7 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
 
   if (user_audio&&future_prefs->encoder.of_allowed_acodecs==0) best_arate=-1;
 
-  if (!strcmp(prefs->encoder.of_restrict,"none")&&best_arate>-1) {
+  if ((strlen(prefs->encoder.of_restrict)==0||!strcmp(prefs->encoder.of_restrict,"none"))&&best_arate>-1) {
     return TRUE;
   }
 
@@ -1333,42 +1334,67 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
     asampsize=rdet->asamps;
   }
 
-  pieces=get_token_count (prefs->encoder.of_restrict,',');
-  checks=g_strsplit(prefs->encoder.of_restrict,",",pieces);
 
 
-  for (r=0;r<pieces;r++) {
-    // check each restriction in turn
 
-    if (!strncmp (checks[r],"fps=",4)) {
-      gdouble allowed_fps;
-      gint mbest_num=0,mbest_denom=0;
-      gint numparts;
-      gchar *fixer;
 
-      best_fps_delta=1000000000.;
-      array=g_strsplit(checks[r],"=",2);
-      numtok=get_token_count (array[1],';');
-      array2=g_strsplit(array[1],";",numtok);
-      for (i=0;i<numtok;i++) {
-	mbest_num=mbest_denom=0;
-	if ((numparts=get_token_count (array2[i],':'))>1) {
-	  gchar **array3=g_strsplit(array2[i],":",2);
-	  mbest_num=atoi (array3[0]);
-	  mbest_denom=atoi (array3[1]);
-	  g_strfreev (array3);
-	  if (mbest_denom==0) continue;
-	  allowed_fps=(mbest_num*1.)/(mbest_denom*1.);
-	}
-	else allowed_fps=g_strtod (array2[i],NULL);
 
-	// convert to 8dp
-	fixer=g_strdup_printf ("%.8f %.8f",allowed_fps,fps);
-	g_free (fixer);
 
-	if (allowed_fps>=fps) {
-	  if (allowed_fps-fps<best_fps_delta) {
-	    best_fps_delta=allowed_fps-fps;
+  if (strlen(prefs->encoder.of_restrict)>0) {
+
+    pieces=get_token_count (prefs->encoder.of_restrict,',');
+    checks=g_strsplit(prefs->encoder.of_restrict,",",pieces);
+    
+
+    for (r=0;r<pieces;r++) {
+      // check each restriction in turn
+      
+      if (!strncmp (checks[r],"fps=",4)) {
+	gdouble allowed_fps;
+	gint mbest_num=0,mbest_denom=0;
+	gint numparts;
+	gchar *fixer;
+	
+	best_fps_delta=1000000000.;
+	array=g_strsplit(checks[r],"=",2);
+	numtok=get_token_count (array[1],';');
+	array2=g_strsplit(array[1],";",numtok);
+	for (i=0;i<numtok;i++) {
+	  mbest_num=mbest_denom=0;
+	  if ((numparts=get_token_count (array2[i],':'))>1) {
+	    gchar **array3=g_strsplit(array2[i],":",2);
+	    mbest_num=atoi (array3[0]);
+	    mbest_denom=atoi (array3[1]);
+	    g_strfreev (array3);
+	    if (mbest_denom==0) continue;
+	    allowed_fps=(mbest_num*1.)/(mbest_denom*1.);
+	  }
+	  else allowed_fps=g_strtod (array2[i],NULL);
+	  
+	  // convert to 8dp
+	  fixer=g_strdup_printf ("%.8f %.8f",allowed_fps,fps);
+	  g_free (fixer);
+	  
+	  if (allowed_fps>=fps) {
+	    if (allowed_fps-fps<best_fps_delta) {
+	      best_fps_delta=allowed_fps-fps;
+	      if (mbest_denom>0) {
+		best_fps_num=mbest_num;
+		best_fps_denom=mbest_denom;
+		best_fps=0.;
+		if (rdet==NULL) cfile->ratio_fps=TRUE;
+		else rdet->ratio_fps=TRUE;
+	      }
+	      else {
+		best_fps_num=best_fps_denom=0;
+		best_fps=allowed_fps;
+		if (rdet==NULL) cfile->ratio_fps=FALSE;
+		else rdet->ratio_fps=FALSE;
+	      }
+	    }
+	  }
+	  else if ((best_fps_denom==0&&allowed_fps>best_fps)||(best_fps_denom>0&&allowed_fps>(best_fps_num*1.)/(best_fps_denom*1.))) {
+	    best_fps_delta=fps-allowed_fps;
 	    if (mbest_denom>0) {
 	      best_fps_num=mbest_num;
 	      best_fps_denom=mbest_denom;
@@ -1377,171 +1403,157 @@ gboolean check_encoder_restrictions (gboolean get_extension, gboolean user_audio
 	      else rdet->ratio_fps=TRUE;
 	    }
 	    else {
-	      best_fps_num=best_fps_denom=0;
 	      best_fps=allowed_fps;
+	      best_fps_num=best_fps_denom=0;
 	      if (rdet==NULL) cfile->ratio_fps=FALSE;
 	      else rdet->ratio_fps=FALSE;
 	    }
 	  }
-	}
-	else if ((best_fps_denom==0&&allowed_fps>best_fps)||(best_fps_denom>0&&allowed_fps>(best_fps_num*1.)/(best_fps_denom*1.))) {
-	  best_fps_delta=fps-allowed_fps;
-	  if (mbest_denom>0) {
-	    best_fps_num=mbest_num;
-	    best_fps_denom=mbest_denom;
-	    best_fps=0.;
-	    if (rdet==NULL) cfile->ratio_fps=TRUE;
-	    else rdet->ratio_fps=TRUE;
-	  }
-	  else {
-	    best_fps=allowed_fps;
-	    best_fps_num=best_fps_denom=0;
-	    if (rdet==NULL) cfile->ratio_fps=FALSE;
-	    else rdet->ratio_fps=FALSE;
+	  if (best_fps_delta<(.0005*prefs->ignore_tiny_fps_diffs)) {
+	    best_fps_delta=0.;
+	    best_fps_denom=best_fps_num=0;
 	  }
+	  if (best_fps_delta==0.) break;
 	}
-	if (best_fps_delta<(.0005*prefs->ignore_tiny_fps_diffs)) {
-	  best_fps_delta=0.;
-	  best_fps_denom=best_fps_num=0;
-	}
-	if (best_fps_delta==0.) break;
+	g_strfreev(array);
+	g_strfreev(array2);
+	continue;
       }
-      g_strfreev(array);
-      g_strfreev(array2);
-      continue;
-    }
-
-    if (!strncmp (checks[r],"size=",5)) {
-      // TODO - allow list for size
-      array=g_strsplit(checks[r],"=",2);
-      array2=g_strsplit(array[1],"x",2);
-      width=atoi (array2[0]);
-      height=atoi (array2[1]);
-      g_strfreev(array2);
-      g_strfreev(array);
-      sizer=TRUE;
-      continue;
-    }
-
-
-
-    if (!strncmp (checks[r],"asigned=",8)&&((mainw->save_with_sound||rdet!=NULL)&&(resaudw==NULL||resaudw->aud_checkbutton==NULL||gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->aud_checkbutton))))&&prefs->encoder.audio_codec!=AUDIO_CODEC_NONE&&(arate*achans*asampsize)) {
-      array=g_strsplit(checks[r],"=",2);
-      if (!strcmp(array[1],"signed")) {
-	asigned=1;
-      }
-
-      if (!strcmp(array[1],"unsigned")) {
-	asigned=2;
-      }
-
-      g_strfreev(array);
       
-      if (asigned!=0&&!capable->has_sox) {
-	do_encoder_sox_error();
-	g_strfreev(checks);
-	return FALSE;
+      if (!strncmp (checks[r],"size=",5)) {
+	// TODO - allow list for size
+	array=g_strsplit(checks[r],"=",2);
+	array2=g_strsplit(array[1],"x",2);
+	width=atoi (array2[0]);
+	height=atoi (array2[1]);
+	g_strfreev(array2);
+	g_strfreev(array);
+	sizer=TRUE;
+	continue;
+      }
+      
+      
+      
+      if (!strncmp (checks[r],"asigned=",8)&&((mainw->save_with_sound||rdet!=NULL)&&(resaudw==NULL||resaudw->aud_checkbutton==NULL||gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->aud_checkbutton))))&&prefs->encoder.audio_codec!=AUDIO_CODEC_NONE&&(arate*achans*asampsize)) {
+	array=g_strsplit(checks[r],"=",2);
+	if (!strcmp(array[1],"signed")) {
+	  asigned=1;
+	}
+	
+	if (!strcmp(array[1],"unsigned")) {
+	  asigned=2;
+	}
+	
+	g_strfreev(array);
+      
+	if (asigned!=0&&!capable->has_sox) {
+	  do_encoder_sox_error();
+	  g_strfreev(checks);
+	  return FALSE;
+	}
+	continue;
+      }
+      
+      
+      if (!strncmp (checks[r],"arate=",6)&&((mainw->save_with_sound||rdet!=NULL)&&(resaudw==NULL||resaudw->aud_checkbutton==NULL||gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->aud_checkbutton))))&&prefs->encoder.audio_codec!=AUDIO_CODEC_NONE&&(arate*achans*asampsize)) {
+	// we only perform this test if we are encoding with audio
+	// find next highest allowed rate from list,
+	// if none are higher, use the highest
+	gint allowed_arate;
+	best_arate_delta=1000000000;
+	
+	array=g_strsplit(checks[r],"=",2);
+	numtok=get_token_count (array[1],';');
+	array2=g_strsplit(array[1],";",numtok);
+	for (i=0;i<numtok;i++) {
+	  allowed_arate=atoi (array2[i]);
+	  if (allowed_arate>=arate) {
+	    if (allowed_arate-arate<best_arate_delta) {
+	      best_arate_delta=allowed_arate-arate;
+	      best_arate=allowed_arate;
+	    }
+	  }
+	  else if (allowed_arate>best_arate) best_arate=allowed_arate;
+	}
+	g_strfreev(array2);
+	g_strfreev(array);
+	
+	if (!capable->has_sox) {
+	  do_encoder_sox_error();
+	  g_strfreev(checks);
+	  return FALSE;
+	}
+	continue;
+      }
+      
+      if (!strncmp (checks[r],"hblock=",7)) {
+	// width must be a multiple of this
+	array=g_strsplit(checks[r],"=",2);
+	hblock=atoi (array[1]);
+	width=(gint)(width/hblock+.5)*hblock;
+	g_strfreev(array);
+	continue;
+      }
+      
+      if (!strncmp (checks[r],"vblock=",7)) {
+	// height must be a multiple of this
+	array=g_strsplit(checks[r],"=",2);
+	vblock=atoi (array[1]);
+	height=(gint)(height/vblock+.5)*vblock;
+	g_strfreev(array);
+	continue;
+      }
+      
+      if (!strncmp (checks[r],"aspect=",7)) {
+	// we calculate the nearest smaller frame size using aspect, 
+	// hblock and vblock
+	calc_aspect=TRUE;
+	array=g_strsplit(checks[r],"=",2);
+	g_snprintf (aspect_buffer,512,"%s",array[1]);
+	g_strfreev(array);
+	continue;
       }
-      continue;
     }
-
-
-    if (!strncmp (checks[r],"arate=",6)&&((mainw->save_with_sound||rdet!=NULL)&&(resaudw==NULL||resaudw->aud_checkbutton==NULL||gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resaudw->aud_checkbutton))))&&prefs->encoder.audio_codec!=AUDIO_CODEC_NONE&&(arate*achans*asampsize)) {
-      // we only perform this test if we are encoding with audio
-      // find next highest allowed rate from list,
-      // if none are higher, use the highest
-      gint allowed_arate;
-      best_arate_delta=1000000000;
-
-      array=g_strsplit(checks[r],"=",2);
-      numtok=get_token_count (array[1],';');
-      array2=g_strsplit(array[1],";",numtok);
+    
+    /// end restrictions
+    g_strfreev(checks);
+    
+    if (!mainw->osc_auto&&calc_aspect&&!sizer) {
+      // we calculate this last, after getting hblock and vblock sizes
+      gchar **array3;
+      gdouble allowed_aspect;
+      gint xwidth=width;
+      gint xheight=height;
+      
+      width=height=1000000;
+      
+      numtok=get_token_count (aspect_buffer,';');
+      array2=g_strsplit(aspect_buffer,";",numtok);
+    
+      // see if we can get a width:height which is nearer an aspect than 
+      // current width:height
+      
       for (i=0;i<numtok;i++) {
-	allowed_arate=atoi (array2[i]);
-	if (allowed_arate>=arate) {
-	  if (allowed_arate-arate<best_arate_delta) {
-	    best_arate_delta=allowed_arate-arate;
-	    best_arate=allowed_arate;
-	  }
-	}
-	else if (allowed_arate>best_arate) best_arate=allowed_arate;
+	array3=g_strsplit(array2[i],":",2);
+	allowed_aspect=g_strtod(array3[0],NULL)/g_strtod(array3[1],NULL);
+	g_strfreev(array3);
+	minimise_aspect_delta (allowed_aspect,hblock,vblock,xwidth,xheight,&width,&height);
       }
       g_strfreev(array2);
-      g_strfreev(array);
       
-      if (!capable->has_sox) {
-	do_encoder_sox_error();
-	g_strfreev(checks);
-	return FALSE;
-      }
-      continue;
-    }
-
-    if (!strncmp (checks[r],"hblock=",7)) {
-      // width must be a multiple of this
-      array=g_strsplit(checks[r],"=",2);
-      hblock=atoi (array[1]);
-      width=(gint)(width/hblock+.5)*hblock;
-      g_strfreev(array);
-      continue;
+      // allow override if current width and height are integer multiples of blocks
+      if (owidth%hblock==0&&oheight%vblock==0) allow_aspect_override=TRUE;
+      
+      // end recheck
     }
     
-    if (!strncmp (checks[r],"vblock=",7)) {
-      // height must be a multiple of this
-      array=g_strsplit(checks[r],"=",2);
-      vblock=atoi (array[1]);
-      height=(gint)(height/vblock+.5)*vblock;
-      g_strfreev(array);
-      continue;
-    }
+    // fps can't be altered if we have a multitrack event_list
+    if (mainw->multitrack!=NULL&&mainw->multitrack->event_list!=NULL) best_fps_delta=0.;
     
-    if (!strncmp (checks[r],"aspect=",7)) {
-      // we calculate the nearest smaller frame size using aspect, 
-      // hblock and vblock
-      calc_aspect=TRUE;
-      array=g_strsplit(checks[r],"=",2);
-      g_snprintf (aspect_buffer,512,"%s",array[1]);
-      g_strfreev(array);
-      continue;
-    }
-  }
-  
-  /// end restrictions
-  g_strfreev(checks);
-
-  if (!mainw->osc_auto&&calc_aspect&&!sizer) {
-    // we calculate this last, after getting hblock and vblock sizes
-    gchar **array3;
-    gdouble allowed_aspect;
-    gint xwidth=width;
-    gint xheight=height;
-
-    width=height=1000000;
-
-    numtok=get_token_count (aspect_buffer,';');
-    array2=g_strsplit(aspect_buffer,";",numtok);
+    if (sizer) allow_aspect_override=FALSE;
     
-    // see if we can get a width:height which is nearer an aspect than 
-    // current width:height
-
-    for (i=0;i<numtok;i++) {
-      array3=g_strsplit(array2[i],":",2);
-      allowed_aspect=g_strtod(array3[0],NULL)/g_strtod(array3[1],NULL);
-      g_strfreev(array3);
-      minimise_aspect_delta (allowed_aspect,hblock,vblock,xwidth,xheight,&width,&height);
-    }
-    g_strfreev(array2);
-
-    // allow override if current width and height are integer multiples of blocks
-    if (owidth%hblock==0&&oheight%vblock==0) allow_aspect_override=TRUE;
-
-    // end recheck
   }
 
-  // fps can't be altered if we have a multitrack event_list
-  if (mainw->multitrack!=NULL&&mainw->multitrack->event_list!=NULL) best_fps_delta=0.;
-
-  if (sizer) allow_aspect_override=FALSE;
 
   if (((width!=owidth||height!=oheight)&&width*height>0)||(best_fps_delta>0.)||(best_arate_delta>0&&best_arate>0)||best_arate<0||asigned!=0) {
     gboolean ofx1_bool=mainw->fx1_bool;
@@ -1655,21 +1667,63 @@ GList *filter_encoders_by_img_ext(GList *encoders, const gchar *img_ext) {
 // decoder plugins
 
 
-const lives_clip_data_t *get_decoder_plugin(file *sfile) {
-  // pass file to each decoder plugin in turn, until we find one that can parse
+LIVES_INLINE gboolean decplugin_supports_palette (const lives_decoder_t *dplug, int palette) {
+  register int i=0;
+  int cpal;
+  while ((cpal=dplug->cdata->palettes[i++])!=WEED_PALETTE_END) if (cpal==palette) return TRUE;
+  return FALSE;
+}
+
+
+
+GList *load_decoders(void) {
+  lives_decoder_sys_t *dplug;
+  gchar *decplugdir=g_strdup_printf("%s%s%s",prefs->lib_dir,PLUGIN_EXEC_DIR,PLUGIN_DECODERS);
+  GList *dlist=NULL;
+  GList *decoder_plugins_o=get_plugin_list (PLUGIN_DECODERS,TRUE,decplugdir,"-so"),*decoder_plugins=decoder_plugins_o;
+  g_free(decplugdir);
+
+  while (decoder_plugins!=NULL) {
+    dplug=open_decoder_plugin((gchar *)decoder_plugins->data);
+    g_free(decoder_plugins->data);
+    if (dplug!=NULL) dlist=g_list_append(dlist,(gpointer)dplug);
+    decoder_plugins=decoder_plugins->next;
+  }
+
+  g_list_free(decoder_plugins_o);
+
+  return dlist;
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+const lives_clip_data_t *get_decoder_cdata(file *sfile) {
+  // pass file to each decoder (demuxer) plugin in turn, until we find one that can parse
   // the file
   // NULL is returned if no decoder plugin recognises the file - then we
   // fall back to other methods
 
   // otherwise we return data for the clip as supplied by the decoder plugin
 
-  // if the file does not exist, we set mainw->error=TRUE and return NULL
+  // If the file does not exist, we set mainw->error=TRUE and return NULL
+
+  // If we find a plugin we also set sfile->ext_src to point to a newly created decoder_plugin_t
+
 
   gchar *tmp=NULL;
-  const lives_clip_data_t *cdata=NULL;
-  GList *decoder_plugins;
-  gchar *decplugdir,*msg;
-  _decoder_plugin *dplug;
+  GList *decoder_plugin;
+  gchar *msg;
+  lives_decoder_t *dplug=NULL;
 
   mainw->error=FALSE;
 
@@ -1683,88 +1737,77 @@ const lives_clip_data_t *get_decoder_plugin(file *sfile) {
 
   sfile->ext_src=NULL;
 
-  decplugdir=g_strdup_printf("%s%s%s",prefs->lib_dir,PLUGIN_EXEC_DIR,PLUGIN_DECODERS);
-  decoder_plugins=get_plugin_list (PLUGIN_DECODERS,TRUE,decplugdir,"-so");
-  g_free(decplugdir);
+  lives_set_cursor_style(LIVES_CURSOR_BUSY,NULL);
 
-  while (decoder_plugins!=NULL) {
-    gchar *decplugname=g_strdup_printf("%s.so",(gchar *)decoder_plugins->data);
-    dplug=open_decoder_plugin(decplugname,sfile);
+  if (!mainw->decoders_loaded) {
+    mainw->decoder_list=load_decoders();
+    mainw->decoders_loaded=TRUE;
+  }
+
+  decoder_plugin=mainw->decoder_list;
+
+  dplug=(lives_decoder_t *)g_malloc(sizeof(lives_decoder_t));
+
+  while (decoder_plugin!=NULL) {
+    lives_decoder_sys_t *dpsys=(lives_decoder_sys_t *)decoder_plugin->data;
 
     //#define DEBUG_DECPLUG
 #ifdef DEBUG_DECPLUG
-    g_print("trying decoder %s\n",decplugname);
+    g_print("trying decoder %s\n",dpsys->name);
 #endif
 
-    g_free(decplugname);
-
-    if (dplug!=NULL) {
-      if ((cdata=(dplug->get_clip_data)((tmp=(char *)g_filename_from_utf8 (sfile->file_name,-1,NULL,NULL,NULL))))!=NULL) {
-	g_free(tmp);
-	sfile->ext_src=dplug;
-	break;
-      }
+    if ((dplug->cdata=(dpsys->get_clip_data)((tmp=(char *)g_filename_from_utf8 (sfile->file_name,-1,NULL,NULL,NULL)),
+					     NULL))!=NULL) {
       g_free(tmp);
-      close_decoder_plugin(sfile,dplug);
+      dplug->decoder=dpsys;
+      sfile->ext_src=dplug;
+      if (strncmp(dpsys->name,"libzz",5)) {
+	mainw->decoder_list=g_list_move_to_first(mainw->decoder_list, decoder_plugin);
+      }
+      break;
     }
-    decoder_plugins=decoder_plugins->next;
+    g_free(tmp);
+    decoder_plugin=decoder_plugin->next;
   }
 
-  g_list_free_strings(decoder_plugins);
-  g_list_free(decoder_plugins);
+  lives_set_cursor_style(LIVES_CURSOR_NORMAL,NULL);
 
   if (sfile->ext_src!=NULL) {
+    dplug=sfile->ext_src;
+    msg=g_strdup_printf(" :: using decoder plugin %s",(dplug->decoder->name));
+    d_print(msg);
+    g_free(msg);
 
-    if (cdata!=NULL) {
-
-      // if plugin allows choice of palettes, we must set one straight away
-      // for now we just use the plugin's preferred palette
-
-      dplug=(_decoder_plugin *)(sfile->ext_src);
-
-      dplug->current_palette=dplug->preferred_palette=cdata->palettes[0];
-      if (dplug->set_palette!=NULL) {
-	if (!(dplug->set_palette)(dplug->current_palette)) {
-	  do_decoder_palette_error();
-	  close_decoder_plugin(sfile,sfile->ext_src);
-	  sfile->ext_src=NULL;
-	  return NULL;
-	}
-      }
-
-      ((_decoder_plugin *)(sfile->ext_src))->YUV_sampling=cdata->YUV_sampling;
-      ((_decoder_plugin *)(sfile->ext_src))->YUV_clamping=cdata->YUV_clamping;
-      ((_decoder_plugin *)(sfile->ext_src))->YUV_subspace=cdata->YUV_subspace;
-      ((_decoder_plugin *)(sfile->ext_src))->interlace=cdata->interlace;
-
-      msg=g_strdup_printf(" :: using decoder plugin %s",((_decoder_plugin *)(sfile->ext_src))->name);
-      d_print(msg);
-      g_free(msg);
-
-    }
-    else {
-      close_decoder_plugin(sfile,sfile->ext_src);
-      sfile->ext_src=NULL;
-      
-    }
+    return dplug->cdata;
   }
-  return cdata;
+
+  return NULL;
 }
 
 
 
 
-void close_decoder_plugin (file *sfile, _decoder_plugin *dplug) {
+
+
+// close one instance of dplug
+void close_decoder_plugin (lives_decoder_t *dplug) {
+  lives_clip_data_t *cdata;
+
   if (dplug==NULL) return;
 
+  cdata=dplug->cdata;
+
+  if (cdata!=NULL) (*dplug->decoder->clip_data_free)(cdata);
+
+  g_free(dplug);
+
+}
+
+
+static void unload_decoder_plugin(lives_decoder_sys_t *dplug) {
   if (dplug->module_unload!=NULL) (*dplug->module_unload)();
 
   if (dplug->name!=NULL) {
-    if (sfile!=NULL) {
-      gchar *com=g_strdup_printf("/bin/rm -f %s/%s/%s",prefs->tmpdir,sfile->handle,dplug->name);
-      dummyvar=system(com);
-      g_free(com);
-    }
     g_free(dplug->name);
   }
 
@@ -1773,33 +1816,48 @@ void close_decoder_plugin (file *sfile, _decoder_plugin *dplug) {
 }
 
 
-_decoder_plugin *open_decoder_plugin(const gchar *plname, file *sfile) {
+void unload_decoder_plugins(void) {
+  GList *dplugs=mainw->decoder_list;
 
-  _decoder_plugin *dplug=(_decoder_plugin *)g_malloc(sizeof(_decoder_plugin));
-  gchar *plugname=g_strdup_printf ("%s%s%s/%s",prefs->lib_dir,PLUGIN_EXEC_DIR,PLUGIN_DECODERS,plname);
-  gchar *nplugname=g_strdup_printf("%s/%s/%s",prefs->tmpdir,sfile->handle,plname);
+  while (dplugs!=NULL) {
+    unload_decoder_plugin(dplugs->data);
+    dplugs=dplugs->next;
+  }
+
+  g_list_free(mainw->decoder_list);
+  mainw->decoder_list=NULL;
+  mainw->decoders_loaded=FALSE;
+}
+
+
+
+lives_decoder_sys_t *open_decoder_plugin(const gchar *plname) {
+  lives_decoder_sys_t *dplug;
+
+  gchar *plugname=g_strdup_printf ("%s%s%s/%s.so",prefs->lib_dir,PLUGIN_EXEC_DIR,PLUGIN_DECODERS,plname);
   gboolean OK=TRUE;
   const gchar *err;
 
-  gchar *com=g_strdup_printf("/bin/cp %s %s",plugname,nplugname); // copy dl into clip directory - so we get a fresh copy for each file
-  dummyvar=system(com);
-  g_free(com);
-  g_free(plugname);
+  if (!strcmp(plname,"ogg_theora_decoder")) {
+    // no longer compatible
+    return NULL;
+  }
 
-  dplug->handle=dlopen(nplugname,RTLD_LAZY);
-  g_free(nplugname);
+  dplug=(lives_decoder_sys_t *)g_malloc(sizeof(lives_decoder_sys_t));
+
+  dplug->name=NULL;
+
+  dplug->handle=dlopen(plugname,RTLD_LAZY);
+  g_free (plugname);
 
   if (dplug->handle==NULL) {
     gchar *msg=g_strdup_printf (_("\n\nFailed to open decoder plugin %s\nError was %s\n"),plname,dlerror());
     d_print(msg);
     g_free (msg);
-    g_free(dplug);
+    g_free (dplug);
     return NULL;
   }
 
-  if ((dplug->module_check_init=dlsym (dplug->handle,"module_check_init"))==NULL) {
-    OK=FALSE;
-  }
   if ((dplug->version=dlsym (dplug->handle,"version"))==NULL) {
     OK=FALSE;
   }
@@ -1809,30 +1867,34 @@ _decoder_plugin *open_decoder_plugin(const gchar *plname, file *sfile) {
   if ((dplug->get_frame=dlsym (dplug->handle,"get_frame"))==NULL) {
     OK=FALSE;
   }
+  if ((dplug->clip_data_free=dlsym (dplug->handle,"clip_data_free"))==NULL) {
+    OK=FALSE;
+  }
 
   if (!OK) {
     gchar *msg=g_strdup_printf (_("\n\nDecoder plugin %s\nis missing a mandatory function.\nUnable to use it.\n"),plname);
     d_print(msg);
     g_free (msg);
-    close_decoder_plugin(0,dplug);
+    unload_decoder_plugin(dplug);
     g_free(dplug);
     return NULL;
   }
 
   // optional
+  dplug->module_check_init=dlsym (dplug->handle,"module_check_init");
   dplug->module_unload=dlsym (dplug->handle,"module_unload");
-  dplug->set_palette=dlsym (dplug->handle,"set_palette");
   dplug->rip_audio=dlsym (dplug->handle,"rip_audio");
   dplug->rip_audio_cleanup=dlsym (dplug->handle,"rip_audio_cleanup");
 
-  err=(*dplug->module_check_init)();
-
-  if (err!=NULL) {
-    g_snprintf(mainw->msg,512,"%s",err);
-    // TODO
-    close_decoder_plugin(0,dplug);
-    g_free(dplug);
-    return NULL;
+  if (dplug->module_check_init!=NULL) {
+    err=(*dplug->module_check_init)();
+    
+    if (err!=NULL) {
+      g_snprintf(mainw->msg,512,"%s",err);
+      unload_decoder_plugin(dplug);
+      g_free(dplug);
+      return NULL;
+    }
   }
 
   dplug->name=g_strdup(plname);
@@ -1841,6 +1903,7 @@ _decoder_plugin *open_decoder_plugin(const gchar *plname, file *sfile) {
 
 
 
+
 void get_mime_type(gchar *text, int maxlen, const lives_clip_data_t *cdata) {
   gchar *audname;
   
@@ -2263,6 +2326,8 @@ void param_copy (lives_param_t *src, lives_param_t *dest, gboolean full) {
     set_int_param (dest->def,get_int_param (src->def));
     if (src->list!=NULL) dest->list=g_list_copy (src->list);
     break;
+  default:
+    break;
   }
   if (!full) return;
   // TODO - copy value, copy widgets
@@ -2295,14 +2360,14 @@ void get_colRGBA32_param(void *value, lives_colRGBA32_t *rgba) {
   w_memcpy(rgba,value,sizeof(lives_colRGBA32_t));
 }
 
-void set_bool_param(void *value, const gboolean _const) {
+void set_bool_param(void *value, gboolean _const) {
   set_int_param(value,!!_const);
 }
 
-void set_int_param(void *value, const gint _const) {
+void set_int_param(void *value, gint _const) {
   w_memcpy(value,&_const,sizint);
 }
-void set_double_param(void *value, const gdouble _const) {
+void set_double_param(void *value, gdouble _const) {
   w_memcpy(value,&_const,sizdbl);
 
 }
@@ -2338,7 +2403,7 @@ void set_colRGBA32_param(void *value, gshort red, gshort green, gshort blue, gsh
 
 
 
-gint find_rfx_plugin_by_name (gchar *name, gshort status) {
+gint find_rfx_plugin_by_name (const gchar *name, gshort status) {
   int i;
   for (i=1;i<mainw->num_rendered_effects_builtin+mainw->num_rendered_effects_custom+mainw->num_rendered_effects_test;i++) {
     if (mainw->rendered_fx[i].name!=NULL&&!strcmp (mainw->rendered_fx[i].name,name)&&mainw->rendered_fx[i].status==status) return (gint)i;
@@ -2382,6 +2447,8 @@ lives_param_t *weed_params_to_rfx(gint npar, weed_plant_t *plant, gboolean show_
 
     rpar[i].flags=flags;
 
+    gui=NULL;
+
     if (weed_plant_has_leaf(wtmpl,"gui")) gui=weed_get_plantptr_value(wtmpl,"gui",&error);
 
     rpar[i].group=0;
@@ -2805,7 +2872,7 @@ GList *get_external_window_hints(lives_rfx_t *rfx) {
 
 
 
-gchar *plugin_run_param_window(gchar *get_com, GtkVBox *vbox, lives_rfx_t **ret_rfx) {
+gchar *plugin_run_param_window(const gchar *get_com, GtkVBox *vbox, lives_rfx_t **ret_rfx) {
 
   // here we create an rfx script from some fixed values and values from the plugin; we will then compile the script to an rfx scrap and use the scrap to get info
   // about additional parameters, and create the parameter window
