@@ -1,6 +1,6 @@
 // plugins.h
 // LiVES
-// (c) G. Finch 2003-2006 <salsaman@xs4all.nl>
+// (c) G. Finch 2003-2010 <salsaman@xs4all.nl>
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
@@ -12,12 +12,11 @@
 #include <string.h>
 #include <fcntl.h>
 #include <stdlib.h>
-#include "main.h"
 
 
 // generic plugins
 
-GList *get_plugin_list (gchar *plugin_type, gboolean allow_nonex, gchar *plugdir, gchar *filter_ext);
+GList *get_plugin_list (const gchar *plugin_type, gboolean allow_nonex, const gchar *plugdir, const gchar *filter_ext);
 #define PLUGIN_ENCODERS "encoders"
 #define PLUGIN_DECODERS "decoders"
 #define PLUGIN_VID_PLAYBACK "playback/video"
@@ -35,7 +34,7 @@ GList *plugin_request (const gchar *plugin_type, const gchar *plugin_name, const
 GList *plugin_request_with_blanks (const gchar *plugin_type, const gchar *plugin_name, const gchar *request);
 GList *plugin_request_by_line (const gchar *plugin_type, const gchar *plugin_name, const gchar *request);
 GList *plugin_request_by_space (const gchar *plugin_type, const gchar *plugin_name, const gchar *request);
-GList *plugin_request_common (const gchar *plugin_type, const gchar *plugin_name, const gchar *request, gchar *delim, gboolean allow_blanks);
+GList *plugin_request_common (const gchar *plugin_type, const gchar *plugin_name, const gchar *request, const gchar *delim, gboolean allow_blanks);
 
 
 // video playback plugins
@@ -155,70 +154,131 @@ _encoder;
 
 // decoder plugins
 
-#define LIVES_INTERLACE_NONE 0
-#define LIVES_INTERLACE_BOTTOM_FIRST 1
-#define LIVES_INTERLACE_TOP_FIRST 2
-#define LIVES_INTERLACE_PROGRESSIVE 3
+
+
+
+
+typedef enum {
+  LIVES_INTERLACE_NONE=0,
+  LIVES_INTERLACE_BOTTOM_FIRST=1,
+  LIVES_INTERLACE_TOP_FIRST=2
+} lives_interlace_t;
+
+// seek_flags is a bitmap
+
+  // good
+#define LIVES_SEEK_FAST (1<<0)
+
+  // not so good
+#define LIVES_SEEK_NEEDS_CALCULATION (1<<1)
+#define LIVES_SEEK_QUALITY_LOSS (1<<2)
+
 
 typedef struct {
-  int width;
-  int height;
-  int nframes;
-  int interlace;
+  gchar *URI; // the URI of this cdata
 
+  gint nclips; // number of clips (titles) in container
+  gchar container_name[512]; // name of container, e.g. "ogg" or NULL
+
+  // plugin should init this to 0 if URI changes
+  gint current_clip; // current clip number in container (starts at 0, MUST be <= nclips) [rw host]
+
+  // video data
+  gint width;
+  gint height;
+  gint64 nframes;
+  lives_interlace_t interlace;
+
+  // x and y offsets of picture within frame
+  // for primary pixel plane
+  gint offs_x;
+  gint offs_y;
+  gint frame_width;
+  gint frame_height;
+
+  gfloat par; // pixel aspect ratio
+
+  gfloat fps;
+
+  int *palettes;
+
+  // plugin should init this to palettes[0] if URI changes
+  int current_palette;  // current palette [rw host]; must be contained in palettes
+  
   int YUV_sampling;
   int YUV_clamping;
   int YUV_subspace;
-  float fps;
-  int *palettes;
+  gchar video_name[512]; // name of video codec, e.g. "theora" or NULL
 
-  char container_name[512]; // name of container, e.g. "ogg"
-  char video_name[512]; // name of video codec, e.g. "theora"
-  char audio_name[512]; // name of audio codec, e.g. "vorbis"
+  /* audio data */
+  gint arate;
+  gint achans;
+  gint asamps;
+  gboolean asigned;
+  gboolean ainterleaf;
+  gchar audio_name[512]; // name of audio codec, e.g. "vorbis" or NULL
+
+  int seek_flag;
+
+  void *priv; // private data for demuxer/decoder - host should not touch this
 
-  int arate;
-  int achans;
-  int asamps;
-  int asigned;
-  int ainterleaf;
 } lives_clip_data_t;
 
 
 
-
 typedef struct {
   // playback
   gchar *name; // plugin name
-  void *handle;
+  void *handle; // may be shared between several instances
 
   // mandatory
-  const char *(*module_check_init)(void);
   const char *(*version) (void);
-  const lives_clip_data_t *(*get_clip_data)(char *URI);
-  gboolean (*get_frame)(char *URI, int64_t frame, void **pixel_data);
+
+  // call first time with NULL cdata
+  // subsequent calls should re-use cdata
+  // set cdata->current_clip > 0 to get data for clip n (0 <= n < cdata->nclips)
+  // we can also set cdata->current_palette (must be in list cdata->palettes[])
+
+  // if URI changes, current_clip and current_palette are reset by plugin
+
+  lives_clip_data_t *(*get_clip_data)(char *URI, lives_clip_data_t *cdata);
+
+  // frame starts at 0 in these functions
+  gboolean (*get_frame)(const lives_clip_data_t *, int64_t frame, void **pixel_data);
+
+  // call this for each cdata before unloading the module
+  void (*clip_data_free)(lives_clip_data_t *);
 
   // optional
-  gboolean (*set_palette)(int palette);
-  gboolean (*rip_audio) (char *URI, char *fname, int stframe, int frames);
-  void (*rip_audio_cleanup) (void);
+  const char *(*module_check_init)(void);
+  int64_t (*rip_audio) (const lives_clip_data_t *, const char *fname, int64_t stframe, int64_t nframes, 
+			unsigned char **abuff);
+  void (*rip_audio_cleanup) (const lives_clip_data_t *cdata);
   void (*module_unload)(void);
 
-  int preferred_palette;
-  int current_palette;
-  int interlace;
-  int YUV_sampling;
-  int YUV_clamping;
-  int YUV_subspace;
-
-  double gamma_needed;
-
-} _decoder_plugin;
+} lives_decoder_sys_t;
 
 
-const lives_clip_data_t *get_decoder_plugin(file *sfile);
-void close_decoder_plugin (file *sfile, _decoder_plugin *dplug);
-_decoder_plugin *open_decoder_plugin(const gchar *plname, file *sfile);
-void get_mime_type(gchar *text, int maxlen, const lives_clip_data_t *cdata);
+
+
+typedef struct {
+  const lives_decoder_sys_t *decoder;
+  lives_clip_data_t *cdata;
+} lives_decoder_t;
+
+
+
+
+
+
+const lives_clip_data_t *get_decoder_cdata(file *);
+void close_decoder_plugin (lives_decoder_t *);
+lives_decoder_sys_t *open_decoder_plugin(const gchar *plname);
+void get_mime_type(gchar *text, int maxlen, const lives_clip_data_t *);
+void unload_decoder_plugins(void);
+
+LIVES_INLINE gboolean decplugin_supports_palette (const lives_decoder_t *dplug, int palette);
+
 
 
 // RFX plugins
@@ -249,6 +309,21 @@ void get_mime_type(gchar *text, int maxlen, const lives_clip_data_t *cdata);
 #define RFX_MAXSTRINGLEN 1024
 
 
+typedef enum {
+
+  LIVES_PARAM_UNKNOWN=0,
+  LIVES_PARAM_NUM,
+  LIVES_PARAM_BOOL,
+  LIVES_PARAM_COLRGB24,
+  LIVES_PARAM_STRING,
+  LIVES_PARAM_STRING_LIST,
+  LIVES_PARAM_COLRGBA32,
+
+  LIVES_PARAM_UNDISPLAYABLE=65536
+  
+} lives_param_type_t;
+
+
 typedef struct {
   // weed style part
   gchar *name;
@@ -273,19 +348,8 @@ typedef struct {
 
   gboolean wrap;
   gint group;
-  gint type;
+  lives_param_type_t type;
 
-#define LIVES_PARAM_UNKNOWN 0
-#define LIVES_PARAM_NUM 1
-#define LIVES_PARAM_BOOL 2
-#define LIVES_PARAM_COLRGB24 3
-#define LIVES_PARAM_STRING 4
-#define LIVES_PARAM_STRING_LIST 5
-#define LIVES_PARAM_COLRGBA32 6
-
-
-#define LIVES_PARAM_UNDISPLAYABLE 65536
-  
   gint dp;  //decimals, 0 for int and bool
   void *value;  // current value(s)
 
@@ -315,23 +379,30 @@ typedef struct {
 } lives_param_t;
 
 
+typedef enum {
+  RFX_STATUS_BUILTIN=0, // factory presets
+  RFX_STATUS_CUSTOM=1, // custom effects in the custom menu
+  RFX_STATUS_TEST=2, // test effects in the advanced menu
+  RFX_STATUS_ANY=3, // indicates free choice of statuses
+  RFX_STATUS_WEED=4, // indicates an internal RFX, created from a weed instance
+  RFX_STATUS_SCRAP=5, // used for parsing RFX scraps from external apps
+
+  // these are only used when prompting for a name
+  RFX_STATUS_COPY=128, // indicates a copy operation to test
+  RFX_STATUS_RENAME=129 // indicates a copy operation to test
+} lives_rfx_status_t;
+
+
+
+
 typedef struct {
   gchar *name;  // the name of the executable (so we can run it !)
   gchar *menu_text; // for Weed, this is the filter_class "name"
   gchar *action_desc; // for Weed "Applying $s"
   gint min_frames; // for Weed, 1
   gint num_in_channels;
-  gshort status;
-#define RFX_STATUS_BUILTIN 0 // factory presets
-#define RFX_STATUS_CUSTOM 1 // custom effects in the custom menu
-#define RFX_STATUS_TEST 2 // test effects in the advanced menu
-#define RFX_STATUS_ANY 3 // indicates free choice of statuses
-#define RFX_STATUS_WEED 4 // indicates an internal RFX, created from a weed instance
-#define RFX_STATUS_SCRAP 5 // used for parsing RFX scraps from external apps
+  lives_rfx_status_t status;
 
-  // these are only used when prompting for a name
-#define RFX_STATUS_COPY 128 // indicates a copy operation to test
-#define RFX_STATUS_RENAME 129 // indicates a copy operation to test
 
   guint32 props;
 #define RFX_PROPS_SLOW        0x0001  // hint to GUI
@@ -362,7 +433,7 @@ void render_fx_get_params (lives_rfx_t *, const gchar *plugin_name, gshort statu
 
 void sort_rfx_array (lives_rfx_t *in_array, gint num_elements);
 
-gint find_rfx_plugin_by_name (gchar *name, gshort status);
+gint find_rfx_plugin_by_name (const gchar *name, gshort status);
 
 void rfx_copy (lives_rfx_t *src, lives_rfx_t *dest, gboolean full);
 
@@ -407,9 +478,9 @@ gdouble get_double_param(void *value);
 void get_colRGB24_param(void *value, lives_colRGB24_t *rgb);
 void get_colRGBA32_param(void *value, lives_colRGBA32_t *rgba);
 
-void set_bool_param(void *value, const gboolean _const);
-void set_int_param(void *value, const gint _const);
-void set_double_param(void *value, const gdouble _const);
+void set_bool_param(void *value, gboolean );
+void set_int_param(void *value, gint );
+void set_double_param(void *value, gdouble );
 void set_colRGB24_param(void *value, gshort red, gshort green, gshort blue);
 void set_colRGBA32_param(void *value, gshort red, gshort green, gshort blue, gshort alpha);
 
@@ -423,7 +494,7 @@ GList *array_to_string_list (gchar **array, gint offset, gint len);
 
 lives_rfx_t *weed_to_rfx (weed_plant_t *plant, gboolean show_reinits);
 
-gchar *plugin_run_param_window(gchar *get_com, GtkVBox *vbox, lives_rfx_t **ret_rfx);
+gchar *plugin_run_param_window(const gchar *get_com, GtkVBox *vbox, lives_rfx_t **ret_rfx);
 
 ////////////////////////////////////////////////////////////
 // video playback plugin window - fixed part
