@@ -132,13 +132,13 @@ typedef struct {
   gboolean osc_udp_started;
   guint osc_udp_port;
 
+  // not used yet
   gboolean omc_noisy;
 
   gshort startup_phase; // -1 = fresh install, 1 = tmpdir set, 2, pre-audio start, 3, pre-tests, 100 = all tests passed
   const gchar *wm; //window manager name
   gint ocp; // open_compression_percent : get/set in prefs
 
-  // whether the 'blend_file' (second file in a transition) is fg or bg
   gboolean antialias;
 
   gboolean ignore_tiny_fps_diffs;
@@ -147,12 +147,13 @@ typedef struct {
 
   guint jack_opts;
 #define JACK_OPTS_TRANSPORT_CLIENT (1<<0)   // jack can start/stop
-#define JACK_OPTS_TRANSPORT_MASTER (1<<1)
-#define JACK_OPTS_START_TSERVER (1<<2)
-#define JACK_OPTS_NOPLAY_WHEN_PAUSED (1<<3)
-#define JACK_OPTS_START_ASERVER (1<<4)
-#define JACK_OPTS_TIMEBASE_CLIENT (1<<5)    // jack sets play start position
-#define JACK_OPTS_TIMEBASE_MASTER (1<<6)
+#define JACK_OPTS_TRANSPORT_MASTER (1<<1)  // transport master
+#define JACK_OPTS_START_TSERVER (1<<2)     // start transport server
+#define JACK_OPTS_NOPLAY_WHEN_PAUSED (1<<3) // play audio even when transport paused
+#define JACK_OPTS_START_ASERVER (1<<4)     // start audio server
+#define JACK_OPTS_TIMEBASE_START (1<<5)    // jack sets play start position
+#define JACK_OPTS_TIMEBASE_CLIENT (1<<6)    // full timebase client
+#define JACK_OPTS_TIMEBASE_MASTER (1<<7)   // timebase master (not implemented yet)
 
   gchar jack_tserver[256];
   gchar jack_aserver[256];
@@ -250,10 +251,38 @@ typedef struct {
 
   gboolean ce_maxspect;
 
-  glong rec_stop_gb;
+  gboolean lamp_buttons;
 
+  glong rec_stop_gb;
 } _prefs;
 
+enum {
+    LIST_ENTRY_GUI,
+    LIST_ENTRY_MULTITRACK,
+    LIST_ENTRY_DECODING,
+    LIST_ENTRY_PLAYBACK,
+    LIST_ENTRY_RECORDING,
+    LIST_ENTRY_ENCODING,
+    LIST_ENTRY_EFFECTS,
+    LIST_ENTRY_DIRECTORIES,
+    LIST_ENTRY_WARNINGS,
+    LIST_ENTRY_MISC,
+    LIST_ENTRY_THEMES,
+    LIST_ENTRY_NET,
+    LIST_ENTRY_JACK,
+    LIST_ENTRY_MIDI
+};
+
+enum {
+    LIST_ICON = 0,
+    LIST_ITEM,
+    LIST_NUM,
+    N_COLUMNS
+};
+
+
+#define PREF_WIN_WIDTH 960
+#define PREF_WIN_HEIGHT 640
 
 
 // prefs window
@@ -261,7 +290,30 @@ typedef struct {
   gulong encoder_ofmt_fn;
   gulong encoder_name_fn;
   GtkWidget *prefs_dialog;
-  GtkWidget *prefs_notebook;
+  
+  // Added by AK
+  GtkWidget *prefs_list;
+  GtkWidget *prefs_table;
+  //
+  GtkWidget *vbox_right_gui;
+  GtkWidget *vbox_right_multitrack;
+  GtkWidget *vbox_right_decoding;
+  GtkWidget *vbox_right_playback;
+  GtkWidget *vbox_right_recording;
+  GtkWidget *vbox_right_encoding;
+  GtkWidget *vbox_right_effects;
+  GtkWidget *table_right_directories;
+  GtkWidget *vbox_right_warnings;
+  GtkWidget *vbox_right_warnings_outer;
+  GtkWidget *vbox_right_misc;
+  GtkWidget *vbox_right_themes;
+  GtkWidget *vbox_right_net;
+  GtkWidget *vbox_right_jack;
+  GtkWidget *vbox_right_midi;
+  GtkWidget *scrollw;
+  //
+  GtkWidget *right_shown;
+
   GtkWidget *cancelbutton;
   GtkWidget *stop_screensaver_check;
   GtkWidget *open_maximised_check;
@@ -332,6 +384,7 @@ typedef struct {
   GtkWidget *jack_aserver_entry;
   GtkWidget *checkbutton_jack_master;
   GtkWidget *checkbutton_jack_client;
+  GtkWidget *checkbutton_jack_tb_start;
   GtkWidget *checkbutton_jack_tb_client;
   GtkWidget *checkbutton_jack_pwp;
   GtkWidget *checkbutton_start_tjack;
@@ -375,8 +428,6 @@ typedef struct {
   GtkWidget *checkbutton_ce_maxspect;
 } _prefsw;
 
-
-
 typedef struct {  // startup overrides from commandline
   gboolean ign_clipset;
   gboolean ign_osc;
@@ -385,7 +436,6 @@ typedef struct {  // startup overrides from commandline
   gboolean ign_stmode;
 } _ign_opts;
 
-
 typedef struct {
   gchar tmpdir[256];
   gchar theme[64];
@@ -412,7 +462,6 @@ _prefs *prefs;
 _future_prefs *future_prefs;
 _prefsw *prefsw;
 
-
 void set_acodec_list_from_allowed (_prefsw *, render_details *);
 void  rdet_acodec_changed (GtkEntry *acodec_entry, gpointer user_data);
 
@@ -428,4 +477,8 @@ void on_prefs_cancel_clicked (GtkButton *, gpointer);
 
 void set_vpp(gboolean set_in_prefs);
 
+void on_prefDomainChanged(GtkTreeSelection *, gpointer);
+
+void select_pref_list_row(guint);
+
 #endif
