@@ -34,6 +34,10 @@ static size_t prb=0;
 
 
 static void pulse_server_cb(pa_context *c,const pa_server_info *info, void *userdata) {
+  if (info==NULL) {
+    pulse_server_rate=0;
+    return;
+  }
   pulse_server_rate=info->sample_spec.rate;
 }
 
@@ -76,25 +80,28 @@ gboolean lives_pulse_init (short startup_phase) {
     pulse_shutdown();
 
     g_printerr("%s",_("\nUnable to connect to pulse audio server\n"));
-    if (startup_phase==0&&capable->has_sox) {
-      do_error_dialog(_("\nUnable to connect to pulse audio server.\nFalling back to sox audio player.\nYou can change this in Preferences/Playback.\n"));
-      switch_aud_to_sox();
-    }
-    else if (startup_phase==0&&capable->has_mplayer) {
-      do_error_dialog(_("\nUnable to connect to pulse audio server.\nFalling back to mplayer audio player.\nYou can change this in Preferences/Playback.\n"));
-      switch_aud_to_mplayer();
-    }
-    else {
-      msg=g_strdup(_("\nUnable to connect to pulse audio server.\n"));
-      if (startup_phase==0) {
-	do_error_dialog(msg);
+
+    if (!mainw->foreign) {
+      if (startup_phase==0&&capable->has_sox) {
+	do_error_dialog(_("\nUnable to connect to pulse audio server.\nFalling back to sox audio player.\nYou can change this in Preferences/Playback.\n"));
+	switch_aud_to_sox();
+      }
+      else if (startup_phase==0&&capable->has_mplayer) {
+	do_error_dialog(_("\nUnable to connect to pulse audio server.\nFalling back to mplayer audio player.\nYou can change this in Preferences/Playback.\n"));
+	switch_aud_to_mplayer();
       }
       else {
-	msg2=g_strdup_printf("%s%s",msg,_("LiVES will exit and you can choose another audio player.\n"));
-	do_blocking_error_dialog(msg2);
-	g_free(msg2);
+	msg=g_strdup(_("\nUnable to connect to pulse audio server.\n"));
+	if (startup_phase==0) {
+	  do_error_dialog(msg);
+	}
+	else {
+	  msg2=g_strdup_printf("%s%s",msg,_("LiVES will exit and you can choose another audio player.\n"));
+	  do_blocking_error_dialog(msg2);
+	  g_free(msg2);
+	}
+	g_free(msg);
       }
-      g_free(msg);
     }
     return FALSE;
   }
@@ -119,6 +126,7 @@ static void pulse_buff_free(void *ptr) {
 
 static void sample_silence_pulse (pulse_driver_t *pdriver, size_t nbytes, size_t xbytes) {
   guchar *buff;
+  if (xbytes<=0) return;
   while (nbytes>0) {
     if (nbytes<xbytes) xbytes=nbytes;
     buff=g_malloc0(xbytes);
@@ -159,8 +167,7 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
     return;
   }
 
-  if ((msg=(aserver_message_t *)pulsed->msgq)!=NULL) {
-
+  while ((msg=(aserver_message_t *)pulsed->msgq)!=NULL) {
     switch (msg->command) {
     case ASERVER_CMD_FILE_OPEN:
       new_file=atoi(msg->data);
@@ -172,34 +179,35 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 	  g_printerr("pulsed: error opening %s\n",filename);
 	  pulsed->playing_file=-1;
 	}
-	if (pulsed->fd>0) {
+	else {
 	  if (pulsed->aPlayPtr->data!=NULL) g_free(pulsed->aPlayPtr->data);
 	  if (nbytes>0) pulsed->aPlayPtr->data=g_malloc(nbytes*100);
 	  else (pulsed->aPlayPtr->data)=NULL;
 	  pulsed->seek_pos=0;
 	  pulsed->playing_file=new_file;
+	  pulsed->audio_ticks=mainw->currticks;
+	  pulsed->frames_written=0;
 	}
 	g_free(filename);
       }
       break;
     case ASERVER_CMD_FILE_CLOSE:
-      if (pulsed->fd>0) close(pulsed->fd);
-      pulsed->fd=0;
+      if (pulsed->fd>=0) close(pulsed->fd);
+      pulsed->fd=-1;
       if (pulsed->aPlayPtr->data!=NULL) g_free(pulsed->aPlayPtr->data);
       pulsed->aPlayPtr->data=NULL;
       pulsed->aPlayPtr->size=0;
       pulsed->playing_file=-1;
       break;
     case ASERVER_CMD_FILE_SEEK:
-      if (pulsed->fd<1) break;
+      if (pulsed->fd<0) break;
       xseek=seek=atol(msg->data);
       if (seek<0.) xseek=0.;
       if (!pulsed->mute) {
 	lseek(pulsed->fd,xseek,SEEK_SET);
       }
       pulsed->seek_pos=seek;
-      gettimeofday(&tv, NULL);
-      pulsed->audio_ticks=U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*U_SEC_RATIO;
+      pulsed->audio_ticks=mainw->currticks;
       pulsed->frames_written=0;
       break;
     default:
@@ -207,9 +215,9 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
     }
     if (msg->data!=NULL) g_free(msg->data);
     msg->command=ASERVER_CMD_PROCESSED;
-    pulsed->msgq = NULL;
+    if (msg->next==NULL) pulsed->msgq=NULL;
+    else pulsed->msgq = msg->next;
   }
-
   if (pulsed->chunk_size!=nbytes) pulsed->chunk_size = nbytes;
 
   pulsed->state=pa_context_get_state(pulsed->con);
@@ -229,7 +237,7 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 
     pulsed->num_calls++;
 
-    if (!pulsed->in_use||((pulsed->fd<1||pulsed->seek_pos<0.)&&pulsed->read_abuf<0)||pulsed->is_paused) {
+    if (!pulsed->in_use||((pulsed->fd<0||pulsed->seek_pos<0.)&&pulsed->read_abuf<0)||pulsed->is_paused) {
 
       sample_silence_pulse(pulsed,nframes*pulsed->out_achans*(pulsed->out_asamps>>3),xbytes);
 
@@ -255,7 +263,7 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 
       }
       else {
-	if (G_LIKELY(pulsed->fd>0)) {
+	if (G_LIKELY(pulsed->fd>=0)) {
 	  pulsed->aPlayPtr->size=0;
 	  in_bytes=ABS((in_frames=((gdouble)pulsed->in_arate/(gdouble)pulsed->out_arate*(gdouble)pulseFramesAvailable+((gdouble)fastrand()/(gdouble)G_MAXUINT32))))*pulsed->in_achans*(pulsed->in_asamps>>3);
 	  //g_print("in bytes=%ld %ld %ld %ld %d %d\n",in_bytes,pulsed->in_arate,pulsed->out_arate,pulseFramesAvailable,pulsed->in_achans,pulsed->in_asamps);
@@ -371,94 +379,95 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 	  //g_print("pt a5 %d %d\n",pulsed->in_use,in_bytes);
 	  return;
 	}
-
+	
 	inputFramesAvailable = pulsed->aPlayPtr->size / (pulsed->in_achans * (pulsed->in_asamps>>3));
 #ifdef DEBUG_PULSE
 	g_printerr("%ld inputFramesAvailable == %ld, %ld, %ld %ld,pulseFramesAvailable == %ld\n", pulsed->aPlayPtr->size, inputFramesAvailable, in_frames, pulsed->in_arate,pulsed->out_arate,pulseFramesAvailable);
 #endif
 	buffer = pulsed->aPlayPtr->data;
     
-	numFramesToWrite = MIN(pulseFramesAvailable, inputFramesAvailable/ABS(shrink_factor));
+	numFramesToWrite = MIN(pulseFramesAvailable, (inputFramesAvailable/ABS(shrink_factor)+.001));
     
 #ifdef DEBUG_PULSE
-    g_printerr("inputFramesAvailable after conversion %ld\n", (gulong)((gdouble)inputFramesAvailable/shrink_factor));
-    g_printerr("nframes == %ld, pulseFramesAvailable == %ld,\n\tpulsed->num_input_channels == %ld, pulsed->out_achans == %ld\n",  nframes, pulseFramesAvailable, pulsed->in_achans, pulsed->out_achans);
+	g_printerr("inputFramesAvailable after conversion %d\n",(gulong)((gdouble)inputFramesAvailable/shrink_factor+.001));
+	g_printerr("nframes == %ld, pulseFramesAvailable == %ld,\n\tpulsed->num_input_channels == %ld, pulsed->out_achans == %ld\n",  nframes, pulseFramesAvailable, pulsed->in_achans, pulsed->out_achans);
 #endif
-    
-    swap_sign=afile->signed_endian&AFORM_UNSIGNED;
-
-    if (pulsed->in_asamps==pulsed->out_asamps&&shrink_factor==1.&&pulsed->in_achans==pulsed->out_achans&&!pulsed->reverse_endian&&!swap_sign) {
-      // no transformation needed
-      pulsed->sound_buffer=buffer;
-    }
-    else {
-      pulsed->sound_buffer=g_malloc0(pulsed->chunk_size);
-
-      if (pulsed->in_asamps==8) {
-	sample_move_d8_d16 ((short *)(pulsed->sound_buffer),(guchar *)buffer, numFramesToWrite, in_bytes, shrink_factor, pulsed->out_achans, pulsed->in_achans, swap_sign?SWAP_U_TO_S:0);
-      }
-      else {
-	sample_move_d16_d16((short*)pulsed->sound_buffer, (short*)buffer, numFramesToWrite, in_bytes, shrink_factor, pulsed->out_achans, pulsed->in_achans, pulsed->reverse_endian?SWAP_X_TO_L:0, swap_sign?SWAP_U_TO_S:0);
-      }
-    }
-
-    pulsed->frames_written+=numFramesToWrite;
-    pulseFramesAvailable-=numFramesToWrite;
-    
+	
+	swap_sign=afile->signed_endian&AFORM_UNSIGNED;
+	
+	if (pulsed->in_asamps==pulsed->out_asamps&&shrink_factor==1.&&pulsed->in_achans==pulsed->out_achans&&!pulsed->reverse_endian&&!swap_sign) {
+	  // no transformation needed
+	  pulsed->sound_buffer=buffer;
+	}
+	else {
+	  pulsed->sound_buffer=g_malloc0(pulsed->chunk_size);
+	  
+	  if (pulsed->in_asamps==8) {
+	    sample_move_d8_d16 ((short *)(pulsed->sound_buffer),(guchar *)buffer, numFramesToWrite, in_bytes, shrink_factor, pulsed->out_achans, pulsed->in_achans, swap_sign?SWAP_U_TO_S:0);
+	  }
+	  else {
+	    sample_move_d16_d16((short*)pulsed->sound_buffer, (short*)buffer, numFramesToWrite, in_bytes, shrink_factor, pulsed->out_achans, pulsed->in_achans, pulsed->reverse_endian?SWAP_X_TO_L:0, swap_sign?SWAP_U_TO_S:0);
+	  }
+	}
+	
+	pulsed->frames_written+=numFramesToWrite;
+	pulseFramesAvailable-=numFramesToWrite;
+	
 #ifdef DEBUG_PULSE
-    g_printerr("pulseFramesAvailable == %ld\n", pulseFramesAvailable);
+	g_printerr("pulseFramesAvailable == %ld\n", pulseFramesAvailable);
 #endif
       }
   
-  // playback from memory or file
-
-  if (mainw->volume!=old_volume) {
-    pa_operation *pa_op;
-    pavol=pa_sw_volume_from_linear(mainw->volume);
-    pa_cvolume_set(&out_vol,pulsed->out_achans,pavol);
-    pa_op=pa_context_set_sink_input_volume(pulsed->con,pa_stream_get_index(pulsed->pstream),&out_vol,NULL,NULL);
-    pa_operation_unref(pa_op);
-    old_volume=mainw->volume;
-  }
-
-
-  while (nbytes>0) {
-    if (nbytes<xbytes) xbytes=nbytes;
-    
-    if (!from_memory) {
-      if (xbytes/pulsed->out_achans/(pulsed->out_asamps>>3)<=numFramesToWrite&&offs==0) {
-	buffer=pulsed->sound_buffer;
+      // playback from memory or file
+      
+      if (mainw->volume!=old_volume) {
+	pa_operation *pa_op;
+	pavol=pa_sw_volume_from_linear(mainw->volume);
+	pa_cvolume_set(&out_vol,pulsed->out_achans,pavol);
+	pa_op=pa_context_set_sink_input_volume(pulsed->con,pa_stream_get_index(pulsed->pstream),&out_vol,NULL,NULL);
+	pa_operation_unref(pa_op);
+	old_volume=mainw->volume;
       }
-      else {
-	buffer=g_malloc(xbytes);
-	w_memcpy(buffer,pulsed->sound_buffer+offs,xbytes);
-	offs+=xbytes;
-	needs_free=TRUE;
+      
+      
+      while (nbytes>0) {
+	if (nbytes<xbytes) xbytes=nbytes;
+	
+	if (!from_memory) {
+	  if (xbytes/pulsed->out_achans/(pulsed->out_asamps>>3)<=numFramesToWrite&&offs==0) {
+	    buffer=pulsed->sound_buffer;
+	  }
+	  else {
+	    buffer=g_malloc(xbytes);
+	    w_memcpy(buffer,pulsed->sound_buffer+offs,xbytes);
+	    offs+=xbytes;
+	    needs_free=TRUE;
+	  }
+	  pa_stream_write(pulsed->pstream,buffer,xbytes,buffer==pulsed->aPlayPtr->data?NULL:pulse_buff_free,0,PA_SEEK_RELATIVE);
+	}
+	else {
+	  if (pulsed->read_abuf>-1&&!pulsed->mute) {
+	    buffer=g_malloc(xbytes);
+	    sample_move_abuf_int16((short *)buffer,pulsed->out_achans,(xbytes>>1)/pulsed->out_achans,pulsed->out_arate);
+	    pa_stream_write(pulsed->pstream,buffer,xbytes,pulse_buff_free,0,PA_SEEK_RELATIVE);
+	  }
+	  else {
+	    sample_silence_pulse(pulsed,xbytes,xbytes);
+	  }
+	}
+	nbytes-=xbytes;
       }
-      pa_stream_write(pulsed->pstream,buffer,xbytes,buffer==pulsed->aPlayPtr->data?NULL:pulse_buff_free,0,PA_SEEK_RELATIVE);
-    }
-    else {
-      if (pulsed->read_abuf>-1&&!pulsed->mute) {
-	buffer=g_malloc(xbytes);
-	sample_move_abuf_int16((short *)buffer,pulsed->out_achans,(xbytes>>1)/pulsed->out_achans,pulsed->out_arate);
-	pa_stream_write(pulsed->pstream,buffer,xbytes,pulse_buff_free,0,PA_SEEK_RELATIVE);
-      }
-      else {
-	sample_silence_pulse(pulsed,nframes*pulsed->out_achans*(pulsed->out_asamps>>3),xbytes);
-      }
-    }
-    nbytes-=xbytes;
-  }
-
-  if (needs_free&&pulsed->sound_buffer!=pulsed->aPlayPtr->data) g_free(pulsed->sound_buffer);
-
+      
+      if (needs_free&&pulsed->sound_buffer!=pulsed->aPlayPtr->data) g_free(pulsed->sound_buffer);
+      
     }
        
     if(pulseFramesAvailable) {
 #ifdef DEBUG_PULSE
       g_printerr("buffer underrun of %ld frames\n", pulseFramesAvailable);
 #endif
-      sample_silence_pulse(pulsed,nframes*pulsed->out_achans*(pulsed->out_asamps>>3),xbytes);
+      xbytes=pa_stream_writable_size(pstream);
+      sample_silence_pulse(pulsed,pulseFramesAvailable*pulsed->out_achans*(pulsed->out_asamps>>3),xbytes);
     }
   }
   else {
@@ -486,7 +495,7 @@ void pulse_flush_read_data(pulse_driver_t *pulsed, size_t rbytes, void *data) {
 
   int swap_sign=afile->signed_endian&AFORM_UNSIGNED;
 
-  if (prb==0) return;
+  if (prb==0||mainw->rec_samples==0) return;
 
   gbuf=(short *)g_malloc(prb);
 
@@ -508,8 +517,6 @@ void pulse_flush_read_data(pulse_driver_t *pulsed, size_t rbytes, void *data) {
     sample_move_d16_d8((uint8_t *)holding_buff,gbuf,frames_out,prb,out_scale,afile->achans,pulsed->in_achans,swap_sign?SWAP_S_TO_U:0);
   }
     
-  pulsed->frames_written+=frames_out;
-
   g_free(gbuf);
   prb=0;
 
@@ -528,21 +535,29 @@ void pulse_flush_read_data(pulse_driver_t *pulsed, size_t rbytes, void *data) {
 
 static void pulse_audio_read_process (pa_stream *pstream, size_t nbytes, void *arg) {
   // read nframes from pulse buffer, and then write to mainw->aud_rec_fd
+
+  // this is the callback from pulse when we are recording
+
   pulse_driver_t* pulsed = (pulse_driver_t*)arg;
   float out_scale=(float)pulsed->in_arate/(float)afile->arate;
   size_t frames_out;
   void *data;
-  size_t rbytes;
-
+  size_t rbytes=nbytes;
+  
   if (mainw->effects_paused) return; // pause during record
 
   pa_stream_peek(pulsed->pstream,(const void**)&data,&rbytes);
 
+  if (data==NULL) return;
+
   prb+=rbytes;
 
   frames_out=(size_t)((gdouble)((prb/(pulsed->in_asamps>>3)/pulsed->in_achans))/out_scale+.5);
   
-  if (prb<PULSE_READ_BYTES&&frames_out<mainw->rec_samples) {
+  // should really be frames_read here
+  pulsed->frames_written+=(size_t)((gdouble)((rbytes/(pulsed->in_asamps>>3)/pulsed->in_achans))/out_scale+.5);
+
+  if (prb<PULSE_READ_BYTES&&(mainw->rec_samples==-1||frames_out<mainw->rec_samples)) {
     // buffer until we have enough
     w_memcpy(&prbuf[prb-rbytes],data,rbytes);
     pa_stream_drop(pulsed->pstream);
@@ -599,7 +614,7 @@ int pulse_audio_init(void) {
   for (j=0;j<PULSE_MAX_OUTPUT_CHANS;j++) pulsed.volume[j]=1.0f;
   pulsed.state=PA_STREAM_UNCONNECTED;
   pulsed.in_arate=44100;
-  pulsed.fd=0;
+  pulsed.fd=-1;
   pulsed.seek_pos=pulsed.seek_end=0;
   pulsed.msgq=NULL;
   pulsed.num_calls=0;
@@ -633,7 +648,7 @@ int pulse_audio_read_init(void) {
 
   for (j=0;j<PULSE_MAX_OUTPUT_CHANS;j++) pulsed_reader.volume[j]=1.0f;
   pulsed_reader.state=PA_STREAM_UNCONNECTED;
-  pulsed_reader.fd=0;
+  pulsed_reader.fd=-1;
   pulsed_reader.seek_pos=pulsed_reader.seek_end=0;
   pulsed_reader.msgq=NULL;
   pulsed_reader.num_calls=0;
@@ -693,9 +708,16 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
   if (G_BYTE_ORDER==G_BIG_ENDIAN) pdriver->out_endian=AFORM_BIG_ENDIAN;
   else pdriver->out_endian=AFORM_LITTLE_ENDIAN;
 
-  pa_battr.maxlength=LIVES_PA_BUFF_MAXLEN;
+  if (pdriver->is_output) {
+    pa_battr.maxlength=LIVES_PA_BUFF_MAXLEN;
+    pa_battr.tlength=LIVES_PA_BUFF_TARGET;
+  }
+  else {
+    pa_battr.maxlength=(uint32_t)-1;
+    pa_battr.tlength=(uint32_t)-1;
+  }
+
   pa_battr.minreq=(uint32_t)-1;
-  pa_battr.tlength=LIVES_PA_BUFF_TARGET;
   pa_battr.prebuf=0;
 
   if (pulse_server_rate==0) {
@@ -707,6 +729,10 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
     pa_operation_unref(pa_op);
   }
 
+  if (pulse_server_rate==0) {
+    g_printerr("Warning - problem getting pulseaudio rate...expect more problems ahead.\n");
+    return 1;
+  }
 
   pa_spec.rate=pdriver->out_arate=pdriver->in_arate=pulse_server_rate;
 
@@ -732,11 +758,13 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
   }
   else {
     // set read callback
+    pdriver->audio_ticks=0;
+    pdriver->frames_written=0;
+    prb=0;
+
     pa_stream_set_read_callback(pdriver->pstream,pulse_audio_read_process,pdriver);
 
-    pa_stream_connect_record(pdriver->pstream,NULL,&pa_battr,0);
-
-    prb=0;
+    pa_stream_connect_record(pdriver->pstream,NULL,&pa_battr,PA_STREAM_START_CORKED|PA_STREAM_EARLY_REQUESTS);
 
     while (pa_stream_get_state(pdriver->pstream)!=PA_STREAM_READY) {
       g_usleep(prefs->sleep_time);
@@ -750,6 +778,11 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
 }
 
 
+void pulse_driver_uncork(pulse_driver_t *pdriver) {
+  pa_stream_cork(pdriver->pstream,0,NULL,NULL);
+}
+
+
 ///////////////////////////////////////////////////////////////
 
 
@@ -779,7 +812,7 @@ gint64 lives_pulse_get_time(pulse_driver_t *pulsed, gboolean absolute) {
   frames_written=pulsed->frames_written;
   if (frames_written<0.) frames_written=0.;
   if (pulsed->is_output) return pulsed->audio_ticks*absolute+(gint64)(frames_written/(gdouble)pulsed->out_arate*U_SEC);
-  return pulsed->audio_ticks*absolute+(gint64)(frames_written/(gdouble)pulsed->in_arate*U_SEC);
+  return pulsed->audio_ticks*absolute+(gint64)(frames_written/(gdouble)afile->arate*U_SEC);
 
 }
 
@@ -789,8 +822,6 @@ gdouble lives_pulse_get_pos(pulse_driver_t *pulsed) {
 }
 
 
-static aserver_message_t pulse_message;
-
 void pulse_audio_seek_frame (pulse_driver_t *pulsed, gint frame) {
   // seek to frame "frame" in current audio file
   // position will be adjusted to (floor) nearest sample
@@ -804,7 +835,7 @@ void pulse_audio_seek_frame (pulse_driver_t *pulsed, gint frame) {
   if (pulsed->playing_file==-1) return;
   if (frame>afile->frames) frame=afile->frames;
   seekstart=(long)((gdouble)(frame-1.)/afile->fps*afile->arate)*afile->achans*(afile->asampsize/8);
-  afile->aseek_pos=pulse_audio_seek_bytes(pulsed,seekstart);
+  pulse_audio_seek_bytes(pulsed,seekstart);
 }
 
 
@@ -813,12 +844,12 @@ long pulse_audio_seek_bytes (pulse_driver_t *pulsed, long bytes) {
   // position will be adjusted to (floor) nearest sample
 
   // if the position is > size of file, we will seek to the end of the file
-
   volatile aserver_message_t *pmsg;
+
   long seekstart;
   do {
     pmsg=pulse_get_msgq(pulsed);
-  } while ((pmsg!=NULL)&&pmsg->command!=ASERVER_CMD_FILE_SEEK);
+  } while ((pmsg!=NULL));
   if (pulsed->playing_file==-1) return 0;
   seekstart=((long)(bytes/afile->achans/(afile->asampsize/8)))*afile->achans*(afile->asampsize/8);
 
