@@ -72,13 +72,13 @@ typedef struct {
 
   int fd; /* if >0 we are playing from a file */
   gboolean is_opening; // TRUE if file is opening (audiodump.pcm)
-  off_t seek_pos;
+  volatile off_t seek_pos;
   off_t seek_end;
   gboolean usigned;
   gboolean reverse_endian;
 
-  gshort *whentostop; // pointer to mainw->whentostop
-  volatile gint *cancelled; // pointer to mainw->cancelled
+  lives_whentostop_t *whentostop; // pointer to mainw->whentostop
+  volatile lives_cancel_t *cancelled; // pointer to mainw->cancelled
 
   /* variables used for trying to restart the connection to pulse */
   gboolean pulsed_died;
@@ -114,9 +114,9 @@ void pulse_close_client(pulse_driver_t *); //
 
 void pulse_shutdown(void); // shudown server, mainloop, context
 
-void pulse_flush_read_data(pulse_driver_t *pulsed, size_t rbytes, void *data);
-
+void pulse_flush_read_data(pulse_driver_t *, size_t rbytes, void *data);
 
+void pulse_driver_uncork(pulse_driver_t *);
 
 // utils
 volatile aserver_message_t *pulse_get_msgq(pulse_driver_t *); // pull last msg from msgq, or return NULL
