@@ -1380,10 +1380,8 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     gtk_widget_show (frame7);
     gtk_box_pack_start (GTK_BOX (vbox21), frame7, TRUE, TRUE, 0);
 
-    if (type!=4) {
-      if (palette->style&STYLE_1) {
-	gtk_widget_modify_bg (frame7, GTK_STATE_NORMAL, &palette->normal_back);
-      }
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_bg (frame7, GTK_STATE_NORMAL, &palette->normal_back);
     }
 
     xvbox = gtk_vbox_new (FALSE, 0);
@@ -1403,12 +1401,10 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
 			G_CALLBACK (label_act_toggle),
 			resaudw->aud_checkbutton);
 
-      if (type!=4) {
-	if (palette->style&STYLE_1) {
-	  gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
-	  gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
-	  gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
-	}
+      if (palette->style&STYLE_1) {
+	gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
+	gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
+	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
       }
 
       resaudw->aud_hbox = gtk_hbox_new (FALSE, 0);
@@ -1430,11 +1426,11 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     
     if (type>=3) label98 = gtk_label_new_with_mnemonic (_("_Rate (Hz) "));
     else label98 = gtk_label_new (_("Rate (Hz) "));
-    if (type!=4) {
-      if (palette->style&STYLE_1) {
-	gtk_widget_modify_fg(label98, GTK_STATE_NORMAL, &palette->normal_fore);
-      }
+
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label98, GTK_STATE_NORMAL, &palette->normal_fore);
     }
+
     gtk_widget_show (label98);
     gtk_box_pack_start (GTK_BOX (hbox25), label98, FALSE, FALSE, 0);
     gtk_label_set_justify (GTK_LABEL (label98), GTK_JUSTIFY_LEFT);
@@ -1458,11 +1454,11 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     
     if (type>=3) label99 = gtk_label_new_with_mnemonic (_("    _Channels "));
     else label99 = gtk_label_new (_("    Channels "));
-    if (type!=4) {
-      if (palette->style&STYLE_1) {
-	gtk_widget_modify_fg(label99, GTK_STATE_NORMAL, &palette->normal_fore);
-      }
+
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label99, GTK_STATE_NORMAL, &palette->normal_fore);
     }
+
     gtk_widget_show (label99);
     gtk_box_pack_start (GTK_BOX (hbox25), label99, FALSE, FALSE, 0);
     gtk_label_set_justify (GTK_LABEL (label99), GTK_JUSTIFY_LEFT);
@@ -1492,11 +1488,11 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
 
     if (type>=3) label100 = gtk_label_new_with_mnemonic (_("    _Sample Size "));
     else label100 = gtk_label_new (_("    Sample Size "));
-    if (type!=4) {
-      if (palette->style&STYLE_1) {
-	gtk_widget_modify_fg(label100, GTK_STATE_NORMAL, &palette->normal_fore);
-      }
+
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label100, GTK_STATE_NORMAL, &palette->normal_fore);
     }
+
     gtk_widget_show (label100);
     gtk_box_pack_start (GTK_BOX (hbox25), label100, FALSE, FALSE, 0);
     gtk_label_set_justify (GTK_LABEL (label100), GTK_JUSTIFY_LEFT);
@@ -1526,11 +1522,11 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
 
     vseparator5 = gtk_vseparator_new ();
     gtk_widget_show (vseparator5);
-    gtk_box_pack_start (GTK_BOX (hbox25), vseparator5, FALSE, FALSE, 12);
+    if (type!=4) gtk_box_pack_start (GTK_BOX (hbox25), vseparator5, FALSE, FALSE, 12);
 
     vbox24 = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox24);
-    gtk_box_pack_start (GTK_BOX (hbox25), vbox24, TRUE, TRUE, 0);
+    if (type!=4) gtk_box_pack_start (GTK_BOX (hbox25), vbox24, TRUE, TRUE, 0);
 
     hbox = gtk_hbox_new (FALSE, 0);
     gtk_widget_show(hbox);
@@ -1553,13 +1549,13 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
 		      G_CALLBACK (label_act_toggle),
 		      resaudw->rb_signed);
-    if (type!=4) {
-      if (palette->style&STYLE_1) {
-	gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
-	gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
-	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
-      }
+
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
     }
+
     gtk_box_pack_start (GTK_BOX (hbox), eventbox, FALSE, FALSE, 10);
   
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_signed), TRUE);
@@ -1587,13 +1583,13 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
 		      G_CALLBACK (label_act_toggle),
 		      resaudw->rb_unsigned);
-    if (type!=4) {
-      if (palette->style&STYLE_1) {
-	gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
-	gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
-	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
-      }
+
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
     }
+
     gtk_box_pack_start (GTK_BOX (hbox), eventbox, FALSE, FALSE, 10);
 
     if (type==7||!is_8bit) gtk_widget_set_sensitive(resaudw->rb_unsigned,FALSE);
@@ -1640,13 +1636,13 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
 		      G_CALLBACK (label_act_toggle),
 		      resaudw->rb_littleend);
-    if (type!=4) {
-      if (palette->style&STYLE_1) {
-	gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
-	gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
-	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
-      }
+
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
     }
+
     gtk_box_pack_start (GTK_BOX (hbox), eventbox, FALSE, FALSE, 10);
 
     if (type==7) gtk_widget_set_sensitive(resaudw->rb_littleend,FALSE);
@@ -1672,13 +1668,13 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     g_signal_connect (GTK_OBJECT (eventbox), "button_press_event",
 		      G_CALLBACK (label_act_toggle),
 		      resaudw->rb_bigend);
-    if (type!=4) {
-      if (palette->style&STYLE_1) {
-	gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
-	gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
-	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
-      }
+
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_fg(eventbox, GTK_STATE_NORMAL, &palette->normal_fore);
+      gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &palette->normal_back);
     }
+
     gtk_box_pack_start (GTK_BOX (hbox), eventbox, FALSE, FALSE, 10);
 
     if (type==7) gtk_widget_set_sensitive(resaudw->rb_bigend,FALSE);
@@ -1701,11 +1697,11 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
 
     if (type>=3) label93 = gtk_label_new (_("Audio"));
     else label93 = gtk_label_new (_("New"));
-    if (type!=4) {
-      if (palette->style&STYLE_1) {
-	gtk_widget_modify_fg(label93, GTK_STATE_NORMAL, &palette->normal_fore);
-      }
+
+    if (palette->style&STYLE_1) {
+      gtk_widget_modify_fg(label93, GTK_STATE_NORMAL, &palette->normal_fore);
     }
+
     gtk_widget_show (label93);
 
     if (type==3&&palette->style&STYLE_1) gtk_widget_modify_bg(frame7, GTK_STATE_NORMAL, &palette->normal_back);
@@ -1742,6 +1738,7 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     resaudw->fps_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 3);
     gtk_widget_show (resaudw->fps_spinbutton);
     gtk_box_pack_start (GTK_BOX (hbox25), resaudw->fps_spinbutton, FALSE, FALSE, 20);
+    gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (resaudw->fps_spinbutton)->entry)), TRUE);
 
     label93 = gtk_label_new (_("Video"));
     gtk_widget_show (label93);
@@ -1768,7 +1765,7 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     radiobutton=gtk_radio_button_new(rbgroup);
     rbgroup = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton));
 
-    gtk_box_pack_start (GTK_BOX (hbox2), radiobutton, FALSE, FALSE, 10);
+    if (type!=6&&type!=7) gtk_box_pack_start (GTK_BOX (hbox2), radiobutton, FALSE, FALSE, 10);
 
     label=gtk_label_new_with_mnemonic ( _("Record for maximum:  "));
     gtk_label_set_mnemonic_widget (GTK_LABEL (label),radiobutton);
@@ -1791,6 +1788,7 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     resaudw->hour_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 0);
     gtk_widget_show (resaudw->hour_spinbutton);
     gtk_box_pack_start (GTK_BOX (hbox), resaudw->hour_spinbutton, TRUE, TRUE, 0);
+    gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (resaudw->hour_spinbutton)->entry)), TRUE);
     
     label=gtk_label_new(_(" hours  "));
     gtk_widget_show(label);
@@ -1803,6 +1801,7 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     resaudw->minute_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 0);
     gtk_widget_show (resaudw->minute_spinbutton);
     gtk_box_pack_start (GTK_BOX (hbox), resaudw->minute_spinbutton, TRUE, TRUE, 0);
+    gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (resaudw->minute_spinbutton)->entry)), TRUE);
 
     label=gtk_label_new(_(" minutes  "));
     gtk_widget_show(label);
@@ -1815,6 +1814,7 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     resaudw->second_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 2);
     gtk_widget_show (resaudw->second_spinbutton);
     gtk_box_pack_start (GTK_BOX (hbox), resaudw->second_spinbutton, TRUE, TRUE, 0);
+    gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (resaudw->second_spinbutton)->entry)), TRUE);
 
     label=gtk_label_new(_(" seconds  "));
     gtk_widget_show(label);
@@ -1823,12 +1823,18 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     }
     gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
 
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox);
+    gtk_box_pack_start (GTK_BOX (dialog_vbox), hbox, TRUE, TRUE, 10);
+
     hbox2 = gtk_hbox_new (FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (dialog_vbox), hbox2, FALSE, FALSE, 10);
+    gtk_box_pack_start (GTK_BOX (hbox), hbox2, FALSE, FALSE, 10);
 
     resaudw->unlim_radiobutton=gtk_radio_button_new(rbgroup);
     rbgroup = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->unlim_radiobutton));
 
+
+    
     gtk_box_pack_start (GTK_BOX (hbox2), resaudw->unlim_radiobutton, FALSE, FALSE, 10);
 
     label=gtk_label_new_with_mnemonic ( _("Unlimited"));
@@ -1846,7 +1852,7 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     }
     gtk_box_pack_start (GTK_BOX (hbox2), eventbox, FALSE, FALSE, 10);
 
-    gtk_widget_show_all (hbox2);
+    if (type!=6&&type!=7) gtk_widget_show_all (hbox2);
 
     g_signal_connect (GTK_OBJECT (radiobutton), "toggled",
                       G_CALLBACK (on_rb_audrec_time_toggled),
@@ -1879,6 +1885,10 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
     gtk_dialog_add_action_widget (GTK_DIALOG (resaudw->dialog), cancelbutton13, GTK_RESPONSE_CANCEL);
     GTK_WIDGET_SET_FLAGS (cancelbutton13, GTK_CAN_DEFAULT);
     
+    gtk_widget_add_accelerator (cancelbutton13, "activate", mainw->accel_group,
+				GDK_Escape, 0, 0);
+    
+      
     okbutton12 = gtk_button_new_from_stock ("gtk-ok");
     gtk_widget_show (okbutton12);
     gtk_dialog_add_action_widget (GTK_DIALOG (resaudw->dialog), okbutton12, GTK_RESPONSE_OK);
@@ -1889,7 +1899,7 @@ create_resaudw (gshort type, render_details *rdet, GtkWidget *top_vbox) {
       g_signal_connect (GTK_OBJECT (cancelbutton13), "clicked",
 			G_CALLBACK (on_cancel_button1_clicked),
 			resaudw);
-      
+
       if (type==1) {
 	g_signal_connect (GTK_OBJECT (okbutton12), "clicked",
 			  G_CALLBACK (on_resaudio_ok_clicked),
