@@ -13,7 +13,7 @@
 
 #include "../libweed/weed.h"
 #include "../libweed/weed-host.h"
-#include "../libweed/weed-effects.h"
+#include "../libweed/weed-palettes.h"
 
 #define L2L_PACKET_LEN 1024
 
@@ -419,7 +419,7 @@ static gboolean l2l_parse_packet_header(lives_vstream_t *lstream, gint strtype,
 
 
 
-void lives2lives_read_stream(gchar *host, int port) {
+void lives2lives_read_stream(const gchar *host, int port) {
   lives_vstream_t *lstream=(lives_vstream_t *)g_malloc(sizeof(lives_vstream_t));
   gboolean done=FALSE;
   gint old_file=mainw->current_file,new_file;
@@ -756,7 +756,7 @@ void weed_layer_set_from_lives2lives(weed_plant_t *layer, gint clip, lives_vstre
 	// where frames were being generated and streamed and then received
 	// - needs testing in other situations
 	gettimeofday(&tv, NULL);
-	currticks=U_SECL*(tv.tv_sec-mainw->startsecs)+tv.tv_usec*U_SEC_RATIO;
+	currticks=U_SECL*(tv.tv_sec-mainw->origsecs)+tv.tv_usec*U_SEC_RATIO-mainw->origusecs*U_SEC_RATIO;
 	if (mainw->record&&!mainw->record_paused) {
 	  if (has_last_delta_ticks&&(abs64(currticks-lstream->timecode))<last_delta_ticks) {
 	    // drop this frame
