@@ -69,7 +69,7 @@ LIVES_INLINE gchar *g_strappend (gchar *string, gint len, const gchar *new) {
 }
 
 
-LIVES_INLINE GList *g_list_append_unique(GList *xlist, gchar *add) {
+LIVES_INLINE GList *g_list_append_unique(GList *xlist, const gchar *add) {
   if (g_list_find_custom(xlist,add,(GCompareFunc)strcmp)==NULL) return g_list_append(xlist,g_strdup(add));
   return xlist;
 }
@@ -90,6 +90,8 @@ void setenv(const char *name, const char *val, int _xx) {
 }
 #endif
 
+
+
 /* convert to/from a big endian 32 bit float for internal use */
 LIVES_INLINE float LEFloat_to_BEFloat(float f) {
   char *b=(char *)(&f);
@@ -144,7 +146,9 @@ LIVES_INLINE gint calc_frame_from_time3 (gint filenum, gdouble time) {
 static gboolean check_for_audio_stop (gint fileno, gint first_frame, gint last_frame) {
   // return FALSE if audio stops playback
 
+#ifdef RT_AUDIO
   guint64 atc;
+#endif
 
 #ifdef ENABLE_JACK
   if (prefs->audio_player==AUD_PLAYER_JACK&&mainw->jackd!=NULL&&mainw->jackd->playing_file==fileno) {
@@ -191,7 +195,7 @@ static gboolean check_for_audio_stop (gint fileno, gint first_frame, gint last_f
 
 
 
-gint calc_new_playback_position(gint fileno, weed_timecode_t otc, weed_timecode_t *ntc) {
+gint calc_new_playback_position(gint fileno, gint64 otc, gint64 *ntc) {
   // returns a frame number (floor) using sfile->last_frameno and ntc-otc
   // takes into account looping modes
 
@@ -219,7 +223,7 @@ gint calc_new_playback_position(gint fileno, weed_timecode_t otc, weed_timecode_
   // this is done so we can check here if audio limits stopped playback
 
 
-  weed_timecode_t dtc=*ntc-otc;
+  gint64 dtc=*ntc-otc;
   file *sfile=mainw->files[fileno];
 
   gint dir=0;
@@ -266,7 +270,7 @@ gint calc_new_playback_position(gint fileno, weed_timecode_t otc, weed_timecode_
 
   nframe=cframe+(gint)((gdouble)dtc/U_SEC*fps+(fps>0?.5:-.5));
 
-  if (nframe==cframe) return nframe;
+  if (nframe==cframe||mainw->foreign) return nframe;
 
   if (mainw->playing_file==fileno) {
     last_frame=(mainw->playing_sel&&!mainw->is_rendering)?sfile->end:mainw->play_end;
@@ -298,7 +302,7 @@ gint calc_new_playback_position(gint fileno, weed_timecode_t otc, weed_timecode_
 
     // check if audio stopped playback
 #ifdef RT_AUDIO
-    if (mainw->whentostop==STOP_ON_AUD_END&&sfile->achans>0) {
+    if (mainw->whentostop==STOP_ON_AUD_END&&sfile->achans>0&&sfile->frames>0) {
       if (!check_for_audio_stop(fileno,first_frame,last_frame)) {
 	mainw->cancelled=CANCEL_AUD_END;
 	return 0;
@@ -307,9 +311,7 @@ gint calc_new_playback_position(gint fileno, weed_timecode_t otc, weed_timecode_
 #endif
   }
   
-
-
-
+  if (sfile->frames==0) return 0;
 
   // get our frame back to within bounds
   
@@ -400,13 +402,19 @@ gint calc_new_playback_position(gint fileno, weed_timecode_t otc, weed_timecode_
   if (nframe<first_frame) nframe=first_frame;
   if (nframe>last_frame) nframe=last_frame;
 
-  if (do_resync||(mainw->scratch!=SCRATCH_NONE&&mainw->playing_file==fileno)) {
-    resync_audio(nframe);
-    if (mainw->whentostop==STOP_ON_AUD_END&&sfile->achans>0) {
-      // we check for audio stop here, but the seek may not have happened yet
-      if (!check_for_audio_stop(fileno,first_frame,last_frame)) {
-	mainw->cancelled=CANCEL_AUD_END;
-	return 0;
+  if (prefs->audio_opts&AUDIO_OPTS_FOLLOW_FPS) {
+    if (do_resync||(mainw->scratch!=SCRATCH_NONE&&mainw->playing_file==fileno)) {
+      gboolean is_jump=FALSE;
+      if (mainw->scratch==SCRATCH_JUMP) is_jump=TRUE;
+      mainw->scratch=SCRATCH_NONE;
+      resync_audio(nframe);
+      if (is_jump) mainw->video_seek_ready=TRUE;
+      if (mainw->whentostop==STOP_ON_AUD_END&&sfile->achans>0) {
+	// we check for audio stop here, but the seek may not have happened yet
+	if (!check_for_audio_stop(fileno,first_frame,last_frame)) {
+	  mainw->cancelled=CANCEL_AUD_END;
+	  return 0;
+	}
       }
     }
   }
@@ -513,7 +521,7 @@ void d_print(const gchar *text) {
 
 
 
-gboolean add_lmap_error(int lerror, gchar *name, gpointer user_data, gint clipno, gint frameno, gdouble atime, gboolean affects_current) {
+gboolean add_lmap_error(lives_lmap_error_t lerror, const gchar *name, gpointer user_data, gint clipno, gint frameno, gdouble atime, gboolean affects_current) {
   // potentially add a layout map error to the layout textbuffer
   GtkTextIter end_iter;
   gchar *text,*name2;
@@ -710,7 +718,7 @@ void clear_lmap_errors(void) {
 }
 
 
-gboolean check_for_lock_file(gchar *set_name, gint type) {
+gboolean check_for_lock_file(const gchar *set_name, gint type) {
   // check for lock file
   int info_fd;
   gchar *msg=NULL;
@@ -762,7 +770,7 @@ gboolean check_for_lock_file(gchar *set_name, gint type) {
 }
 
 
-gboolean is_legal_set_name(gchar *set_name, gboolean allow_dupes) {
+gboolean is_legal_set_name(const gchar *set_name, gboolean allow_dupes) {
   gchar *msg;
   gchar *reject=" /\\*\"";
 
@@ -829,6 +837,7 @@ void get_frame_count(gint idx) {
   g_free(com);
   
   info_fd=open(info_file,O_RDONLY);
+
   if ((bytes=read(info_fd,mainw->msg,256))>0) {
     memset(mainw->msg+bytes,0,1);
     mainw->files[idx]->frames=atoi(mainw->msg);
@@ -909,12 +918,9 @@ gchar *get_extension(const gchar *filename) {
 
 
 
-gchar *ensure_extension(gchar *fname, gchar *ext) {
-  gchar *ret;
-  if (!strcmp(fname+strlen(fname)-strlen(ext),ext)) return fname;
-  ret=g_strconcat(fname,ext,NULL);
-  g_free(fname);
-  return ret;
+gchar *ensure_extension(const gchar *fname, const gchar *ext) {
+  if (!strcmp(fname+strlen(fname)-strlen(ext),ext)) return g_strdup(fname);
+  return g_strconcat(fname,ext,NULL);
 }
 
 
@@ -951,7 +957,7 @@ get_location(const gchar *exe, gchar *val, gint maxlen) {
 }
 
 
-gchar *repl_tmpdir(gchar *entry, gboolean fwd) {
+gchar *repl_tmpdir(const gchar *entry, gboolean fwd) {
   // replace prefs->tmpdir with string tmpdir or vice-versa. This allows us to relocate tmpdir if necessary.
   // used for layout.map file
   // return value should be g_free()'d
@@ -1603,13 +1609,13 @@ find_when_to_stop (void) {
   // a>v    stop on video end    stop on audio end           no stop
   // v>a    stop on video end    stop on video end           no stop
   // generator start - not playing : stop on vid_end, unless pure audio;
-  if (cfile->clip_type==CLIP_TYPE_GENERATOR) mainw->whentostop=STOP_ON_VID_END;
-  else if (mainw->multitrack!=NULL) mainw->whentostop=STOP_ON_VID_END;
+  if (cfile->clip_type==CLIP_TYPE_GENERATOR||mainw->aud_rec_fd!=-1) mainw->whentostop=STOP_ON_VID_END;
+  else if (mainw->multitrack!=NULL&&cfile->frames>0) mainw->whentostop=STOP_ON_VID_END;
   else if (cfile->clip_type!=CLIP_TYPE_DISK&&cfile->clip_type!=CLIP_TYPE_FILE) mainw->whentostop=NEVER_STOP;
   else if (cfile->opening_only_audio) mainw->whentostop=STOP_ON_AUD_END;
   else if (cfile->opening_audio) mainw->whentostop=STOP_ON_VID_END;
   else if (mainw->loop_cont&&!mainw->preview) mainw->whentostop=NEVER_STOP;
-  else if (mainw->loop&&cfile->achans>0&&!mainw->is_rendering&&(mainw->audio_end/cfile->fps)<MAX (cfile->laudio_time,cfile->raudio_time)) mainw->whentostop=STOP_ON_AUD_END;
+  else if (cfile->frames==0||(mainw->loop&&cfile->achans>0&&!mainw->is_rendering&&(mainw->audio_end/cfile->fps)<MAX (cfile->laudio_time,cfile->raudio_time))) mainw->whentostop=STOP_ON_AUD_END;
   else mainw->whentostop=STOP_ON_VID_END; // tada...
 }
 
@@ -1922,10 +1928,14 @@ prepare_to_play_foreign(void) {
     cfile->asampsize=mainw->rec_asamps;
     cfile->signed_endian=mainw->rec_signed_endian;
 #ifdef HAVE_PULSE_AUDIO
-    if (mainw->rec_achans>0&&prefs->audio_player==AUD_PLAYER_PULSE&&mainw->pulsed_read==NULL) pulse_rec_audio_to_clip(mainw->current_file,TRUE);
+    if (mainw->rec_achans>0&&prefs->audio_player==AUD_PLAYER_PULSE&&mainw->pulsed_read==NULL) {
+      lives_pulse_init(0);
+      pulse_audio_read_init();
+      pulse_rec_audio_to_clip(mainw->current_file,-1,RECA_WINDOW_GRAB);
+    }
 #endif
 #ifdef ENABLE_JACK
-    if (mainw->rec_achans>0&&prefs->audio_player==AUD_PLAYER_JACK&&mainw->jackd_read==NULL) jack_rec_audio_to_clip(mainw->current_file,TRUE);
+    if (mainw->rec_achans>0&&prefs->audio_player==AUD_PLAYER_JACK&&mainw->jackd_read==NULL) jack_rec_audio_to_clip(mainw->current_file,-1,RECA_WINDOW_GRAB);
 #endif
   }
 
@@ -1988,7 +1998,7 @@ after_foreign_play(void) {
   gchar *capfile=g_strdup_printf("%s/.capture.%d",prefs->tmpdir,getpid());
   gchar capbuf[256];
   ssize_t length;
-  gint new_file;
+  gint new_file=-1;
   gint new_frames=0;
   gint old_file=mainw->current_file;
 
@@ -2003,81 +2013,64 @@ after_foreign_play(void) {
       if (get_token_count (capbuf,'|')>2) {
 	array=g_strsplit(capbuf,"|",3);
 	new_frames=atoi(array[1]);
-	new_file=mainw->first_free_file;
-	mainw->current_file=new_file;
-	cfile=(file *)(g_malloc(sizeof(file)));
-	g_snprintf(cfile->handle,255,"%s",array[0]);
-	g_strfreev(array);
-	create_cfile();
-	g_snprintf(cfile->file_name,256,"Capture %d",mainw->cap_number);
-	g_snprintf(cfile->name,256,"Capture %d",mainw->cap_number++);
-	g_snprintf(cfile->type,40,"Frames");
-	cfile->progress_start=cfile->start=1;
-	cfile->progress_end=cfile->frames=cfile->end=new_frames;
-	cfile->pb_fps=cfile->fps=mainw->rec_fps;
 
-	if (mainw->rec_achans>0) {
-	  cfile->arate=cfile->arps=mainw->rec_arate;
-	  cfile->achans=mainw->rec_achans;
-	  cfile->asampsize=mainw->rec_asamps;
-	  cfile->signed_endian=mainw->rec_signed_endian;
-	}
-
-	g_snprintf(file_name,256,"%s/%s/",prefs->tmpdir,cfile->handle);
-	
-	com=g_strdup_printf("smogrify fill_and_redo_frames %s %d %d %d %d %s %.4f %d %d %d %d %d",cfile->handle,1,cfile->frames,mainw->foreign_width,mainw->foreign_height,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png",cfile->fps,cfile->arate,cfile->achans,cfile->asampsize,!(cfile->signed_endian&AFORM_UNSIGNED),!(cfile->signed_endian&AFORM_BIG_ENDIAN));
-	unlink(cfile->info_file);
-	dummyvar=system(com);
-	cfile->nopreview=TRUE;
-	if (do_progress_dialog(TRUE,TRUE,_ ("Cleaning up clip"))) {
-	  array=g_strsplit(mainw->msg,"|",5);
-	  new_frames=atoi(array[1]);
-	  cfile->hsize=atoi(array[2]);
-	  cfile->vsize=atoi(array[3]);
-	  cfile->bpp=cfile->img_type==IMG_TYPE_JPEG?24:32;
+	if (new_frames>0) {
+	  new_file=mainw->first_free_file;
+	  mainw->current_file=new_file;
+	  cfile=(file *)(g_malloc(sizeof(file)));
+	  g_snprintf(cfile->handle,255,"%s",array[0]);
 	  g_strfreev(array);
+	  create_cfile();
+	  g_snprintf(cfile->file_name,256,"Capture %d",mainw->cap_number);
+	  g_snprintf(cfile->name,256,"Capture %d",mainw->cap_number++);
+	  g_snprintf(cfile->type,40,"Frames");
+
+	  cfile->progress_start=cfile->start=1;
+	  cfile->progress_end=cfile->frames=cfile->end=new_frames;
+	  cfile->pb_fps=cfile->fps=mainw->rec_fps;
+
+	  cfile->hsize=mainw->foreign_width;
+	  cfile->vsize=mainw->foreign_height;
 	  
-	  if (new_frames>0) {
-	    cfile->start=1;
-	    cfile->end=cfile->frames=new_frames;
+	  if (mainw->rec_achans>0) {
+	    cfile->arate=cfile->arps=mainw->rec_arate;
+	    cfile->achans=mainw->rec_achans;
+	    cfile->asampsize=mainw->rec_asamps;
+	    cfile->signed_endian=mainw->rec_signed_endian;
+	  }
+	  
+	  g_snprintf(file_name,256,"%s/%s/",prefs->tmpdir,cfile->handle);
+	  
+	  com=g_strdup_printf("smogrify fill_and_redo_frames %s %d %d %d %s %.4f %d %d %d %d %d",cfile->handle,cfile->frames,mainw->foreign_width,mainw->foreign_height,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png",cfile->fps,cfile->arate,cfile->achans,cfile->asampsize,!(cfile->signed_endian&AFORM_UNSIGNED),!(cfile->signed_endian&AFORM_BIG_ENDIAN));
+	  unlink(cfile->info_file);
+	  dummyvar=system(com);
+	  cfile->nopreview=TRUE;
+	  if (do_progress_dialog(TRUE,TRUE,_ ("Cleaning up clip"))) {
 	    get_next_free_file();
 	  }
 	  else {
-	    // failed cleanup
-	    cfile->frames=cfile->start=cfile->end=new_frames=0;
+	    // cancelled cleanup
+	    new_frames=0;
+	    close_current_file(old_file);
 	  }
+	  g_free(com);
 	}
-	else {
-	  // cancelled cleanup
-	  cfile->frames=cfile->start=cfile->end=new_frames=0;
-	}
-	g_free(com);
+	else g_strfreev(array);
       }
+      close(capture_fd);
+      unlink(capfile);
     }
-    close(capture_fd);
-    unlink(capfile);
   }
-  else {
-    // nothing captured
-    g_free(capfile);
-    if (mainw->current_file>0) {
-      switch_to_file (old_file,old_file);
-      cfile->nopreview=FALSE;
-    }
-    else {
-      close_current_file(old_file);
-    }
-    return FALSE;
-  }
-  cfile->nopreview=FALSE;
-  g_free(capfile);
 
   if (new_frames==0) {
-    // failed to get any frames from this clip
-    close_current_file(old_file);
+    // nothing captured; or cancelled
+    g_free(capfile);
     return FALSE;
   }
 
+  cfile->nopreview=FALSE;
+  g_free(capfile);
+
   add_to_winmenu();
   if (mainw->multitrack==NULL) switch_to_file(old_file,mainw->current_file);
   
@@ -2217,7 +2210,7 @@ check_file(const gchar *file_name, gboolean check_existing) {
 
 
 gboolean 
-check_dir_access (gchar *dir) {
+check_dir_access (const gchar *dir) {
   // if a directory exists, make sure it is readable and writable
   // otherwise create it and then check
 
@@ -2294,7 +2287,7 @@ get_file_size(int fd) {
 }
 
 gulong
-sget_file_size(gchar *name) {
+sget_file_size(const gchar *name) {
   // get the size of file fd
   struct stat filestat;
   int fd;
@@ -2492,7 +2485,7 @@ add_to_recent(const gchar *filename, gdouble start, gint frames, const gchar *ex
 }
 
 
-void lives_set_cursor_style(gint cstyle, GdkWindow *window) {
+void lives_set_cursor_style(lives_cursor_t cstyle, GdkWindow *window) {
   if (mainw->cursor!=NULL) gdk_cursor_unref(mainw->cursor);
   mainw->cursor=NULL;
 
@@ -2510,6 +2503,8 @@ void lives_set_cursor_style(gint cstyle, GdkWindow *window) {
     mainw->cursor=gdk_cursor_new(GDK_WATCH);
     gdk_window_set_cursor (window, mainw->cursor);
     return;
+  default:
+    return;
   }
 }
 
@@ -2634,7 +2629,7 @@ LIVES_INLINE void g_list_free_strings(GList *slist) {
 }
 
 
-void cache_file_contents(gchar *filename) {
+void cache_file_contents(const gchar *filename) {
   FILE *hfile;
   gchar buff[65536];
 
@@ -2693,7 +2688,7 @@ gchar *get_val_from_cached_list(const gchar *key, size_t maxlen) {
 
 
 
-gboolean get_clip_value(int which, int what, void *retval, size_t maxlen) {
+gboolean get_clip_value(int which, lives_clip_details_t what, void *retval, size_t maxlen) {
   FILE *valfile;
   gchar *vfile;
   gchar *lives_header=NULL;
@@ -2802,9 +2797,14 @@ gboolean get_clip_value(int which, int what, void *retval, size_t maxlen) {
   case CLIP_DETAILS_FILENAME:
     key=g_strdup("filename");
     break;
+  case CLIP_DETAILS_INTERLACE:
+    key=g_strdup("interlace");
+    break;
   default:
     g_printerr("invalid detail %d requested from file %s",which,lives_header);
     if (lives_header!=NULL) g_free(lives_header);
+    if (mainw->debug)
+      assert(FALSE);
     return FALSE;
   }
 
@@ -2882,6 +2882,9 @@ gboolean get_clip_value(int which, int what, void *retval, size_t maxlen) {
     *(gint *)retval=atoi(val);
     if (retval==0) *(gint *)retval=mainw->files[which]->arate;
     break;
+  case CLIP_DETAILS_INTERLACE:
+    *(gint *)retval=atoi(val);
+    break;
   case CLIP_DETAILS_FPS:
     *(gdouble *)retval=strtod(val,NULL);
     if (*(gdouble *)retval==0.) *(gdouble *)retval=prefs->default_fps;
@@ -2919,7 +2922,7 @@ gboolean get_clip_value(int which, int what, void *retval, size_t maxlen) {
 
 
 
-void save_clip_value(int which, int what, void *val) {
+void save_clip_value(int which, lives_clip_details_t what, void *val) {
   gchar *lives_header;
   gchar *com;
   gchar *myval;
@@ -2954,12 +2957,7 @@ void save_clip_value(int which, int what, void *val) {
     break;
   case CLIP_DETAILS_UNIQUE_ID:
     key=g_strdup("unique_id");
-    if (capable->cpu_bits==32) {
-      myval=g_strdup_printf("%lld",*(gint64 *)val);
-    }
-    else {
-      myval=g_strdup_printf("%ld",*(gint64 *)val);
-    }
+    myval=g_strdup_printf("%"PRId64,*(gint64 *)val);
     break;
   case CLIP_DETAILS_ARATE:
     key=g_strdup("audio_rate");
@@ -2990,6 +2988,10 @@ void save_clip_value(int which, int what, void *val) {
     key=g_strdup("frames");
     myval=g_strdup_printf("%d",*(gint *)val);
     break;
+  case CLIP_DETAILS_INTERLACE:
+    key=g_strdup("interlace");
+    myval=g_strdup_printf("%d",*(gint *)val);
+    break;
   case CLIP_DETAILS_TITLE:
     key=g_strdup("title");
     myval=g_strdup(val);
@@ -3025,6 +3027,8 @@ void save_clip_value(int which, int what, void *val) {
   default:
     g_printerr("invalid detail %d set in file %s",which,lives_header);
     g_free(lives_header);
+    if (mainw->debug)
+      assert(FALSE);
     return;
   }
 
@@ -3123,7 +3127,7 @@ gboolean check_for_ratio_fps (gdouble fps) {
 }
 
 
-gdouble get_ratio_fps(gchar *string) {
+gdouble get_ratio_fps(const gchar *string) {
   // return a ratio (8dp) fps from a string with format num:denom
   gdouble fps;
   gchar *fps_string;
@@ -3153,22 +3157,24 @@ gchar *remove_trailing_zeroes(gdouble val) {
 }
 
 
-guint 
-get_signed_endian (int asigned, int endian) {
-  if (asigned==1) {
-    if (endian==1) {
+guint get_signed_endian (gboolean is_signed, gboolean little_endian) {
+  // asigned TRUE == signed, FALSE == unsigned
+
+
+  if (is_signed) {
+    if (little_endian) {
       return 0;
     }
-    if (endian==0) {
+    else {
       return AFORM_BIG_ENDIAN;
     }
   }
   else {
-    if (asigned==0) { 
-      if (endian==1) {
+    if (!is_signed) { 
+      if (little_endian) {
 	return AFORM_UNSIGNED;
       }
-      if (endian==0) {
+      else {
 	return AFORM_UNSIGNED|AFORM_BIG_ENDIAN;
       }
     }
@@ -3194,7 +3200,7 @@ get_token_count (const gchar *string, int delim) {
 
 
 
-gchar *subst (gchar *string, gchar *from, gchar *to) {
+gchar *subst (const gchar *string, const gchar *from, const gchar *to) {
   // return a string with all occurrences of from replaced with to
   // return value should be freed after use
   gchar *ret=g_strdup(string),*first;
@@ -3261,7 +3267,7 @@ gint hextodec (const gchar *string) {
   return tot;
 }
 
-gint get_hex_digit (gchar *c) {
+gint get_hex_digit (const gchar *c) {
   if (!strcmp (c,"a")||!strcmp (c,"A")) return 10;
   if (!strcmp (c,"b")||!strcmp (c,"B")) return 11;
   if (!strcmp (c,"c")||!strcmp (c,"C")) return 12;
@@ -3443,3 +3449,19 @@ gulong get_fs_free(const gchar *dir) {
   return bytes;
 }
 
+
+
+
+LIVES_INLINE gint get_interp_value(gshort quality) {
+  if (quality==PB_QUALITY_HIGH) return GDK_INTERP_HYPER;
+  else if (quality==PB_QUALITY_MED) return GDK_INTERP_BILINEAR;
+  return GDK_INTERP_NEAREST;
+}
+
+
+
+LIVES_INLINE GList *g_list_move_to_first(GList *list, GList *item) {
+  // move item to first in list
+  GList *xlist=g_list_remove_link(list,item); // item becomes standalone list
+  return g_list_concat(item,xlist); // concat rest of list after item
+}
